//
//  main.cpp
//  Entrepreneurship
//
//  Created by Huang Zhiyi on 23/1/2016.
//  Modified by Guo Naijia on 16/10/2020.
//  Copyright © 2016 Huang Zhiyi. All rights reserved.
//  Temporary shock on non-pecuniary utility for unincorporated and incorporated
//  This counterfactual provides 0-100% subsidy to elite college students, adjusted for admission rates, partial equilibrium


#include <cmath>
#include <iostream>
#include <fstream>
#include <vector>
#include <cstdio>
#include <algorithm>
#include <iomanip>
#include <Eigen/Dense>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/normal_distribution.hpp>
#include <bits/stdc++.h>

#include "parameters.h"
#include "normal.h"
#include "norm_gen.h"

using namespace std;

std::ofstream fout;

float correlationCoefficient(double X[], double Y[], int n)
{

    double sum_X = 0, sum_Y = 0, sum_XY = 0;
    double squareSum_X = 0, squareSum_Y = 0;

    for (int i = 0; i < n; i++)
    {
        // sum of elements of array X.
        sum_X = sum_X + X[i];

        // sum of elements of array Y.
        sum_Y = sum_Y + Y[i];

        // sum of X[i] * Y[i].
        sum_XY = sum_XY + X[i] * Y[i];

        // sum of square of array elements.
        squareSum_X = squareSum_X + X[i] * X[i];
        squareSum_Y = squareSum_Y + Y[i] * Y[i];
    }

    // use formula for calculating correlation coefficient.
    float corr = (float)(n * sum_XY - sum_X * sum_Y)
    / sqrt((n * squareSum_X - sum_X * sum_X)
           * (n * squareSum_Y - sum_Y * sum_Y));

    return corr;
}

float olsCoefficient(double X[], double Y[], int n)
{

    double sum_X = 0, sum_Y = 0, sum_XY = 0;
    double squareSum_X = 0, squareSum_Y = 0;

    for (int i = 0; i < n; i++)
    {
        // sum of elements of array X.
        sum_X = sum_X + X[i];

        // sum of elements of array Y.
        sum_Y = sum_Y + Y[i];

        // sum of X[i] * Y[i].
        sum_XY = sum_XY + X[i] * Y[i];

        // sum of square of array elements.
        squareSum_X = squareSum_X + X[i] * X[i];
        squareSum_Y = squareSum_Y + Y[i] * Y[i];
    }

    // use formula for calculating correlation coefficient.
    // float ols = (float)(sum_XY / squareSum_X);
    float ols = (float)(n * sum_XY - sum_X * sum_Y) / (n * squareSum_X - sum_X * sum_X);

    return ols;
}

double sum(vector<double> a)
{
    double s = 0;
    for (int i = 0; i < a.size(); i++)
    {
        s += a[i];
    }
    return s;
}

double mean(vector<double> a)
{
    return sum(a) / a.size();
}

double sqsum(vector<double> a)
{
    double s = 0;
    for (int i = 0; i < a.size(); i++)
    {
        s += pow(a[i], 2);
    }
    return s;
}

double stdev(vector<double> nums)
{
    double N = nums.size();
    return pow(sqsum(nums) / N - pow(sum(nums) / N, 2), 0.5);
}


vector<double> operator-(vector<double> a, double b)
{
    vector<double> retvect;
    for (int i = 0; i < a.size(); i++)
    {
        retvect.push_back(a[i] - b);
    }
    return retvect;
}

vector<double> operator*(vector<double> a, vector<double> b)
{
    vector<double> retvect;
    for (int i = 0; i < a.size() ; i++)
    {
        retvect.push_back(a[i] * b[i]);
    }
    return retvect;
}

double pearsoncoeff(vector<double> X, vector<double> Y)
{
    return sum((X - mean(X))*(Y - mean(Y))) / (X.size()*stdev(X)* stdev(Y));
}


namespace Eigen {
    namespace internal {
        template<typename Scalar>
        struct scalar_normal_dist_op
        {
            static boost::mt19937 rng;    // The uniform pseudo-random algorithm
            mutable boost::normal_distribution<Scalar> norm;  // The gaussian combinator

            EIGEN_EMPTY_STRUCT_CTOR(scalar_normal_dist_op)

            template<typename Index>
            inline const Scalar operator() (Index, Index = 0) const { return norm(rng); }
        };

        template<typename Scalar> boost::mt19937 scalar_normal_dist_op<Scalar>::rng;

        template<typename Scalar>
        struct functor_traits<scalar_normal_dist_op<Scalar> >
        { enum { Cost = 50 * NumTraits<Scalar>::MulCost, PacketAccess = false, IsRepeatable = false }; };
    } // end namespace internal
} // end namespace Eigen


int backward_induction(double abi_em_std,
                       double abi_ub_std,
                       double abi_ib_std,
                       double w_std,
                       double n_std,
                       double i_std,
                       double lambda_e,
                       double lambda_c,
                       double omega,
                       double theta_em,
                       double theta_ub,
                       double theta_ib,
                       double subsidy) {


    // ability grid
    double abi_grid_em[ABI];
    double abi_grid_ub[ABI];
    double abi_grid_ib[ABI];

    for (int i = 0; i < ABI; i++) {
        abi_grid_em[i] = -RANGE_ABI * ABI_EM_STD + 2 * RANGE_ABI * ABI_EM_STD * i / (ABI - 1);
        abi_grid_ub[i] = -RANGE_ABI * ABI_UB_STD + 2 * RANGE_ABI * ABI_UB_STD * i / (ABI - 1);
        abi_grid_ib[i] = -RANGE_ABI * ABI_IB_STD + 2 * RANGE_ABI * ABI_IB_STD * i / (ABI - 1);
        //std::cout << "abi_grid_em[" << i << "] = " << abi_grid_em[i] << std::endl;
        //std::cout << "abi_grid_ub[" << i << "] = " << abi_grid_ub[i] << std::endl;
        //std::cout << "abi_grid_ib[" << i << "] = " << abi_grid_ib[i] << std::endl;
    }

    double mid_abi_grid_em[ABI - 1];
    double mid_abi_grid_ub[ABI - 1];
    double mid_abi_grid_ib[ABI - 1];
    double prob_abi_em[ABI];
    double prob_abi_ub[ABI];
    double prob_abi_ib[ABI];

    for (int i = 0; i < ABI - 1; i++) {
        mid_abi_grid_em[i] = (abi_grid_em[i] + abi_grid_em[i + 1]) / 2;
        mid_abi_grid_ub[i] = (abi_grid_ub[i] + abi_grid_ub[i + 1]) / 2;
        mid_abi_grid_ib[i] = (abi_grid_ib[i] + abi_grid_ib[i + 1]) / 2;
        //std::cout << "mid_abi_grid_em[" << i << "] = " << mid_abi_grid_em[i] << std::endl;
        //std::cout << "mid_abi_grid_ub[" << i << "] = " << mid_abi_grid_ub[i] << std::endl;
        //std::cout << "mid_abi_grid_ib[" << i << "] = " << mid_abi_grid_ib[i] << std::endl;
    }

    prob_abi_em[0] = norm_cdf(mid_abi_grid_em[0], 0, ABI_EM_STD);
    for (int i = 1; i < ABI - 1; i++) {
        prob_abi_em[i] = norm_cdf(mid_abi_grid_em[i], 0, ABI_EM_STD) - norm_cdf(mid_abi_grid_em[i-1], 0, ABI_EM_STD);
    }
    prob_abi_em[ABI-1] = 1 - norm_cdf(mid_abi_grid_em[ABI-2], 0, ABI_EM_STD);

    prob_abi_ub[0] = norm_cdf(mid_abi_grid_ub[0], 0, ABI_UB_STD);
    for (int i = 1; i < ABI - 1; i++) {
        prob_abi_ub[i] = norm_cdf(mid_abi_grid_ub[i], 0, ABI_UB_STD) - norm_cdf(mid_abi_grid_ub[i-1], 0, ABI_UB_STD);
    }
    prob_abi_ub[ABI-1] = 1 - norm_cdf(mid_abi_grid_ub[ABI-2], 0, ABI_UB_STD);

    prob_abi_ib[0] = norm_cdf(mid_abi_grid_ib[0], 0, ABI_IB_STD);
    for (int i = 1; i < ABI - 1; i++) {
        prob_abi_ib[i] = norm_cdf(mid_abi_grid_ib[i], 0, ABI_IB_STD) - norm_cdf(mid_abi_grid_ib[i-1], 0, ABI_IB_STD);
    }
    prob_abi_ib[ABI-1] = 1 - norm_cdf(mid_abi_grid_ib[ABI-2], 0, ABI_IB_STD);

    //    for (int i = 0; i < ABI; i++){
    //        std::cout << "prob_abi_em[" << i << "] = " << prob_abi_em[i] << std::endl;
    //        std::cout << "prob_abi_ub[" << i << "] = " << prob_abi_ub[i] << std::endl;
    //        std::cout << "prob_abi_ib[" << i << "] = " << prob_abi_ib[i] << std::endl;
    //    }

    // shock grid
    // wage and investment
    // consumption value of non-incorporated and incorporated
    // consumption value of elite and non-elite college
    double shock_grid_w[SHOCK];
    double shock_grid_n[SHOCK];
    double shock_grid_i[SHOCK];
    double consump_grid_n[SHOCK];
    double consump_grid_i[SHOCK];
    double consump_grid_c[SHOCK];
    double consump_grid_e[SHOCK];

    for (int i = 0; i < SHOCK; i++) {
        shock_grid_w[i] = -RANGE_SHOCK * W_STD + 2 * RANGE_SHOCK * W_STD * i / (SHOCK - 1);
        shock_grid_n[i] = -RANGE_SHOCK * N_STD + 2 * RANGE_SHOCK * N_STD * i / (SHOCK - 1);
        shock_grid_i[i] = -RANGE_SHOCK * I_STD + 2 * RANGE_SHOCK * I_STD * i / (SHOCK - 1);
        consump_grid_n[i] = -RANGE_SHOCK * CONSUMP_N_STD + 2 * RANGE_SHOCK * CONSUMP_N_STD * i / (SHOCK - 1);
        consump_grid_i[i] = -RANGE_SHOCK * CONSUMP_I_STD + 2 * RANGE_SHOCK * CONSUMP_I_STD * i / (SHOCK - 1);
        consump_grid_c[i] = -RANGE_SHOCK * CONSUMP_C_STD + 2 * RANGE_SHOCK * CONSUMP_C_STD * i / (SHOCK - 1);
        consump_grid_e[i] = -RANGE_SHOCK * CONSUMP_E_STD + 2 * RANGE_SHOCK * CONSUMP_E_STD * i / (SHOCK - 1);
        //std::cout << "consump_grid_n[" << i << "] = " << consump_grid_n[i] << std::endl;
        //std::cout << "shock_grid_i[" << i << "] = " << shock_grid_i[i] << std::endl;
    }

    double mid_shock_grid_w[SHOCK - 1];
    double mid_shock_grid_n[SHOCK - 1];
    double mid_shock_grid_i[SHOCK - 1];
    double mid_consump_grid_n[SHOCK - 1];
    double mid_consump_grid_i[SHOCK - 1];
    double mid_consump_grid_c[SHOCK - 1];
    double mid_consump_grid_e[SHOCK - 1];
    double prob_shock_w[SHOCK];
    double prob_shock_n[SHOCK];
    double prob_shock_i[SHOCK];
    double prob_consump_n[SHOCK];
    double prob_consump_i[SHOCK];
    double prob_consump_c[SHOCK];
    double prob_consump_e[SHOCK];

    for (int i = 0; i < SHOCK - 1; i++) {
        mid_shock_grid_w[i] = (shock_grid_w[i] + shock_grid_w[i + 1]) / 2;
        mid_shock_grid_n[i] = (shock_grid_n[i] + shock_grid_n[i + 1]) / 2;
        mid_shock_grid_i[i] = (shock_grid_i[i] + shock_grid_i[i + 1]) / 2;
        mid_consump_grid_n[i] = (consump_grid_n[i] + consump_grid_n[i + 1]) / 2;
        mid_consump_grid_i[i] = (consump_grid_i[i] + consump_grid_i[i + 1]) / 2;
        mid_consump_grid_c[i] = (consump_grid_n[i] + consump_grid_n[i + 1]) / 2;
        mid_consump_grid_e[i] = (consump_grid_i[i] + consump_grid_i[i + 1]) / 2;
        //std::cout << "mid_shock_grid_w[" << i << "] = " << mid_shock_grid_w[i] << std::endl;
        //std::cout << "mid_shock_grid_i[" << i << "] = " << mid_shock_grid_i[i] << std::endl;
    }

    prob_shock_w[0] = norm_cdf(mid_shock_grid_w[0], 0, W_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_shock_w[i] = norm_cdf(mid_shock_grid_w[i], 0, W_STD) - norm_cdf(mid_shock_grid_w[i-1], 0, W_STD);
    }
    prob_shock_w[SHOCK-1] = 1 - norm_cdf(mid_shock_grid_w[SHOCK-2], 0, W_STD);

    prob_shock_n[0] = norm_cdf(mid_shock_grid_n[0], 0, N_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_shock_n[i] = norm_cdf(mid_shock_grid_n[i], 0, N_STD) - norm_cdf(mid_shock_grid_n[i-1], 0, N_STD);
    }
    prob_shock_n[SHOCK-1] = 1 - norm_cdf(mid_shock_grid_n[SHOCK-2], 0, N_STD);

    prob_shock_i[0] = norm_cdf(mid_shock_grid_i[0], 0, I_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_shock_i[i] = norm_cdf(mid_shock_grid_i[i], 0, I_STD) - norm_cdf(mid_shock_grid_i[i-1], 0, I_STD);
    }
    prob_shock_i[SHOCK-1] = 1 - norm_cdf(mid_shock_grid_i[SHOCK-2], 0, I_STD);

    prob_consump_n[0] = norm_cdf(mid_consump_grid_n[0], 0, CONSUMP_N_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_consump_n[i] = norm_cdf(mid_consump_grid_n[i], 0, CONSUMP_N_STD) - norm_cdf(mid_consump_grid_n[i-1], 0, CONSUMP_N_STD);
    }
    prob_consump_n[SHOCK-1] = 1 - norm_cdf(mid_consump_grid_n[SHOCK-2], 0, CONSUMP_N_STD);

    prob_consump_i[0] = norm_cdf(mid_consump_grid_i[0], 0, CONSUMP_I_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_consump_i[i] = norm_cdf(mid_consump_grid_i[i], 0, CONSUMP_I_STD) - norm_cdf(mid_consump_grid_i[i-1], 0, CONSUMP_I_STD);
    }
    prob_consump_i[SHOCK-1] = 1 - norm_cdf(mid_consump_grid_i[SHOCK-2], 0, CONSUMP_I_STD);

    prob_consump_c[0] = norm_cdf(mid_consump_grid_c[0], 0, CONSUMP_C_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_consump_c[i] = norm_cdf(mid_consump_grid_c[i], 0, CONSUMP_C_STD) - norm_cdf(mid_consump_grid_c[i-1], 0, CONSUMP_C_STD);
    }
    prob_consump_c[SHOCK-1] = 1 - norm_cdf(mid_consump_grid_c[SHOCK-2], 0, CONSUMP_C_STD);

    prob_consump_e[0] = norm_cdf(mid_consump_grid_e[0], 0, CONSUMP_E_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_consump_e[i] = norm_cdf(mid_consump_grid_e[i], 0, CONSUMP_E_STD) - norm_cdf(mid_consump_grid_e[i-1], 0, CONSUMP_E_STD);
    }
    prob_consump_e[SHOCK-1] = 1 - norm_cdf(mid_consump_grid_e[SHOCK-2], 0, CONSUMP_E_STD);

    //for (int i = 0; i < SHOCK; i++){
    //std::cout << "prob_shock_n[" << i << "] = " << prob_shock_w[i] << std::endl;
    //std::cout << "prob_shock_i[" << i << "] = " << prob_shock_i[i] << std::endl;
    //}

    // ability intergenerational transition probability
    double tranprob_abi_em[ABI][ABI];
    double tranprob_abi_ub[ABI][ABI];
    double tranprob_abi_ib[ABI][ABI];
    for (int i = 0; i < ABI; i++){
        tranprob_abi_em[i][0] = norm_cdf(mid_abi_grid_em[0] - THETA_EM * mid_abi_grid_em[i], 0, ABI_EM_STD);
        tranprob_abi_em[i][ABI-1] = 1 - norm_cdf(mid_abi_grid_em[ABI-2] - THETA_EM * mid_abi_grid_em[i], 0, ABI_EM_STD);
        tranprob_abi_ub[i][0] = norm_cdf(mid_abi_grid_ub[0] - THETA_UB * mid_abi_grid_ub[i], 0, ABI_UB_STD);
        tranprob_abi_ub[i][ABI-1] = 1 - norm_cdf(mid_abi_grid_ub[ABI-2] - THETA_UB * mid_abi_grid_ub[i], 0, ABI_UB_STD);
        tranprob_abi_ib[i][0] = norm_cdf(mid_abi_grid_ib[0] - THETA_IB * mid_abi_grid_ib[i], 0, ABI_IB_STD);
        tranprob_abi_ib[i][ABI-1] = 1 - norm_cdf(mid_abi_grid_ib[ABI-2] - THETA_IB * mid_abi_grid_ib[i], 0, ABI_IB_STD);
        for (int j = 1; j < ABI - 1; j++) {
            tranprob_abi_em[i][j] = norm_cdf(mid_abi_grid_em[j] - THETA_EM * mid_abi_grid_em[i], 0, ABI_EM_STD) - norm_cdf(mid_abi_grid_em[j - 1] - THETA_EM * mid_abi_grid_em[i], 0, ABI_EM_STD);
            tranprob_abi_ub[i][j] = norm_cdf(mid_abi_grid_ub[j] - THETA_UB * mid_abi_grid_ub[i], 0, ABI_UB_STD) - norm_cdf(mid_abi_grid_ub[j - 1] - THETA_UB * mid_abi_grid_ub[i], 0, ABI_UB_STD);
            tranprob_abi_ib[i][j] = norm_cdf(mid_abi_grid_ib[j] - THETA_IB * mid_abi_grid_ib[i], 0, ABI_IB_STD) - norm_cdf(mid_abi_grid_ib[j - 1] - THETA_IB * mid_abi_grid_ib[i], 0, ABI_IB_STD);
        }
    }


    //    for (int i = 0; i < ABI; i++){
    //        for (int j = 0; j < ABI; j++){
    //            std::cout << "tranprob_abi_w[" << i << "][" << j << "] = " << tranprob_abi_w[i][j] << std::endl;
    //            std::cout << "tranprob_abi_e[" << i << "][" << j << "] = " << tranprob_abi_e[i][j] << std::endl;
    //        }
    //    }

    // value function
    vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<double> > > > > > > > > > value_retire(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > > > career_retire(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > > > > capitalchoice_retire(ITER);

    vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<double> > > > > > > > > > value(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > > > career(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > > > > capitalchoice(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<double> > > > > > > > value_worker_noshock(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<double> > > > > > > > value_non_incorp_noshock(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<double> > > > > > > > value_incorp_noshock(ITER);

    vector<vector<vector<vector<vector<vector<vector<vector<double> > > > > > > > schoolvalue(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > schoolchoice(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > schoolcapital(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > schoolchoice2(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > schoolcapital2(ITER);
    vector<vector<vector<vector<vector<vector<double> > > > > > value_non_elite_noshock(ITER);
    vector<vector<vector<vector<vector<vector<double> > > > > > value_elite_noshock(ITER);

    // simulation array
    int abi_em_sim[num_id];
    int abi_ub_sim[num_id];
    int abi_ib_sim[num_id];
    double abi_em_org_sim[num_id];
    double abi_ub_org_sim[num_id];
    double abi_ib_org_sim[num_id];
    int edu_sim[num_id];
    int k_idx_sim[num_id][AGE + 1];
    int k_fam_idx_sim[num_id];
    int career_sim[num_id][AGE];
    double income_sim[num_id][AGE];
    int exp_bus_sim[num_id][AGE];
    int shock_w_sim[num_id][AGE];
    int shock_n_sim[num_id][AGE];
    int shock_i_sim[num_id][AGE];
    int shock_s_sim[num_id][AGE];
    int consump_n_sim[num_id][AGE];
    int consump_i_sim[num_id][AGE];
    int consump_c_sim[num_id];
    int consump_e_sim[num_id];


    // iteration on W and R
    double W2 = W_ORIGINAL;
    double R2 = R_ORIGINAL;
    double TAX2 = TAX_ORIGINAL;
    double W = 0;
    double R = 0;
    double TAX = 1;
    while ( abs(TAX2 - TAX) > 0.0001) {

        W = W2;
        R = R2;
        TAX = TAX2;
        for(int iter = 0; iter < ITER; iter++) {
            std::cout << "Iteration : " << iter << std::endl;

            value_retire[iter].resize(AGE-RETIRE_AGE);
            career_retire[iter].resize(AGE-RETIRE_AGE);
            capitalchoice_retire[iter].resize(AGE-RETIRE_AGE);

            std::cout << "Processing Retirement age" << std::endl;
            for(int t = AGE-1; t >= RETIRE_AGE; t--) {
                value_retire[iter][t-RETIRE_AGE].resize(EDU);
                career_retire[iter][t-RETIRE_AGE].resize(EDU);
                capitalchoice_retire[iter][t-RETIRE_AGE].resize(EDU);
                for(int e = 0; e < EDU; e++) {
                    value_retire[iter][t-RETIRE_AGE][e].resize(ABI);
                    career_retire[iter][t-RETIRE_AGE][e].resize(ABI);
                    capitalchoice_retire[iter][t-RETIRE_AGE][e].resize(ABI);
                    for(int a_em = 0; a_em < ABI; a_em++) {
                        value_retire[iter][t-RETIRE_AGE][e][a_em].resize(ABI);
                        career_retire[iter][t-RETIRE_AGE][e][a_em].resize(ABI);
                        capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em].resize(ABI);
                        for(int a_ub = 0; a_ub < ABI; a_ub++) {
                            value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub].resize(ABI);
                            career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub].resize(ABI);
                            capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub].resize(ABI);
                            for(int a_ib = 0; a_ib < ABI; a_ib++) {
                                value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE_RETIRE);
                                career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE_RETIRE);
                                capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE_RETIRE);
                                for(int exp_bus = 0; exp_bus < EXP_BUS_TYPE_RETIRE; exp_bus++) {
                                    value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                    career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                    capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                    for(int k = 0; k < CAP; k++) {
                                        value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k].resize(SHOCK);
                                        career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k].resize(SHOCK);
                                        capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k].resize(SHOCK);
                                        for(int consump_n = 0; consump_n < SHOCK; consump_n++) {
                                            value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n].resize(SHOCK);
                                            career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n].resize(SHOCK);
                                            capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n].resize(SHOCK);
                                            for(int consump_i = 0; consump_i < SHOCK; consump_i++) {
                                                capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i].resize(SHOCK);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            for(int t = AGE-1; t >= RETIRE_AGE; t--) {
                std::cout << "Processing age = " << age(t) << std::endl;
#pragma omp parallel for collapse(4)
                for(int e = 0; e < EDU; e++) {
                    for(int a_em = 0; a_em < ABI; a_em++) {
                        for(int a_ub = 0; a_ub < ABI; a_ub++) {
                            for(int a_ib = 0; a_ib < ABI; a_ib++) {
                                for(int exp_bus = 0; exp_bus < EXP_BUS_TYPE_RETIRE; exp_bus++) {
                                    vector<double> futurevalue_retire0(CAP, 0);
                                    vector<double> futurevalue_retire1(CAP, 0);
                                    vector<double> futurevalue_retire2(CAP, 0);
                                    for (int k_next_idx = 0; k_next_idx < CAP; k_next_idx++){
                                        if (t < AGE - 1){
                                            for(int consump_n_next = 0; consump_n_next < SHOCK; consump_n_next++) {
                                                for(int consump_i_next = 0; consump_i_next < SHOCK; consump_i_next++) {
                                                    futurevalue_retire0[k_next_idx] += BETA * ZETA[t-RETIRE_AGE] * value_retire[iter][t-RETIRE_AGE+1][e][a_em][a_ub][a_ib][0][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                                    futurevalue_retire1[k_next_idx] += BETA * ZETA[t-RETIRE_AGE] * value_retire[iter][t-RETIRE_AGE+1][e][a_em][a_ub][a_ib][1][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                                    futurevalue_retire2[k_next_idx] += BETA * ZETA[t-RETIRE_AGE] * value_retire[iter][t-RETIRE_AGE+1][e][a_em][a_ub][a_ib][2][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                                }
                                            }
                                        }
                                    }
                                    for(int k = 0; k < CAP; k++) {
                                        // worker's decision
                                        double value_worker = -9999;
                                        double capitalchoice_worker = -1;
                                        double k_worker = capital(k) * (1 + R) + P * W * h_em(a_em, e) * exp(4 * ALPHA_1 + 22.7 * ALPHA_2);
                                        if (t == AGE - 1){
                                            value_worker = util(k_worker);
                                            capitalchoice_worker = 4; // capital = 0
                                        } else {
                                            for(int k_next = 0; k_next <= k_worker && k_next < capital(CAP); k_next += CAP_STEP) {
                                                double value_tmp = util(k_worker - k_next) + futurevalue_retire0[(k_next - CAP_MIN)/CAP_STEP];
                                                if (value_worker < value_tmp) {
                                                    value_worker = value_tmp;
                                                    capitalchoice_worker = (k_next - CAP_MIN)/CAP_STEP;
                                                }
                                            }
                                        }


                                        // non-incorporated business owner decision
                                        double value_non_incorp = 0;
                                        double value_non_incorp_tmp;
                                        vector<double> capitalchoice_non_incorp(SHOCK);
                                        // only able to run a business when there is no debt
                                        if (capital(k) >= 0){
                                            for(int shock_n = 0; shock_n < SHOCK; shock_n++) {
                                                value_non_incorp_tmp = -9999;
                                                capitalchoice_non_incorp[shock_n] = -1;
                                                double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                                                double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                                                double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                                                // liquidity constraint
                                                if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                                                    invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                                                double k_non_incorp = (1 - DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) + (capital(k) - invest_non_incorp) * (1 + R);
                                                if (t == AGE - 1){
                                                    value_non_incorp_tmp = util(k_non_incorp);
                                                    capitalchoice_non_incorp[shock_n] = 4;
                                                } else {
                                                    for(int k_next = 0; k_next <= k_non_incorp && k_next < capital(CAP); k_next += CAP_STEP) {
                                                        double value_tmp = util(k_non_incorp - k_next) + futurevalue_retire1[(k_next - CAP_MIN)/CAP_STEP];
                                                        if (value_non_incorp_tmp < value_tmp) {
                                                            value_non_incorp_tmp = value_tmp;
                                                            capitalchoice_non_incorp[shock_n] = (k_next - CAP_MIN)/CAP_STEP;
                                                        }
                                                    }
                                                }
                                                value_non_incorp += value_non_incorp_tmp * prob_shock_n[shock_n];
                                            }
                                        } else {
                                            value_non_incorp = -999;
                                        }

                                        // incorporated business owner decision
                                        double value_incorp = 0;
                                        double value_incorp_tmp;
                                        vector<double> capitalchoice_incorp(SHOCK);
                                        // only able to run a business when there is no debt
                                        if (capital(k) >= 0){
                                            for(int shock_i = 0; shock_i < SHOCK; shock_i++) {
                                                value_incorp_tmp = -9999;
                                                capitalchoice_incorp[shock_i] = -1;
                                                double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                                                double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                                                double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                                                // liquidity constraint
                                                if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                                                    invest_incorp = (1 + LAMBDA_E) * capital(k);
                                                double k_incorp = (1 - DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - I_COST + (capital(k) - invest_incorp) * (1 + R);
                                                if (exp_bus == 2) {
                                                    k_incorp = (1 - DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) + (capital(k) - invest_incorp) * (1 + R);
                                                }
                                                if (t == AGE - 1){
                                                    value_incorp_tmp = util(k_incorp);
                                                    capitalchoice_incorp[shock_i] = 4;
                                                } else {
                                                    for(int k_next = 0; k_next <= k_incorp && k_next < capital(CAP); k_next += CAP_STEP) {
                                                        double value_tmp = util(k_incorp - k_next) + futurevalue_retire2[(k_next - CAP_MIN)/CAP_STEP];
                                                        if (value_incorp_tmp < value_tmp) {
                                                            value_incorp_tmp = value_tmp;
                                                            capitalchoice_incorp[shock_i] = (k_next - CAP_MIN)/CAP_STEP;
                                                        }
                                                    }
                                                }
                                                value_incorp += value_incorp_tmp * prob_shock_i[shock_i];
                                            }
                                        } else {
                                            value_incorp = -999;
                                        }

                                        for(int consump_n = 0; consump_n < SHOCK; consump_n++){
                                            for (int consump_i = 0; consump_i < SHOCK; consump_i++) {
                                                if (exp_bus > 0){
                                                    if (value_worker > value_non_incorp + consump_grid_n[consump_n] && value_worker > value_incorp + consump_grid_i[consump_i]) {
                                                        value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_worker;
                                                        career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = WORKER;
                                                        for (int shock_w = 0; shock_w < SHOCK; shock_w++) {
                                                            capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w] = capitalchoice_worker;
                                                        }
                                                    }
                                                    else if (value_non_incorp + consump_grid_n[consump_n] >= value_incorp + consump_grid_i[consump_i]) {
                                                        value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_non_incorp + consump_grid_n[consump_n];
                                                        career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = NON_INCORP;
                                                        for (int shock_n = 0; shock_n < SHOCK; shock_n++) {
                                                            capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n] = capitalchoice_non_incorp[shock_n];
                                                        }
                                                    }
                                                    else {
                                                        value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_incorp + consump_grid_i[consump_i];
                                                        career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = INCORP;
                                                        for (int shock_i = 0; shock_i < SHOCK; shock_i++) {
                                                            capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i] = capitalchoice_incorp[shock_i];
                                                        }
                                                    }
                                                }
                                                else{
                                                    value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_worker;
                                                    career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = WORKER;
                                                    for (int shock_w = 0; shock_w < SHOCK; shock_w++) {
                                                        capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w] = capitalchoice_worker;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            /*fout << "high school value function at age 100" << std::endl;
             for(int k = 0; k < 20; k++) {
             fout << "valuefunction_retire[" << 0 << "][" << 7 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << value_retire[0][7][0][3][3][0][k][4][4] << std::endl;
             fout << "capitalchoice_retire[" << 0 << "][" << 7 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << capitalchoice_retire[0][7][0][3][3][0][k][4][4][4] << std::endl;
             fout << "careerchoice_retire[" << 0 << "][" << 7 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << career_retire[0][7][0][3][3][0][k][4][4] << std::endl;
             }

             fout << "college value function at age 100" << std::endl;
             for(int k = 0; k < 20; k++) {
             fout << "valuefunction_retire[" << 0 << "][" << 7 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << value_retire[0][7][1][3][3][0][k][4][4] << std::endl;
             fout << "capitalchoice_retire[" << 0 << "][" << 7 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << capitalchoice_retire[0][7][1][3][3][0][k][4][4][4] << std::endl;
             fout << "careerchoice_retire[" << 0 << "][" << 7 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << career_retire[0][7][1][3][3][0][k][4][4] << std::endl;
             }

             fout << "elite college value function at age 100" << std::endl;
             for(int k = 0; k < 20; k++) {
             fout << "valuefunction[" << 0 << "][" << 7 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << value_retire[0][7][2][3][3][0][k][4][4] << std::endl;
             fout << "capitalchoice_retire[" << 0 << "][" << 7 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << capitalchoice_retire[0][7][2][3][3][0][k][4][4][4] << std::endl;
             fout << "careerchoice_retire[" << 0 << "][" << 7 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << career_retire[0][7][2][3][3][0][k][4][4] << std::endl;
             }

             fout << "high school value function and policy function at age 65" << std::endl;
             for(int k = 0; k < 30; k++) {
             fout << "valuefunction_retire[" << 0 << "][" << 0 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << value_retire[0][0][0][3][3][0][k][4][4] << std::endl;
             fout << "capitalchoice_retire[" << 0 << "][" << 0 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << capitalchoice_retire[0][0][0][3][3][0][k][4][4][4] << std::endl;
             fout << "careerchoice_retire[" << 0 << "][" << 0 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << career_retire[0][0][0][3][3][0][k][4][4] << std::endl;
             }

             fout << "college value function and policy function at age 65" << std::endl;
             for(int k = 0; k < 30; k++) {
             fout << "valuefunction_retire[" << 0 << "][" << 0 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << value_retire[0][0][1][3][3][0][k][4][4] << std::endl;
             fout << "capitalchoice_retire[" << 0 << "][" << 0 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << capitalchoice_retire[0][0][1][3][3][0][k][4][4][4] << std::endl;
             fout << "careerchoice_retire[" << 0 << "][" << 0 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << career_retire[0][0][1][3][3][0][k][4][4] << std::endl;
             }

             fout << "elite college value function and policy function at age 65" << std::endl;
             for(int k = 0; k < 30; k++) {
             fout << "valuefunction_retire[" << 0 << "][" << 0 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << value_retire[0][0][2][3][3][0][k][4][4] << std::endl;
             fout << "capitalchoice_retire[" << 0 << "][" << 0 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << capitalchoice_retire[0][0][2][3][3][0][k][4][4][4] << std::endl;
             fout << "careerchoice_retire[" << 0 << "][" << 0 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << career_retire[0][0][2][3][3][0][k][4][4] << std::endl;
             }*/


            std::cout << "Processing Working age" << std::endl;
            value[iter].resize(RETIRE_AGE);
            career[iter].resize(RETIRE_AGE);
            capitalchoice[iter].resize(RETIRE_AGE);
            value_worker_noshock[iter].resize(RETIRE_AGE);
            value_non_incorp_noshock[iter].resize(RETIRE_AGE);
            value_incorp_noshock[iter].resize(RETIRE_AGE);
            for(int t = RETIRE_AGE-1; t >= 0; t--) {
                value[iter][t].resize(EDU);
                career[iter][t].resize(EDU);
                capitalchoice[iter][t].resize(EDU);
                value_worker_noshock[iter][t].resize(EDU);
                value_non_incorp_noshock[iter][t].resize(EDU);
                value_incorp_noshock[iter][t].resize(EDU);
                for(int e = 0; e < EDU; e++) {
                    value[iter][t][e].resize(ABI);
                    career[iter][t][e].resize(ABI);
                    capitalchoice[iter][t][e].resize(ABI);
                    value_worker_noshock[iter][t][e].resize(ABI);
                    value_non_incorp_noshock[iter][t][e].resize(ABI);
                    value_incorp_noshock[iter][t][e].resize(ABI);
                    for(int a_em = 0; a_em < ABI; a_em++) {
                        value[iter][t][e][a_em].resize(ABI);
                        career[iter][t][e][a_em].resize(ABI);
                        capitalchoice[iter][t][e][a_em].resize(ABI);
                        value_worker_noshock[iter][t][e][a_em].resize(ABI);
                        value_non_incorp_noshock[iter][t][e][a_em].resize(ABI);
                        value_incorp_noshock[iter][t][e][a_em].resize(ABI);
                        for(int a_ub = 0; a_ub < ABI; a_ub++) {
                            value[iter][t][e][a_em][a_ub].resize(ABI);
                            career[iter][t][e][a_em][a_ub].resize(ABI);
                            capitalchoice[iter][t][e][a_em][a_ub].resize(ABI);
                            value_worker_noshock[iter][t][e][a_em][a_ub].resize(ABI);
                            value_non_incorp_noshock[iter][t][e][a_em][a_ub].resize(ABI);
                            value_incorp_noshock[iter][t][e][a_em][a_ub].resize(ABI);
                            for(int a_ib = 0; a_ib < ABI; a_ib++) {
                                value[iter][t][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE);
                                career[iter][t][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE);
                                capitalchoice[iter][t][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE);
                                value_worker_noshock[iter][t][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE);
                                value_non_incorp_noshock[iter][t][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE);
                                value_incorp_noshock[iter][t][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE);
                                for(int exp_bus = 0; exp_bus < EXP_BUS_TYPE; exp_bus++) {
                                    value[iter][t][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                    career[iter][t][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                    capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                    value_worker_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                    value_non_incorp_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                    value_incorp_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                    for(int k = 0; k < CAP; k++) {
                                        value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k].resize(SHOCK);
                                        career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k].resize(SHOCK);
                                        capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k].resize(SHOCK);
                                        for(int consump_n = 0; consump_n < SHOCK; consump_n++) {
                                            value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n].resize(SHOCK);
                                            career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n].resize(SHOCK);
                                            capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n].resize(SHOCK);
                                            for(int consump_i = 0; consump_i < SHOCK; consump_i++) {
                                                capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i].resize(SHOCK);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            for(int t = RETIRE_AGE-1; t >= 0; t--) {
                std::cout << "Processing age = " << age(t) << std::endl;
#pragma omp parallel for collapse(4)
                for(int e = 0; e < EDU; e++) {
                    for(int a_em = 0; a_em < ABI; a_em++) {
                        for(int a_ub = 0; a_ub < ABI; a_ub++) {
                            for(int a_ib = 0; a_ib < ABI; a_ib++) {
                                for(int exp_bus = 0; exp_bus < EXP_BUS_TYPE; exp_bus++) {
                                    vector<double> futurevalue0(CAP, 0);
                                    vector<double> futurevalue1(CAP, 0);
                                    vector<double> futurevalue_retire0(CAP, 0);
                                    vector<double> futurevalue_retire1(CAP, 0);
                                    vector<double> futurevalue_retire2(CAP, 0);
                                    for (int k_next_idx = 0; k_next_idx < CAP; k_next_idx++){
                                        if (t == RETIRE_AGE - 1){
                                            for(int consump_n_next = 0; consump_n_next < SHOCK; consump_n_next++) {
                                                for(int consump_i_next = 0; consump_i_next < SHOCK; consump_i_next++) {
                                                    futurevalue_retire0[k_next_idx] += BETA * value_retire[iter][0][e][a_em][a_ub][a_ib][0][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                                    futurevalue_retire1[k_next_idx] += BETA * value_retire[iter][0][e][a_em][a_ub][a_ib][1][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                                    futurevalue_retire2[k_next_idx] += BETA * value_retire[iter][0][e][a_em][a_ub][a_ib][2][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                                }
                                            }
                                        } else {
                                            for(int consump_n_next = 0; consump_n_next < SHOCK; consump_n_next++) {
                                                for(int consump_i_next = 0; consump_i_next < SHOCK; consump_i_next++) {
                                                    futurevalue0[k_next_idx] += BETA * value[iter][t+1][e][a_em][a_ub][a_ib][0][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                                    futurevalue1[k_next_idx] += BETA * value[iter][t+1][e][a_em][a_ub][a_ib][1][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                                }
                                            }
                                        }
                                    }

                                    for(int k = 0; k < CAP; k++) {
                                        // worker's decision
                                        double value_worker = 0;
                                        double value_worker_tmp;
                                        vector<double> capitalchoice_worker(SHOCK);
                                        for(int shock_w = 0; shock_w < SHOCK; shock_w++) {
                                            value_worker_tmp = -9999;
                                            capitalchoice_worker[shock_w] = -1;
                                            double k_worker = capital(k) * (1 + R) + W * h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e) + shock_grid_w[shock_w]);
                                            for(int k_next = 0; k_next <= k_worker && k_next < capital(CAP); k_next += CAP_STEP) {
                                                double value_tmp = util(k_worker - k_next);
                                                if (t == RETIRE_AGE - 1) {
                                                    value_tmp += futurevalue_retire0[(k_next - CAP_MIN)/CAP_STEP];
                                                }
                                                else {
                                                    value_tmp += futurevalue0[(k_next - CAP_MIN)/CAP_STEP];
                                                }
                                                if (value_worker_tmp < value_tmp) {
                                                    value_worker_tmp = value_tmp;
                                                    capitalchoice_worker[shock_w] = (k_next - CAP_MIN)/CAP_STEP;
                                                }
                                            }
                                            value_worker += value_worker_tmp * prob_shock_w[shock_w];
                                            //if (t == 4 && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 14 ){
                                            //    std::cout << t << ", " << k << ", " << k_worker << ", " << h_w(a_em, e) * exp(ALPHA_0 + ALPHA_1 * exp_w(t, e) + ALPHA_2 * exp_w(t, e) * exp_w(t, e) + shock_grid_w[shock_w]) << ", " << capital(capitalchoice_worker[shock_w]) << ", " << util(k_worker - capital(capitalchoice_worker[shock_w])) << ", " << futurevalue0[capitalchoice_worker[shock_w]] << ", " << value_worker_tmp << ", " << value_worker << std::endl;
                                            //}

                                        }

                                        // non-incorporated business owner decision
                                        double value_non_incorp = 0;
                                        double value_non_incorp_tmp;
                                        vector<double> capitalchoice_non_incorp(SHOCK);
                                        // only able to run a business when there is no debt
                                        if (capital(k) >= 0){
                                            for(int shock_n = 0; shock_n < SHOCK; shock_n++) {
                                                value_non_incorp_tmp = -9999;
                                                capitalchoice_non_incorp[shock_n] = -1;
                                                double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                                                double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                                                double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                                                // liquidity constraint
                                                if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                                                    invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                                                double k_non_incorp = (1 - DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) + (capital(k) - invest_non_incorp) * (1 + R);
                                                for(int k_next = 0; k_next <= k_non_incorp && k_next < capital(CAP); k_next += CAP_STEP) {
                                                    double value_tmp = util(k_non_incorp - k_next);
                                                    if (t == RETIRE_AGE - 1) {
                                                        value_tmp += futurevalue_retire1[(k_next - CAP_MIN)/CAP_STEP];
                                                    }
                                                    else {
                                                        value_tmp += futurevalue0[(k_next - CAP_MIN)/CAP_STEP];
                                                    }
                                                    if (value_non_incorp_tmp < value_tmp) {
                                                        value_non_incorp_tmp = value_tmp;
                                                        capitalchoice_non_incorp[shock_n] = (k_next - CAP_MIN)/CAP_STEP;
                                                    }
                                                }
                                                value_non_incorp += value_non_incorp_tmp * prob_shock_n[shock_n];
                                                //if (t == 4 && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 14 ){
                                                //    std::cout << t << ", " << k << ", " << i_scaler_non_incorp << ", " << invest_non_incorp << ", " << k_non_incorp << ", " << i_scaler_non_incorp * pow(invest_non_incorp, NU) - N_COST - invest_non_incorp * (1 + R) << ", " << capital(capitalchoice_non_incorp[shock_n]) << ", " << util(k_non_incorp - capital(capitalchoice_non_incorp[shock_n])) << ", " << futurevalue1[capitalchoice_non_incorp[shock_n]] << ", " << value_non_incorp_tmp << ", " << value_non_incorp << std::endl;
                                                //}
                                            }
                                        } else {
                                            value_non_incorp = -999;
                                        }

                                        // incorporated business owner decision
                                        double value_incorp = 0;
                                        double value_incorp_tmp;
                                        vector<double> capitalchoice_incorp(SHOCK);
                                        // only able to run a business when there is no debt
                                        if (capital(k) >= 0){
                                            for(int shock_i = 0; shock_i < SHOCK; shock_i++) {
                                                value_incorp_tmp = -9999;
                                                capitalchoice_incorp[shock_i] = -1;
                                                double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                                                double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                                                double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                                                // liquidity constraint
                                                if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                                                    invest_incorp = (1 + LAMBDA_E) * capital(k);
                                                double k_incorp = (1 - DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - I_COST + (capital(k) - invest_incorp) * (1 + R);
                                                if (exp_bus == 1) {
                                                    k_incorp = (1 - DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) + (capital(k) - invest_incorp) * (1 + R);
                                                }
                                                for(int k_next = 0; k_next <= k_incorp && k_next < capital(CAP); k_next += CAP_STEP) {
                                                    double value_tmp = util(k_incorp - k_next);
                                                    if (t == RETIRE_AGE - 1) {
                                                        value_tmp += futurevalue_retire2[(k_next - CAP_MIN)/CAP_STEP];
                                                    }
                                                    else {
                                                        value_tmp += futurevalue1[(k_next - CAP_MIN)/CAP_STEP];
                                                    }
                                                    if (value_incorp_tmp < value_tmp) {
                                                        value_incorp_tmp = value_tmp;
                                                        capitalchoice_incorp[shock_i] = (k_next - CAP_MIN)/CAP_STEP;
                                                    }
                                                }
                                                value_incorp += value_incorp_tmp * prob_shock_i[shock_i];
                                            }
                                        } else {
                                            value_incorp = -999;
                                        }

                                        /*if (k > 10 && k < 30 && exp_bus == 0) {
                                         fout << "t = " << t << ", e = " << e << ", a_em = " << a_em << ", a_ub = " << a_ub << ", k = " << k << "value_worker = " << value_worker << ", value_non_incorp = " << value_non_incorp << ", value_incorp =" << value_incorp << std::endl;
                                         }*/

                                        value_worker_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus][k] = value_worker;
                                        value_non_incorp_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus][k] = value_non_incorp;
                                        value_incorp_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus][k] = value_incorp;

                                        for(int consump_n = 0; consump_n < SHOCK; consump_n++) {
                                            for(int consump_i = 0; consump_i < SHOCK; consump_i++) {
                                                //if (t == 4  && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 14 && consump_n == 5 && consump_i == 2){
                                                //    std::cout << t << ", " << k << ", " << exp_bus << ",  " << value_worker << ", " << value_non_incorp + consump_grid_n[consump_n] << ", " << value_incorp + consump_grid_n[consump_n] + consump_grid_i[consump_i] << std::endl;
                                                //}
                                                if (value_worker > value_non_incorp + consump_grid_n[consump_n] && value_worker > value_incorp + consump_grid_i[consump_i]) {
                                                    value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_worker;
                                                    career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = WORKER;
                                                    for (int shock_w = 0; shock_w < SHOCK; shock_w++) {
                                                        capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w] = capitalchoice_worker[shock_w];
                                                    }
                                                }
                                                else if (value_non_incorp + consump_grid_n[consump_n] >= value_incorp + consump_grid_i[consump_i]) {
                                                    value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_non_incorp + consump_grid_n[consump_n];
                                                    career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = NON_INCORP;
                                                    for (int shock_n = 0; shock_n < SHOCK; shock_n++) {
                                                        capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n] = capitalchoice_non_incorp[shock_n];
                                                    }
                                                }
                                                else {
                                                    value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_incorp + consump_grid_i[consump_i];
                                                    career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = INCORP;
                                                    for (int shock_i = 0; shock_i < SHOCK; shock_i++) {
                                                        capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i] = capitalchoice_incorp[shock_i];
                                                    }
                                                }
                                                //if (t == 4 && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 14 && consump_n == 5 && consump_i == 2){
                                                //    std::cout << t << ", " << k << ", " << career[0][t][e][a_em][a_ub][exp_bus][k][consump_n][consump_i] << std::endl;
                                                //}
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            /*fout << "high school value function and policy function at age 35" << std::endl;
             for(int k = 0; k < 30; k++) {
             fout << "valuefunction[" << 0 << "][" << 4 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 1 << "][" << k << "][" << 4 << "][" << 4 << "] = " << value[0][3][0][3][3][1][k][4][4] << std::endl;
             fout << "careerchoice[" << 0 << "][" << 4 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 1 << "][" << k << "][" << 4 << "][" << 4 << "] = " << career[0][3][0][3][3][1][k][4][4] << std::endl;
             }


             fout << "college value function and policy function at age 35" << std::endl;
             for(int k = 0; k < 30; k++) {
             fout << "valuefunction[" << 0 << "][" << 3 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 1 << "][" << k << "][" << 4 << "][" << 4 << "] = " << value[0][3][1][3][3][1][k][4][4] << std::endl;
             fout << "careerchoice[" << 0 << "][" << 3 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 1 << "][" << k << "][" << 4 << "][" << 4 << "] = " << career[0][3][1][3][3][1][k][4][4] << std::endl;
             }

             fout << "elite college value function and policy function at age 35" << std::endl;
             for(int k = 0; k < 30; k++) {
             fout << "valuefunction[" << 0 << "][" << 3 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 1 << "][" << k << "][" << 4 << "][" << 4 << "] = " << value[0][3][2][3][3][1][k][4][4] << std::endl;
             fout << "careerchoice[" << 0 << "][" << 3 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 1 << "][" << k << "][" << 4 << "][" << 4 << "] = " << career[0][3][2][3][3][1][k][4][4] << std::endl;
             }*/


            std::cout << "start schooling decision" << std::endl;

            // schooling decision
            schoolchoice[iter].resize(ABI);
            schoolvalue[iter].resize(ABI);
            schoolcapital[iter].resize(ABI);
            schoolchoice2[iter].resize(ABI);
            schoolcapital2[iter].resize(ABI);
            value_non_elite_noshock[iter].resize(ABI);
            value_elite_noshock[iter].resize(ABI);

            for(int a_em = 0; a_em < ABI; a_em++) {
                schoolchoice[iter][a_em].resize(ABI);
                schoolvalue[iter][a_em].resize(ABI);
                schoolcapital[iter][a_em].resize(ABI);
                schoolchoice2[iter][a_em].resize(ABI);
                schoolcapital2[iter][a_em].resize(ABI);
                value_non_elite_noshock[iter][a_em].resize(ABI);
                value_elite_noshock[iter][a_em].resize(ABI);
                for(int a_ub = 0; a_ub < ABI; a_ub++) {
                    schoolchoice[iter][a_em][a_ub].resize(ABI);
                    schoolvalue[iter][a_em][a_ub].resize(ABI);
                    schoolcapital[iter][a_em][a_ub].resize(ABI);
                    schoolchoice2[iter][a_em][a_ub].resize(ABI);
                    schoolcapital2[iter][a_em][a_ub].resize(ABI);
                    value_non_elite_noshock[iter][a_em][a_ub].resize(ABI);
                    value_elite_noshock[iter][a_em][a_ub].resize(ABI);
                    for(int a_ib = 0; a_ib < ABI; a_ib++) {
                        schoolchoice[iter][a_em][a_ub][a_ib].resize(CAP);
                        schoolvalue[iter][a_em][a_ub][a_ib].resize(CAP);
                        schoolcapital[iter][a_em][a_ub][a_ib].resize(CAP);
                        schoolchoice2[iter][a_em][a_ub][a_ib].resize(CAP);
                        schoolcapital2[iter][a_em][a_ub][a_ib].resize(CAP);
                        value_non_elite_noshock[iter][a_em][a_ub][a_ib].resize(CAP);
                        value_elite_noshock[iter][a_em][a_ub][a_ib].resize(CAP);
                        for(int k = 0; k < CAP; k++) {
                            schoolchoice[iter][a_em][a_ub][a_ib][k].resize(CAP);
                            schoolvalue[iter][a_em][a_ub][a_ib][k].resize(CAP);
                            schoolcapital[iter][a_em][a_ub][a_ib][k].resize(CAP);
                            schoolchoice2[iter][a_em][a_ub][a_ib][k].resize(CAP);
                            schoolcapital2[iter][a_em][a_ub][a_ib][k].resize(CAP);
                            value_non_elite_noshock[iter][a_em][a_ub][a_ib][k].resize(CAP);
                            value_elite_noshock[iter][a_em][a_ub][a_ib][k].resize(CAP);
                            for(int k_fam = 0; k_fam < CAP; k_fam++) {
                                schoolchoice[iter][a_em][a_ub][a_ib][k][k_fam].resize(SHOCK);
                                schoolvalue[iter][a_em][a_ub][a_ib][k][k_fam].resize(SHOCK);
                                schoolcapital[iter][a_em][a_ub][a_ib][k][k_fam].resize(SHOCK);
                                schoolchoice2[iter][a_em][a_ub][a_ib][k][k_fam].resize(SHOCK);
                                schoolcapital2[iter][a_em][a_ub][a_ib][k][k_fam].resize(SHOCK);
                                for(int consump_c = 0; consump_c < SHOCK; consump_c++) {
                                    schoolchoice[iter][a_em][a_ub][a_ib][k][k_fam][consump_c].resize(SHOCK);
                                    schoolvalue[iter][a_em][a_ub][a_ib][k][k_fam][consump_c].resize(SHOCK);
                                    schoolcapital[iter][a_em][a_ub][a_ib][k][k_fam][consump_c].resize(SHOCK);
                                    schoolchoice2[iter][a_em][a_ub][a_ib][k][k_fam][consump_c].resize(SHOCK);
                                    schoolcapital2[iter][a_em][a_ub][a_ib][k][k_fam][consump_c].resize(SHOCK);
                                }
                            }
                        }
                    }
                }
            }

#pragma omp parallel for collapse(3)
            for(int a_em = 0; a_em < ABI; a_em++) {
                for(int a_ub = 0; a_ub < ABI; a_ub++) {
                    for(int a_ib = 0; a_ib < ABI; a_ib++) {
                        vector<double> schoolfuture1(CAP, 0);
                        vector<double> schoolfuture2(CAP, 0);
                        for (int k_next_idx = 0; k_next_idx < CAP; k_next_idx++){
                            for(int consump_n_next = 0; consump_n_next < SHOCK; consump_n_next++) {
                                for(int consump_i_next = 0; consump_i_next < SHOCK; consump_i_next++) {
                                    schoolfuture1[k_next_idx] += BETA * value[iter][1][1][a_em][a_ub][a_ib][0][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                    schoolfuture2[k_next_idx] += BETA * value[iter][1][2][a_em][a_ub][a_ib][0][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                }
                            }
                        }

                        for(int k = 0; k < CAP; k++) {
                            for(int k_fam = 0; k_fam < CAP; k_fam++) {
                                // high school edu = 0
                                double value_highschool = 0;
                                for(int consump_n = 0; consump_n < SHOCK; consump_n++) {
                                    for(int consump_i = 0; consump_i < SHOCK; consump_i++) {
                                        value_highschool += value[iter][0][0][a_em][a_ub][a_ib][0][k][consump_n][consump_i] * prob_consump_n[consump_n] * prob_consump_i[consump_i];
                                    }
                                }

                                // non-elite college edu = 1
                                double value_college = -9999;
                                double capitalchoice_college = -1;
                                double k_college = (1 + R) * (capital(k) - COLLEGE_TUITION + financial_aid_nonelite(k_fam, a_em));
                                for(int k_next = CAP_MIN; k_next <= k_college && k_next < capital(CAP); k_next += CAP_STEP) {
                                    double value_tmp = util(k_college - k_next) + CONSUMPTION_COLLEGE + schoolfuture1[(k_next - CAP_MIN)/CAP_STEP];
                                    if (value_college < value_tmp) {
                                        value_college = value_tmp;
                                        capitalchoice_college = (k_next - CAP_MIN)/CAP_STEP;
                                    }
                                }
                                if (a_em == 0) {
                                  value_college = -9999;
                                }

                                // elite college = 2
                                double value_elitecollege = -9999;
                                double capitalchoice_elitecollege = -1;
                                double k_elitecollege = (1 + R) * (capital(k) - elite_tuition(subsidy) + financial_aid_elite(k_fam, a_em));
                                for(int k_next = CAP_MIN; k_next <= k_elitecollege && k_next < capital(CAP); k_next += CAP_STEP) {
                                    double value_tmp = util(k_elitecollege - k_next) + CONSUMPTION_ELITE + schoolfuture2[(k_next - CAP_MIN)/CAP_STEP];
                                    if (value_elitecollege < value_tmp) {
                                        value_elitecollege = value_tmp;
                                        capitalchoice_elitecollege = (k_next - CAP_MIN)/CAP_STEP;
                                    }
                                }
                                if (a_em < 2) {
                                  value_elitecollege = -9999;
                                }

                                value_non_elite_noshock[iter][a_em][a_ub][a_ib][k][k_fam] = value_college;
                                value_elite_noshock[iter][a_em][a_ub][a_ib][k][k_fam] = value_elitecollege;

                                for(int consump_c = 0; consump_c < SHOCK; consump_c++) {
                                    for(int consump_e = 0; consump_e < SHOCK; consump_e++) {
                                        if (value_highschool > value_college + consump_grid_c[consump_c] && value_highschool > value_elitecollege + consump_grid_e[consump_e]){
                                            schoolchoice[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = NO_COLLEGE;
                                            schoolvalue[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = value_highschool;
                                            schoolcapital[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = -1;
                                        }
                                        else if (value_college + consump_grid_c[consump_c] > value_elitecollege + consump_grid_e[consump_e]) {
                                            schoolchoice[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = COLLEGE;
                                            schoolvalue[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = value_college + consump_grid_c[consump_c];
                                            schoolcapital[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = capitalchoice_college;
                                        }
                                        else {
                                            schoolchoice[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = ELITE;
                                            // check if the student is admitted by elite college
							                int sat_g = 0;
							                if (a_em >= 5) {
								                sat_g = 2;
							                }  else if (a_em >= 2) {
							 	                sat_g = 1;
							                }
                                            int subsidy_idx = subsidy*10;
                                            //if (a_em == 3 & a_ib == 3 & a_ub == 3 & k == 7 & k_fam == 7 & consump_c == 3 & consump_e == 3)
                                            //    std::cout << subsidy_idx << std::endl;
                                            schoolvalue[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = ELITE_ADMIT[sat_g] * ADMIT_ADJUSTED[subsidy_idx] * value_elitecollege + (1-ELITE_ADMIT[sat_g]* ADMIT_ADJUSTED[subsidy_idx]) * value_college + consump_grid_e[consump_e];
                                            schoolcapital[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = capitalchoice_elitecollege;
                                            if (value_highschool > value_college + consump_grid_c[consump_c]) {
                                                schoolchoice2[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = NO_COLLEGE;
                                                schoolcapital2[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = -1;
                                            }
                                            else {
                                                schoolchoice2[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = COLLEGE;
                                                schoolcapital2[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = capitalchoice_college;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }




        //*********************** Simulate shocks and initial value ***************************//


        std::cout << "start simulation" << std::endl;
        int size = 3; // Dimensionality (rows)

        Eigen::internal::scalar_normal_dist_op<double> randN; // Gaussian functor
        Eigen::internal::scalar_normal_dist_op<double>::rng.seed(1); // Seed the rng

        // Define mean and covariance of the distribution
        Eigen::VectorXd mean(size);
        Eigen::MatrixXd covar(size,size);

        mean  <<  0,  0,  0;
        covar << ABI_EM_STD*ABI_EM_STD, 0, 0,
        0,  ABI_UB_STD*ABI_UB_STD, 0,
        0, 0, ABI_IB_STD*ABI_IB_STD;

        Eigen::MatrixXd normTransform(size,size);

        Eigen::LLT<Eigen::MatrixXd> cholSolver(covar);

        // We can only use the cholesky decomposition if
        // the covariance matrix is symmetric, pos-definite.
        // But a covariance matrix might be pos-semi-definite.
        // In that case, we'll go to an EigenSolver
        if (cholSolver.info()==Eigen::Success) {
            // Use cholesky solver
            normTransform = cholSolver.matrixL();
        } else {
            // Use eigen solver
            Eigen::SelfAdjointEigenSolver<Eigen::MatrixXd> eigenSolver(covar);
            normTransform = eigenSolver.eigenvectors()
            * eigenSolver.eigenvalues().cwiseSqrt().asDiagonal();
        }


        Eigen::MatrixXd samples = (normTransform * Eigen::MatrixXd::NullaryExpr(size,num_id,randN)).colwise() + mean;



        // initial value
        int seed = 1987;
        srand(1);
        for (int id = 0; id < num_id; id++){
            // initial ability
            double abi_em_rand = samples(0, id);
            abi_em_org_sim[id] = abi_em_rand;
            double abi_em_min = abi_grid_em[0];
            double abi_em_step = 2 * RANGE_ABI * ABI_EM_STD / (ABI - 1);
            abi_em_sim[id] = round((abi_em_rand - abi_em_min)/abi_em_step);
            if (abi_em_sim[id] > ABI - 1){
                abi_em_sim[id] = ABI - 1;
            } else if (abi_em_sim[id] < 0){
                abi_em_sim[id] = 0;
            }
            double abi_ub_rand = samples(1, id);
            abi_ub_org_sim[id] = abi_ub_rand;
            double abi_ub_min = abi_grid_ub[0];
            double abi_ub_step = 2 * RANGE_ABI * ABI_UB_STD / (ABI - 1);
            abi_ub_sim[id] = round((abi_ub_rand - abi_ub_min)/abi_ub_step);
            if (abi_ub_sim[id] > ABI - 1){
                abi_ub_sim[id] = ABI - 1;
            } else if (abi_ub_sim[id] < 0){
                abi_ub_sim[id] = 0;
            }
            double abi_ib_rand = samples(2, id);
            abi_ib_org_sim[id] = abi_ib_rand;
            double abi_ib_min = abi_grid_ib[0];
            double abi_ib_step = 2 * RANGE_ABI * ABI_IB_STD / (ABI - 1);
            abi_ib_sim[id] = round((abi_ib_rand - abi_ib_min)/abi_ib_step);
            if (abi_ib_sim[id] > ABI - 1){
                abi_ib_sim[id] = ABI - 1;
            } else if (abi_ib_sim[id] < 0){
                abi_ib_sim[id] = 0;
            }
            // initial wealth
            double k_random_number = ((double) rand() / (RAND_MAX)) * 100; //draw a random number between 0 and 100
            double cdf = K_CON_DIST[abi_em_sim[id]][0];
            int group_id = 0;
            while (cdf < k_random_number) {
                group_id ++;
                cdf += K_CON_DIST[abi_em_sim[id]][group_id];
            }
            k_idx_sim[id][0] = group_mapping(group_id);
            // initial family wealth
            double k_fam_rand = r8_normal(0, K_FAM_STD, seed);
            double k_fam;
            if (capital(k_idx_sim[id][0]) > 0) {
                k_fam = exp(log(capital(k_idx_sim[id][0])) * COEF_K + COEF_ABI[abi_em_sim[id]] + k_fam_rand);
            } else {
                k_fam = exp(COEF_ABI[abi_em_sim[id]] + k_fam_rand);
            }
            k_fam_idx_sim[id] = round((k_fam - CAP_MIN)/CAP_STEP);
            //if (id < 500)
            //    std::cout << k_idx_sim[id][0] << ", " << k_fam_rand << ", " << k_fam << ", " << k_fam_idx_sim[id] << std::endl;
            if (k_fam_idx_sim[id] < 0){
                k_fam_idx_sim[id] = 0;
            } else if (k_fam_idx_sim[id] > CAP-1){
                k_fam_idx_sim[id] = CAP - 1;
            }
            // if (id < 10)
            //     fout << abi_em_rand << ", " << abi_ub_rand << ", " << k_random_number << ", " << k_fam_rand << std::endl;
        }

        // Initial ability distribution
        int sum_abi_em[ABI];
        double frac_abi_em[ABI];
        int sum_abi_ub[ABI];
        double frac_abi_ub[ABI];
        int sum_abi_ib[ABI];
        double frac_abi_ib[ABI];

        for (int a = 0; a < ABI; a++){
            sum_abi_em[a] = 0;
            sum_abi_ub[a] = 0;
            sum_abi_ib[a] = 0;
        }
        for (int id = 0; id < num_id; id++) {
            int a_em = abi_em_sim[id];
            int a_ub = abi_ub_sim[id];
            int a_ib = abi_ib_sim[id];
            sum_abi_em[a_em] += 1;
            sum_abi_ub[a_ub] += 1;
            sum_abi_ib[a_ib] += 1;
        }
        for (int a = 0; a < ABI; a++){
            frac_abi_em[a] = (double)sum_abi_em[a] / num_id;
            // std::cout << "frac_abi_em[" << a << "] = " << frac_abi_em[a] << std::endl;
            frac_abi_ub[a] = (double)sum_abi_ub[a] / num_id;
            // std::cout << "frac_abi_ub[" << a << "] = " << frac_abi_ub[a] << std::endl;
            frac_abi_ib[a] = (double)sum_abi_ib[a] / num_id;
            // std::cout << "frac_abi_ib[" << a << "] = " << frac_abi_ib[a] << std::endl;
        }

        // Initial wealth distribution
        int sum_wealth[CAP];
        double frac_wealth[CAP];

        for (int k = 0; k < CAP; k++){
            sum_wealth[k] = 0;
        }
        for (int id = 0; id < num_id; id++) {
            int k = k_idx_sim[id][0];
            sum_wealth[k] += 1;
        }
        for (int k = 0; k < CAP; k++){
            frac_wealth[k] = (double)sum_wealth[k] / num_id;
            // std::cout << "frac_wealth[" << k << "] = " << frac_wealth[k] << std::endl;
        }

        // Initial family wealth distribution
        int sum_wealth_fam[CAP];
        double frac_wealth_fam[CAP];

        for (int k = 0; k < CAP; k++){
            sum_wealth_fam[k] = 0;
        }
        for (int id = 0; id < num_id; id++) {
            int k = k_fam_idx_sim[id];
            sum_wealth_fam[k] += 1;
        }
        for (int k = 0; k < CAP; k++){
            frac_wealth_fam[k] = (double)sum_wealth_fam[k] / num_id;
            // std::cout << "frac_wealth_fam[" << k << "] = " << frac_wealth_fam[k] << std::endl;
        }

        // simulate shocks
        for (int id = 0; id < num_id; id++) {
            for (int t = 0; t < RETIRE_AGE; t++) {
                double shock_w_rand = r8_normal(0, W_STD, seed);
                double shock_w_min = shock_grid_w[0];
                double shock_w_step = 2 * RANGE_SHOCK * W_STD / (SHOCK - 1);
                shock_w_sim[id][t] = round((shock_w_rand - shock_w_min)/shock_w_step);
                if (shock_w_sim[id][t] > SHOCK - 1){
                    shock_w_sim[id][t] = SHOCK - 1;
                } else if (shock_w_sim[id][t] < 0){
                    shock_w_sim[id][t] = 0;
                }
                double shock_n_rand = r8_normal(0, N_STD, seed);
                double shock_n_min = shock_grid_n[0];
                double shock_n_step = 2 * RANGE_SHOCK * N_STD / (SHOCK - 1);
                shock_n_sim[id][t] = round((shock_n_rand - shock_n_min)/shock_n_step);
                if (shock_n_sim[id][t] > SHOCK - 1){
                    shock_n_sim[id][t] = SHOCK - 1;
                } else if (shock_n_sim[id][t] < 0){
                    shock_n_sim[id][t] = 0;
                }
                double shock_i_rand = r8_normal(0, I_STD, seed);
                double shock_i_min = shock_grid_i[0];
                double shock_i_step = 2 * RANGE_SHOCK * I_STD / (SHOCK - 1);
                shock_i_sim[id][t] = round((shock_i_rand - shock_i_min)/shock_i_step);
                if (shock_i_sim[id][t] > SHOCK - 1){
                    shock_i_sim[id][t] = SHOCK - 1;
                } else if (shock_i_sim[id][t] < 0){
                    shock_i_sim[id][t] = 0;
                }
                double consump_n_rand = r8_normal(0, CONSUMP_N_STD, seed);
                double consump_n_min = consump_grid_n[0];
                double consump_n_step = 2 * RANGE_SHOCK * CONSUMP_N_STD / (SHOCK - 1);
                consump_n_sim[id][t] = round((consump_n_rand - consump_n_min)/consump_n_step);
                if (consump_n_sim[id][t] > SHOCK - 1){
                    consump_n_sim[id][t] = SHOCK - 1;
                } else if (consump_n_sim[id][t] < 0){
                    consump_n_sim[id][t] = 0;
                }
                double consump_i_rand = r8_normal(0, CONSUMP_I_STD, seed);
                double consump_i_min = consump_grid_i[0];
                double consump_i_step = 2 * RANGE_SHOCK * CONSUMP_I_STD / (SHOCK - 1);
                consump_i_sim[id][t] = round((consump_i_rand - consump_i_min)/consump_i_step);
                if (consump_i_sim[id][t] > SHOCK - 1){
                    consump_i_sim[id][t] = SHOCK - 1;
                } else if (consump_i_sim[id][t] < 0){
                    consump_i_sim[id][t] = 0;
                }
            }
            double consump_c_rand = r8_normal(0, CONSUMP_C_STD, seed);
            double consump_c_min = consump_grid_c[0];
            double consump_c_step = 2 * RANGE_SHOCK * CONSUMP_C_STD / (SHOCK - 1);
            consump_c_sim[id] = round((consump_c_rand - consump_c_min)/consump_c_step);
            if (consump_c_sim[id] > SHOCK - 1){
                consump_c_sim[id] = SHOCK - 1;
            } else if (consump_c_sim[id] < 0){
                consump_c_sim[id] = 0;
            }
            double consump_e_rand = r8_normal(0, CONSUMP_E_STD, seed);
            double consump_e_min = consump_grid_e[0];
            double consump_e_step = 2 * RANGE_SHOCK * CONSUMP_E_STD / (SHOCK - 1);
            consump_e_sim[id] = round((consump_e_rand - consump_e_min)/consump_e_step);
            if (consump_e_sim[id] > SHOCK - 1){
                consump_e_sim[id] = SHOCK - 1;
            } else if (consump_e_sim[id] < 0){
                consump_e_sim[id] = 0;
            }

        }

        for (int id = 0; id < num_id; id++) {
            for (int t = RETIRE_AGE; t < AGE; t++) {
                double shock_n_rand = r8_normal(0, N_STD, seed);
                double shock_n_min = shock_grid_n[0];
                double shock_n_step = 2 * RANGE_SHOCK * N_STD / (SHOCK - 1);
                shock_n_sim[id][t] = round((shock_n_rand - shock_n_min)/shock_n_step);
                if (shock_n_sim[id][t] > SHOCK - 1){
                    shock_n_sim[id][t] = SHOCK - 1;
                } else if (shock_n_sim[id][t] < 0){
                    shock_n_sim[id][t] = 0;
                }
                double shock_i_rand = r8_normal(0, I_STD, seed);
                double shock_i_min = shock_grid_i[0];
                double shock_i_step = 2 * RANGE_SHOCK * I_STD / (SHOCK - 1);
                shock_i_sim[id][t] = round((shock_i_rand - shock_i_min)/shock_i_step);
                if (shock_i_sim[id][t] > SHOCK - 1){
                    shock_i_sim[id][t] = SHOCK - 1;
                } else if (shock_i_sim[id][t] < 0){
                    shock_i_sim[id][t] = 0;
                }
                double consump_n_rand = r8_normal(0, CONSUMP_N_STD, seed);
                double consump_n_min = consump_grid_n[0];
                double consump_n_step = 2 * RANGE_SHOCK * CONSUMP_N_STD / (SHOCK - 1);
                consump_n_sim[id][t] = round((consump_n_rand - consump_n_min)/consump_n_step);
                if (consump_n_sim[id][t] > SHOCK - 1){
                    consump_n_sim[id][t] = SHOCK - 1;
                } else if (consump_n_sim[id][t] < 0){
                    consump_n_sim[id][t] = 0;
                }
                double consump_i_rand = r8_normal(0, CONSUMP_I_STD, seed);
                double consump_i_min = consump_grid_i[0];
                double consump_i_step = 2 * RANGE_SHOCK * CONSUMP_I_STD / (SHOCK - 1);
                consump_i_sim[id][t] = round((consump_i_rand - consump_i_min)/consump_i_step);
                if (consump_i_sim[id][t] > SHOCK - 1){
                    consump_i_sim[id][t] = SHOCK - 1;
                } else if (consump_i_sim[id][t] < 0){
                    consump_i_sim[id][t] = 0;
                }
                shock_s_sim[id][t] = ((double) rand() / RAND_MAX);
            }
        }

        double laborw_sum = 0;
        double capital_sum = 0;
        double labor_sum = 0;
        double capitalr_sum = 0;
        double pretax_labor_income_sum = 0;
        double subsidy_sum = 0;
        // simulate first period: education decision
        for (int id = 0; id < num_id; id++) {
            int iter = 0;
            int t = 0;
            int a_em = abi_em_sim[id];
            int a_ub = abi_ub_sim[id];
            int a_ib = abi_ib_sim[id];
            int k = k_idx_sim[id][0];
            int k_fam = k_fam_idx_sim[id];
            int consump_c = consump_c_sim[id];
            int consump_e = consump_e_sim[id];
            double edu_random_number = ((double) rand() / (RAND_MAX)); 

            edu_sim[id] = schoolchoice[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e];
            // check if the student is admitted by elite college
	        int flag = 0;
	        if (edu_sim[id] == 2){
		        int sat_group = 0;
		        if (a_em >= 5) {
			        sat_group = 2;
		        } else if (a_em >= 2) {
			        sat_group = 1;
		        }
                int subsidy_idx = subsidy*10;
		        if (edu_random_number > ELITE_ADMIT[sat_group] * ADMIT_ADJUSTED[subsidy_idx]) {
			        edu_sim[id] = schoolchoice2[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e];
			        flag = 1;
		        }   
	        }
            // high school - work
            if (edu_sim[id] == 0){
                int e = edu_sim[id];
                int exp_bus = 0;
                int shock_w = shock_w_sim[id][t];
                int shock_n = shock_n_sim[id][t];
                int shock_i = shock_i_sim[id][t];
                int consump_n = consump_n_sim[id][t];
                int consump_i = consump_i_sim[id][t];

                career_sim[id][t] = career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i];
                if (career_sim[id][t] == 0){
                    // worker
                    income_sim[id][t] = W * h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e) + shock_grid_w[shock_w]);
                    exp_bus_sim[id][t] = 0;
                    k_idx_sim[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w];
                    labor_sum += h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e) + shock_grid_w[shock_w]);
                    laborw_sum += income_sim[id][t];
                    capital_sum += k_idx_sim[id][t];
                    capitalr_sum += R * k_idx_sim[id][t];
                    pretax_labor_income_sum += income_sim[id][t] / (1 - TAX);
                } else if (career_sim[id][0] == 1) {
                    double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                    double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                    double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                    // liquidity constraint
                    if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                        invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                    income_sim[id][0] = (1 - DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) - invest_non_incorp * (1 + R);
                    exp_bus_sim[id][0] = 0;
                    k_idx_sim[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n];
                    capital_sum += (invest_non_incorp - k_idx_sim[id][t]);
                    capitalr_sum += R * (invest_non_incorp - k_idx_sim[id][t]);
                } else {
                    double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                    double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                    double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                    // liquidity constraint
                    if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                        invest_incorp = (1 + LAMBDA_E) * capital(k);
                    income_sim[id][0] = (1- DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - I_COST - invest_incorp * (1 + R);
                    exp_bus_sim[id][1] = 1;
                    k_idx_sim[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i];
                    capital_sum += (invest_incorp - k_idx_sim[id][t]);
                    capitalr_sum += R * (invest_incorp - k_idx_sim[id][t]);
                }
            } else {
                // college
                career_sim[id][0] = -1;
                income_sim[id][0] = 0;
                exp_bus_sim[id][0] = 0;
                k_idx_sim[id][t+1] = schoolcapital[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e];
                if (flag == 1) {
			        k_idx_sim[id][t+1] = schoolcapital2[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e];
		        }
                capital_sum += k_idx_sim[id][t];
                capitalr_sum += R * k_idx_sim[id][t];
                if (edu_sim[id] == 2) {
                    subsidy_sum += 4*33046*subsidy;
                }
            }
            //if (id < 50)
            //    std::cout << "ability = " << abi_w_sim[id] << ", " << abi_e_sim[id] << ", shock = " << consump_c_sim[id] << ", " << consump_e_sim[id] << ", edu = " << edu_sim[id] << ", k = " << k_idx_sim[id][0] << ", " << k_idx_sim[id][1] << ", career/income/exp_bus = " << career_sim[id][0] << ", " << income_sim[id][0] << ", " << exp_bus_sim[id][0] << std::endl;
        }

        // simulate period 2 - RETIRE_AGE
        for (int id = 0; id < num_id; id++) {
            for (int t = 1; t < RETIRE_AGE; t++) {
                int iter = 0;
                int a_em = abi_em_sim[id];
                int a_ub = abi_ub_sim[id];
                int a_ib = abi_ib_sim[id];
                int e = edu_sim[id];
                int exp_bus = exp_bus_sim[id][t-1];
                int k = k_idx_sim[id][t];
                int consump_n = consump_n_sim[id][t];
                int consump_i = consump_i_sim[id][t];
                int shock_w = shock_w_sim[id][t];
                int shock_n = shock_n_sim[id][t];
                int shock_i = shock_i_sim[id][t];


                career_sim[id][t] = career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i];
                if (career_sim[id][t] == 0){
                    income_sim[id][t] = W * h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e) + shock_grid_w[shock_w]);
                    exp_bus_sim[id][t] = 0;
                    k_idx_sim[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w];
                    labor_sum += h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e) + shock_grid_w[shock_w]);
                    laborw_sum += income_sim[id][t];
                    capital_sum += k_idx_sim[id][t];
                    capitalr_sum += R * k_idx_sim[id][t];
                    pretax_labor_income_sum += income_sim[id][t] / (1 - TAX);
                } else if (career_sim[id][t] == 1) {
                    double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                    double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                    double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                    // liquidity constraint
                    if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                        invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                    income_sim[id][t] = (1-DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) - invest_non_incorp * (1 + R);
                    exp_bus_sim[id][t] = 0;
                    if (t == RETIRE_AGE - 1) {
                        exp_bus_sim[id][t] = 1;
                    }
                    k_idx_sim[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n];
                    capital_sum += (invest_non_incorp - k_idx_sim[id][t]);
                    capitalr_sum += R * (invest_non_incorp - k_idx_sim[id][t]);
                } else {
                    double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                    double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                    double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                    // liquidity constraint
                    if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                        invest_incorp = (1 + LAMBDA_E) * capital(k);
                    income_sim[id][t] = (1-DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - I_COST - invest_incorp * (1 + R);
                    if (exp_bus == 1) {
                        income_sim[id][t] = (1-DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - invest_incorp * (1 + R);
                    }
                    exp_bus_sim[id][t] = 1;
                    if (t == RETIRE_AGE - 1) {
                        exp_bus_sim[id][t] = 2;
                    }
                    k_idx_sim[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i];
                    capital_sum += (invest_incorp - k_idx_sim[id][t]);
                    capitalr_sum += R * (invest_incorp - k_idx_sim[id][t]);
                }

                //if (id < 50){
                //    std::cout << id << ", " << t << ", ability =" << abi_w_sim[id] << ", " << abi_e_sim[id] << ", shock = " << consump_n_sim[id] << ", " << shock_w_sim[id][t] << ", " << shock_n_sim[id][t] << ", " << shock_i_sim[id][t] << ", edu = " << edu_sim[id] << ", k = " << k_idx_sim[id][t] << ", " << k_idx_sim[id][t+1] << ", career/income/exp_bus = " << career_sim[id][t] << ", " << income_sim[id][t] << ", " << exp_bus_sim[id][t] << std::endl;
                //}
            }
        }

        // simulate RETIRE_AGE to Max_age
        for (int id = 0; id < num_id; id++) {
            for (int t = RETIRE_AGE; t < AGE; t++) {
                int iter = 0;
                int a_em = abi_em_sim[id];
                int a_ub = abi_ub_sim[id];
                int a_ib = abi_ib_sim[id];
                int e = edu_sim[id];
                int exp_bus = exp_bus_sim[id][t-1];
                int k = k_idx_sim[id][t];
                int consump_n = consump_n_sim[id][t];
                int consump_i = consump_i_sim[id][t];
                int shock_w = shock_w_sim[id][t];
                int shock_n = shock_n_sim[id][t];
                int shock_i = shock_i_sim[id][t];

                if (shock_s_sim[id][t] < ZETA[t-RETIRE_AGE]){
                    career_sim[id][t] = career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i];
                    if (career_sim[id][t] == 0){
                        income_sim[id][t] = P * W * h_em(a_em, e) * exp(4 * ALPHA_1 + 22.7 * ALPHA_2);
                        exp_bus_sim[id][t] = 0;
                        k_idx_sim[id][t+1] = capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w];
                        capital_sum += k_idx_sim[id][t];
                        capitalr_sum += R * k_idx_sim[id][t];
                    } else if (career_sim[id][t] == 1) {
                        double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                        double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                        double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                        // liquidity constraint
                        if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                            invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                        income_sim[id][t] = (1-DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) - invest_non_incorp * (1 + R);
                        exp_bus_sim[id][t] = 1;
                        k_idx_sim[id][t+1] = capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n];
                        capital_sum += (invest_non_incorp - k_idx_sim[id][t]);
                        capitalr_sum += R * (invest_non_incorp - k_idx_sim[id][t]);
                    } else {
                        double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                        double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                        double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                        // liquidity constraint
                        if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                            invest_incorp = (1 + LAMBDA_E) * capital(k);
                        income_sim[id][t] = (1-DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - I_COST - invest_incorp * (1 + R);
                        if (exp_bus == 2) {
                            income_sim[id][t] = (1-DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - invest_incorp * (1 + R);
                        }
                        exp_bus_sim[id][t] = 2;
                        k_idx_sim[id][t+1] = capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i];
                        capital_sum += (invest_incorp - k_idx_sim[id][t]);
                        capitalr_sum += R * (invest_incorp - k_idx_sim[id][t]);
                    }
                } else {
                    career_sim[id][t] = -1;
                }
            }
        }


        //**********************************Calculate new wage rate and interest rate****************************************************//
        //fout << "wage rate, interest rate and tax rate" << std::endl;
        //fout << capital_sum << ", " << capitalr_sum << ", " << labor_sum << ", " << laborw_sum << std::endl;
        TAX2 = subsidy_sum / pretax_labor_income_sum;
        //W2 = (1 - TAX2) * PROD_A * (1 - PROD_ALPHA) * pow(capital_sum/labor_sum, PROD_ALPHA);
        //R2 = PROD_A * PROD_ALPHA * pow(labor_sum/capital_sum, 1-PROD_ALPHA);
        fout << "W2 = " << W2 << ", R2 = " << R2 << ", TAX2 = " << TAX2 << std::endl;

    }

    std::cout << "start parental transfer" << std::endl;
    // parent transfer at age 50 (t = 6)
    int iter = 0;
    vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > > > transfer(ITER);
    transfer[iter].resize(EDU);
    for(int e = 0; e < EDU; e++) {
        transfer[iter][e].resize(ABI);
        for(int a_em = 0; a_em < ABI; a_em++) {
            transfer[iter][e][a_em].resize(ABI);
            for(int a_ub = 0; a_ub < ABI; a_ub++) {
                transfer[iter][e][a_em][a_ub].resize(ABI);
                for(int a_ib = 0; a_ib < ABI; a_ib++) {
                    transfer[iter][e][a_em][a_ub][a_ib].resize(exp_em(6,e)+1);
                    for(int exp_bus = 0; exp_bus < EXP_BUS_TYPE; exp_bus++) {
                        transfer[iter][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                        for(int k = 0; k < CAP; k++) {
                            transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k].resize(SHOCK);
                            for(int consump_n = 0; consump_n < SHOCK; consump_n++) {
                                transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n].resize(SHOCK);
                                for(int consump_i = 0; consump_i < SHOCK; consump_i++) {
                                    transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i].resize(SHOCK);
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    for(int t = 6; t <= 6; t++) {
        std::cout << "Processing age = " << age(t) << std::endl;
#pragma omp parallel for collapse(4)
        for(int e = 0; e < EDU; e++) {
            for(int a_em = 0; a_em < ABI; a_em++) {
                for(int a_ub = 0; a_ub < ABI; a_ub++) {
                    for(int a_ib = 0; a_ib < ABI; a_ib++) {
                        vector< vector <double> > childvalue(CAP);
                        for (int c_transfer_idx = 0; c_transfer_idx < CAP; c_transfer_idx++){
                            childvalue[c_transfer_idx].resize(CAP);
                            for (int k = 0; k < CAP; k++) {
                                childvalue[c_transfer_idx][k] = 0;
                                for(int consump_c_next = 0; consump_c_next < SHOCK; consump_c_next++) {
                                    for(int consump_e_next = 0; consump_e_next < SHOCK; consump_e_next++) {
                                        for (int aem_next = 0; aem_next < ABI; aem_next++) {
                                            for (int aub_next = 0; aub_next < ABI; aub_next++) {
                                                for (int aib_next = 0; aib_next < ABI; aib_next++) {
                                                    childvalue[c_transfer_idx][k] += OMEGA * schoolvalue[iter][aem_next][aub_next][aib_next][c_transfer_idx][k][consump_c_next][consump_e_next] * prob_consump_c[consump_c_next] * prob_consump_e[consump_e_next] * tranprob_abi_em[a_em][aem_next] * tranprob_abi_ub[a_ub][aub_next] * tranprob_abi_ib[a_ib][aib_next];
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        for(int exp_bus = 0; exp_bus < EXP_BUS_TYPE; exp_bus++) {
                            vector<double> futurevalue0(CAP, 0);
                            vector<double> futurevalue1(CAP, 0);
                            for (int k_next_idx = 0; k_next_idx < CAP; k_next_idx++){
                                for(int consump_n_next = 0; consump_n_next < SHOCK; consump_n_next++) {
                                    for(int consump_i_next = 0; consump_i_next < SHOCK; consump_i_next++) {
                                        futurevalue0[k_next_idx] += BETA * value[iter][t+1][e][a_em][a_ub][a_ib][0][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                        futurevalue1[k_next_idx] += BETA * value[iter][t+1][e][a_em][a_ub][a_ib][1][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                    }
                                }
                            }

                            for(int k = 0; k < CAP; k++) {
                                // worker's decision
                                double value_worker = 0;
                                double value_worker_tmp;
                                vector<double> capitalchoice_worker(SHOCK);
                                vector<double> transferchoice_worker(SHOCK);
                                for(int shock_w = 0; shock_w < SHOCK; shock_w++) {
                                    value_worker_tmp = -9999;
                                    capitalchoice_worker[shock_w] = -1;
                                    transferchoice_worker[shock_w] = -1;
                                    double k_worker = capital(k) * (1 + R) + W * h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e) + shock_grid_w[shock_w]);
                                    for(int k_next = 0; k_next <= k_worker && k_next < capital(CAP); k_next += CAP_STEP) {
                                        for (int c_transfer = 0; c_transfer <= k_worker - k_next && c_transfer < capital(CAP); c_transfer += CAP_STEP) {
                                            double value_tmp = util(k_worker - k_next - c_transfer) + childvalue[(c_transfer - CAP_MIN)/CAP_STEP][k] + futurevalue0[(k_next - CAP_MIN)/CAP_STEP];
                                            if (value_worker_tmp < value_tmp) {
                                                value_worker_tmp = value_tmp;
                                                capitalchoice_worker[shock_w] = (k_next - CAP_MIN)/CAP_STEP;
                                                transferchoice_worker[shock_w] = (c_transfer - CAP_MIN)/CAP_STEP;
                                            }
                                        }
                                    }
                                    value_worker += value_worker_tmp * prob_shock_w[shock_w];
                                    //if (t == 6 && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 18 ){
                                    //    std::cout << t << ", " << k << ", " << k_worker << ", " << h_w(a_em, e) * exp(ALPHA_0 + ALPHA_1 * exp_w(t, e) + ALPHA_2 * exp_w(t, e) * exp_w(t, e) + shock_grid_w[shock_w]) << ", " << capital(capitalchoice_worker[shock_w]) << ", " << util(k_worker - capital(capitalchoice_worker[shock_w])) << ", " << futurevalue0[capitalchoice_worker[shock_w]] << ", " << value_worker_tmp << ", " << value_worker << std::endl;
                                    //}

                                }

                                // non-incorporated business owner decision
                                double value_non_incorp = 0;
                                double value_non_incorp_tmp;
                                vector<double> capitalchoice_non_incorp(SHOCK);
                                vector<double> transferchoice_non_incorp(SHOCK);
                                // only able to run a business when there is no debt
                                if (capital(k) >= 0){
                                    for(int shock_n = 0; shock_n < SHOCK; shock_n++) {
                                        value_non_incorp_tmp = -9999;
                                        capitalchoice_non_incorp[shock_n] = -1;
                                        transferchoice_non_incorp[shock_n] = -1;
                                        double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                                        double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                                        double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                                        // liquidity constraint
                                        if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                                            invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                                        double k_non_incorp = (1 - DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) + (capital(k) - invest_non_incorp) * (1 + R);
                                        for(int k_next = 0; k_next <= k_non_incorp && k_next < capital(CAP); k_next += CAP_STEP) {
                                            for (int c_transfer = 0; c_transfer <= k_non_incorp - k_next && c_transfer < capital(CAP); c_transfer += CAP_STEP) {
                                                double value_tmp = util(k_non_incorp - k_next - c_transfer) + childvalue[(c_transfer - CAP_MIN)/CAP_STEP][k] + futurevalue0[(k_next - CAP_MIN)/CAP_STEP];
                                                if (value_non_incorp_tmp < value_tmp) {
                                                    value_non_incorp_tmp = value_tmp;
                                                    capitalchoice_non_incorp[shock_n] = (k_next - CAP_MIN)/CAP_STEP;
                                                    transferchoice_non_incorp[shock_n] = (c_transfer - CAP_MIN)/CAP_STEP;
                                                }
                                            }
                                        }
                                        value_non_incorp += value_non_incorp_tmp * prob_shock_n[shock_n];
                                        //if (t == 6 && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 18 ){
                                        //    std::cout << t << ", " << k << ", " << i_scaler_non_incorp << ", " << invest_non_incorp << ", " << k_non_incorp << ", " << i_scaler_non_incorp * pow(invest_non_incorp, NU) - N_COST - invest_non_incorp * (1 + R) << ", " << capital(capitalchoice_non_incorp[shock_n]) << ", " << util(k_non_incorp - capital(capitalchoice_non_incorp[shock_n])) << ", " << futurevalue1[capitalchoice_non_incorp[shock_n]] << ", " << value_non_incorp_tmp << ", " << value_non_incorp << std::endl;
                                        //}

                                    }
                                } else {
                                    value_non_incorp = -999;
                                }

                                // incorporated business owner decision
                                double value_incorp = 0;
                                double value_incorp_tmp;
                                vector<double> capitalchoice_incorp(SHOCK);
                                vector<double> transferchoice_incorp(SHOCK);
                                // only able to run a business when there is no debt
                                if (capital(k) >= 0){
                                    for(int shock_i = 0; shock_i < SHOCK; shock_i++) {
                                        value_incorp_tmp = -9999;
                                        capitalchoice_incorp[shock_i] = -1;
                                        transferchoice_incorp[shock_i] = -1;
                                        double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                                        double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                                        double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                                        // liquidity constraint
                                        if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                                            invest_incorp = (1 + LAMBDA_E) * capital(k);
                                        double k_incorp = (1 - DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - I_COST + (capital(k) - invest_incorp) * (1 + R);
                                        if (exp_bus == 1) {
                                            k_incorp = (1 - DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) + (capital(k) - invest_incorp) * (1 + R);
                                        }
                                        for(int k_next = 0; k_next <= k_incorp && k_next < capital(CAP); k_next += CAP_STEP) {
                                            for (int c_transfer = 0; c_transfer < k_incorp - k_next && c_transfer < capital(CAP); c_transfer += CAP_STEP) {
                                                double value_tmp = util(k_incorp - k_next - c_transfer) + childvalue[(c_transfer - CAP_MIN)/CAP_STEP][k] + futurevalue1[(k_next - CAP_MIN)/CAP_STEP];
                                                if (value_incorp_tmp < value_tmp) {
                                                    value_incorp_tmp = value_tmp;
                                                    capitalchoice_incorp[shock_i] = (k_next - CAP_MIN)/CAP_STEP;
                                                    transferchoice_incorp[shock_i] = (c_transfer - CAP_MIN)/CAP_STEP;
                                                }
                                            }
                                        }
                                        value_incorp += value_incorp_tmp * prob_shock_i[shock_i];
                                    }
                                } else {
                                    value_incorp = -999;
                                }

                                value_worker_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus][k] = value_worker;
                                value_non_incorp_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus][k] = value_non_incorp;
                                value_incorp_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus][k] = value_incorp;

                                for(int consump_n = 0; consump_n < SHOCK; consump_n++) {
                                    for(int consump_i = 0; consump_i < SHOCK; consump_i++) {
                                        //if (t == 6  && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 18 && consump_n == 5 && consump_i == 2){
                                        //    std::cout << t << ", " << k << ", " << exp_bus << ",  " << value_worker << ", " << value_non_incorp + consump_grid_n[consump_n] << ", " << value_incorp + consump_grid_n[consump_n] + consump_grid_i[consump_i] << std::endl;
                                        //}
                                        if (value_worker > value_non_incorp + consump_grid_n[consump_n] && value_worker > value_incorp + consump_grid_i[consump_i]) {
                                            value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_worker;
                                            career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = WORKER;
                                            for (int shock_w = 0; shock_w < SHOCK; shock_w++) {
                                                capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w] = capitalchoice_worker[shock_w];
                                                transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w] = transferchoice_worker[shock_w];
                                            }
                                        }
                                        else if (value_non_incorp + consump_grid_n[consump_n] >= value_incorp + consump_grid_i[consump_i]) {
                                            value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_non_incorp + consump_grid_n[consump_n];
                                            career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = NON_INCORP;
                                            for (int shock_n = 0; shock_n < SHOCK; shock_n++) {
                                                capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n] = capitalchoice_non_incorp[shock_n];
                                                transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n] = transferchoice_non_incorp[shock_n];
                                            }
                                        }
                                        else {
                                            value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_incorp + consump_grid_i[consump_i];
                                            career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = INCORP;
                                            for (int shock_i = 0; shock_i < SHOCK; shock_i++) {
                                                capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i] = capitalchoice_incorp[shock_i];
                                                transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i] = transferchoice_incorp[shock_i];
                                            }
                                        }
                                        //if (t == 6 && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 18 && consump_n == 5 && consump_i == 2){
                                        //    std::cout << t << ", " << k << ", " << career[0][t][e][a_em][a_ub][exp_bus][k][consump_n][consump_i] << std::endl;
                                        //}
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    // simulate period 2 - RETIRE_AGE
    int transfer_sim[num_id];
    for (int id = 0; id < num_id; id++) {
        for (int t = 6; t < 7; t++) {
            int iter = 0;
            int a_em = abi_em_sim[id];
            int a_ub = abi_ub_sim[id];
            int a_ib = abi_ib_sim[id];
            int e = edu_sim[id];
            int exp_bus = exp_bus_sim[id][t-1];
            int k = k_idx_sim[id][t];
            int consump_n = consump_n_sim[id][t];
            int consump_i = consump_i_sim[id][t];
            int shock_w = shock_w_sim[id][t];
            int shock_n = shock_n_sim[id][t];
            int shock_i = shock_i_sim[id][t];


            career_sim[id][t] = career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i];
            if (career_sim[id][t] == 0){
                transfer_sim[id] = transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w];
            } else if (career_sim[id][t] == 1) {
                transfer_sim[id] = transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n];
            } else {
                transfer_sim[id] = transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i];
            }
        }
    }

    // Second generation
    std::cout << "Simulate the second generation" << std::endl;
    int seed = 2;
    // simulation array
    int abi_em_sim2[num_id];
    int abi_ub_sim2[num_id];
    int abi_ib_sim2[num_id];
    int edu_sim2[num_id];
    int k_idx_sim2[num_id][RETIRE_AGE + 1];
    int k_fam_idx_sim2[num_id];
    int career_sim2[num_id][RETIRE_AGE];
    double income_sim2[num_id][RETIRE_AGE];
    int exp_bus_sim2[num_id][RETIRE_AGE];
    int shock_w_sim2[num_id][RETIRE_AGE];
    int shock_n_sim2[num_id][RETIRE_AGE];
    int shock_i_sim2[num_id][RETIRE_AGE];
    int consump_n_sim2[num_id][RETIRE_AGE];
    int consump_i_sim2[num_id][RETIRE_AGE];
    int consump_c_sim2[num_id];
    int consump_e_sim2[num_id];

    // initial value

    for (int id = 0; id < num_id; id++){
        double abi_em_shock_rand = r8_normal(0, ABI_EM_STD, seed);
        double abi_em_rand2 = abi_em_org_sim[id] * THETA_EM + abi_em_shock_rand;
        double abi_em_min = abi_grid_em[0];
        double abi_em_step = 2 * RANGE_ABI * ABI_EM_STD / (ABI - 1);
        abi_em_sim2[id] = round((abi_em_rand2 - abi_em_min)/abi_em_step);
        if (abi_em_sim2[id] > ABI - 1){
            abi_em_sim2[id] = ABI - 1;
        } else if (abi_em_sim2[id] < 0){
            abi_em_sim2[id] = 0;
        }
        double abi_ub_shock_rand = r8_normal(0, ABI_UB_STD, seed);
        double abi_ub_rand2 = abi_ub_org_sim[id] * THETA_UB + abi_ub_shock_rand;
        double abi_ub_min = abi_grid_ub[0];
        double abi_ub_step = 2 * RANGE_ABI * ABI_UB_STD / (ABI - 1);
        abi_ub_sim2[id] = round((abi_ub_rand2 - abi_ub_min)/abi_ub_step);
        if (abi_ub_sim2[id] > ABI - 1){
            abi_ub_sim2[id] = ABI - 1;
        } else if (abi_ub_sim2[id] < 0){
            abi_ub_sim2[id] = 0;
        }
        double abi_ib_shock_rand = r8_normal(0, ABI_IB_STD, seed);
        double abi_ib_rand2 = abi_ib_org_sim[id] * THETA_IB + abi_ib_shock_rand;
        double abi_ib_min = abi_grid_ib[0];
        double abi_ib_step = 2 * RANGE_ABI * ABI_IB_STD / (ABI - 1);
        abi_ib_sim2[id] = round((abi_ib_rand2 - abi_ib_min)/abi_ib_step);
        if (abi_ib_sim2[id] > ABI - 1){
            abi_ib_sim2[id] = ABI - 1;
        } else if (abi_ib_sim2[id] < 0){
            abi_ib_sim2[id] = 0;
        }
        // initial wealth
        k_idx_sim2[id][0] = transfer_sim[id];
        // initial family wealth
        k_fam_idx_sim2[id] = k_idx_sim[id][6];
        //if (id < 50){
        //    std::cout << id << ", " << abi_em_sim[id] << ", " << abi_ub_sim[id] << ", " << abi_ib_sim[id] << ", " << abi_em_sim2[id] << ", " << abi_ub_sim2[id] << ", " << abi_ib_sim2[id] << ", " << k_idx_sim2[id][0] << ", " << k_fam_idx_sim2[id] << std::endl;
        //}
    }


    // simulate shocks
    for (int id = 0; id < num_id; id++) {
        for (int t = 0; t < RETIRE_AGE; t++) {
            double shock_w_rand = r8_normal(0, W_STD, seed);
            double shock_w_min = shock_grid_w[0];
            double shock_w_step = 2 * RANGE_SHOCK * W_STD / (SHOCK - 1);
            shock_w_sim2[id][t] = round((shock_w_rand - shock_w_min)/shock_w_step);
            if (shock_w_sim2[id][t] > SHOCK - 1){
                shock_w_sim2[id][t] = SHOCK - 1;
            } else if (shock_w_sim2[id][t] < 0){
                shock_w_sim2[id][t] = 0;
            }
            double shock_n_rand = r8_normal(0, N_STD, seed);
            double shock_n_min = shock_grid_n[0];
            double shock_n_step = 2 * RANGE_SHOCK * N_STD / (SHOCK - 1);
            shock_n_sim2[id][t] = round((shock_n_rand - shock_n_min)/shock_n_step);
            if (shock_n_sim2[id][t] > SHOCK - 1){
                shock_n_sim2[id][t] = SHOCK - 1;
            } else if (shock_n_sim2[id][t] < 0){
                shock_n_sim2[id][t] = 0;
            }
            double shock_i_rand = r8_normal(0, I_STD, seed);
            double shock_i_min = shock_grid_i[0];
            double shock_i_step = 2 * RANGE_SHOCK * I_STD / (SHOCK - 1);
            shock_i_sim2[id][t] = round((shock_i_rand - shock_i_min)/shock_i_step);
            if (shock_i_sim2[id][t] > SHOCK - 1){
                shock_i_sim2[id][t] = SHOCK - 1;
            } else if (shock_i_sim2[id][t] < 0){
                shock_i_sim2[id][t] = 0;
            }
            double consump_n_rand = r8_normal(0, CONSUMP_N_STD, seed);
            double consump_n_min = consump_grid_n[0];
            double consump_n_step = 2 * RANGE_SHOCK * CONSUMP_N_STD / (SHOCK - 1);
            consump_n_sim2[id][t] = round((consump_n_rand - consump_n_min)/consump_n_step);
            if (consump_n_sim2[id][t] > SHOCK - 1){
                consump_n_sim2[id][t] = SHOCK - 1;
            } else if (consump_n_sim2[id][t] < 0){
                consump_n_sim2[id][t] = 0;
            }
            double consump_i_rand = r8_normal(0, CONSUMP_I_STD, seed);
            double consump_i_min = consump_grid_i[0];
            double consump_i_step = 2 * RANGE_SHOCK * CONSUMP_I_STD / (SHOCK - 1);
            consump_i_sim2[id][t] = round((consump_i_rand - consump_i_min)/consump_i_step);
            if (consump_i_sim2[id][t] > SHOCK - 1){
                consump_i_sim2[id][t] = SHOCK - 1;
            } else if (consump_i_sim2[id][t] < 0){
                consump_i_sim2[id][t] = 0;
            }
            //if (id < 10 && t < 10)
            //    std::cout << u_rand << ", " << u_sim[id][t] << ", " << eps_rand << ", " << eps_sim[id][t] << std::endl;
        }
        double consump_c_rand = r8_normal(0, CONSUMP_C_STD, seed);
        double consump_c_min = consump_grid_c[0];
        double consump_c_step = 2 * RANGE_SHOCK * CONSUMP_C_STD / (SHOCK - 1);
        consump_c_sim2[id] = round((consump_c_rand - consump_c_min)/consump_c_step);
        if (consump_c_sim2[id] > SHOCK - 1){
            consump_c_sim2[id] = SHOCK - 1;
        } else if (consump_c_sim2[id] < 0){
            consump_c_sim2[id] = 0;
        }
        double consump_e_rand = r8_normal(0, CONSUMP_E_STD, seed);
        double consump_e_min = consump_grid_e[0];
        double consump_e_step = 2 * RANGE_SHOCK * CONSUMP_E_STD / (SHOCK - 1);
        consump_e_sim2[id] = round((consump_e_rand - consump_e_min)/consump_e_step);
        if (consump_e_sim2[id] > SHOCK - 1){
            consump_e_sim2[id] = SHOCK - 1;
        } else if (consump_e_sim2[id] < 0){
            consump_e_sim2[id] = 0;
        }
    }

    // simulate first period: education decision
    for (int id = 0; id < num_id; id++) {
        int iter = 0;
        int t = 0;
        int a_em = abi_em_sim2[id];
        int a_ub = abi_ub_sim2[id];
        int a_ib = abi_ib_sim2[id];
        int k = k_idx_sim2[id][0];
        int k_fam = k_fam_idx_sim2[id];
        int consump_c = consump_c_sim2[id];
        int consump_e = consump_e_sim2[id];

        edu_sim2[id] = schoolchoice[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e];
        //if (edu_sim2[id] == 2){
        //    std::cout << id << ", " << abi_em_sim[id] << ", " << abi_ub_sim[id] << ", " << abi_ib_sim[id] << ", " << abi_em_sim2[id] << ", " << abi_ub_sim2[id] << ", " << abi_ib_sim2[id] << ", " << k_idx_sim2[id][0] << ", " << k_fam_idx_sim2[id] << std::endl;
        //}
        // high school - work
        if (edu_sim2[id] == 0){
            int e = edu_sim2[id];
            int exp_bus = 0;
            int shock_w = shock_w_sim2[id][t];
            int shock_n = shock_n_sim2[id][t];
            int shock_i = shock_i_sim2[id][t];
            int consump_i = consump_i_sim2[id][t];
            int consump_n = consump_n_sim2[id][t];

            career_sim2[id][t] = career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i];
            if (career_sim2[id][t] == 0){
                // worker
                income_sim2[id][t] = W * h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e) + shock_grid_w[shock_w]);
                exp_bus_sim2[id][t] = 0;
                k_idx_sim2[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w];
            } else if (career_sim2[id][0] == 1) {
                double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                // liquidity constraint
                if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                    invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                income_sim2[id][0] = (1-DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) - invest_non_incorp * (1 + R);
                exp_bus_sim2[id][0] = 0;
                k_idx_sim2[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n];
            } else {
                double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                // liquidity constraint
                if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                    invest_incorp = (1 + LAMBDA_E) * capital(k);
                income_sim2[id][0] = (1-DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - I_COST - invest_incorp * (1 + R);
                exp_bus_sim2[id][1] = 1;
                k_idx_sim2[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i];
            }
        } else {
            // college
            career_sim2[id][0] = -1;
            income_sim2[id][0] = 0;
            exp_bus_sim2[id][0] = 0;
            k_idx_sim2[id][t+1] = schoolcapital[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e];
        }
        //if (id < 50)
        //    std::cout << "ability = " << abi_em_sim2[id] << ", " << abi_ub_sim2[id] << ", " << abi_ib_sim2[id] << ", edu = " << edu_sim2[id] << ", k = " << k_idx_sim2[id][0] << ", " << k_idx_sim2[id][1] << ", career/income/exp_bus = " << career_sim2[id][0] << ", " << income_sim2[id][0] << ", " << exp_bus_sim2[id][0] << std::endl;
    }

    // simulate period 2 - RETIRE_AGE
    for (int id = 0; id < num_id; id++) {
        for (int t = 1; t < RETIRE_AGE; t++) {
            int iter = 0;
            int a_em = abi_em_sim2[id];
            int a_ub = abi_ub_sim2[id];
            int a_ib = abi_ib_sim2[id];
            int e = edu_sim2[id];
            int exp_bus = exp_bus_sim2[id][t-1];
            int k = k_idx_sim2[id][t];
            int consump_n = consump_n_sim2[id][t];
            int consump_i = consump_i_sim2[id][t];
            int shock_w = shock_w_sim2[id][t];
            int shock_n = shock_n_sim2[id][t];
            int shock_i = shock_i_sim2[id][t];


            career_sim2[id][t] = career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i];
            if (career_sim2[id][t] == 0){
                income_sim2[id][t] = W * h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e) + shock_grid_w[shock_w]);
                exp_bus_sim2[id][t] = 0;
                k_idx_sim2[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w];
            } else if (career_sim2[id][t] == 1) {
                double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                // liquidity constraint
                if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                    invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                income_sim2[id][t] = (1-DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) - invest_non_incorp * (1 + R);
                exp_bus_sim2[id][t] = 0;
                k_idx_sim2[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n];
            } else {
                double h_em = h_em(a_em, e) * exp(ALPHA_1 * exp_em(t, e) + ALPHA_2 * exp_em(t, e) * exp_em(t, e));
                double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                // liquidity constraint
                if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                    invest_incorp = (1 + LAMBDA_E) * capital(k);
                income_sim2[id][t] = (1-DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - invest_incorp * (1 + R);
                if (exp_bus == 1) {
                    income_sim2[id][t] = (1-DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - invest_incorp * (1 + R);
                }
                exp_bus_sim2[id][t] = 1;
                k_idx_sim2[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i];
            }

            //if (id < 50)
            //    std::cout << id << ", " << t << ", ability =" << abi_em_sim2[id] << ", " << abi_ub_sim2[id] << ", " << abi_ib_sim2[id] << ", shock = " << shock_w_sim2[id][t] << ", " << shock_n_sim2[id][t] << ", " << shock_i_sim2[id][t] << ", edu = " << edu_sim2[id] << ", k = " << k_idx_sim2[id][t] << ", " << k_idx_sim2[id][t+1] << ", career/income/exp_bus = " << career_sim2[id][t] << ", " << income_sim2[id][t] << ", " << exp_bus_sim2[id][t] << std::endl;

        }
    }


    //************************************** Moment Calculation **************************************//
    std::cout << "finish simulation, start moment calculation" << std::endl;

    fout << "education choie, career choice, income by edu, income by career, career transition, age of starting business, duration of business, intergenerational income correlation, welfare, income gini" << std::endl;

    // Proportion by edu group
    int sum_edu[EDU];
    double frac_edu[EDU];

    for (int e = 0; e < EDU; e++){
        sum_edu[e] = 0;
    }
    for (int id = 0; id < num_id; id++) {
        int e = edu_sim[id];
        sum_edu[e] += 1;
    }
    for (int e = 0; e < EDU; e++){
        //fout << "sum_edu[" << e << "] = " << sum_edu[e] << std::endl;
        frac_edu[e] = (double)sum_edu[e] / num_id;
        fout << frac_edu[e] << ", ";
    }



    // Proportion by career type
    int sum_career[CAREER_CHOICES];
    int count_career = 0;
    double frac_career[CAREER_CHOICES];

    for (int car = 0; car < CAREER_CHOICES; car++){
        sum_career[car] = 0;
    }
    for (int id = 0; id < num_id; id++){
        for (int t = 1; t < RETIRE_AGE - 1; t++) {
            int car = career_sim[id][t];
            if (car >=0){
                sum_career[car] += 1;
                count_career += 1;
            }
        }
    }
    for (int car = 0; car < CAREER_CHOICES; car++){
        frac_career[car] = (double) sum_career[car]/count_career;
        fout << frac_career[car] << ", ";
    }



    // Wage/Income by edu
    double sum_income_edu[EDU];
    double avg_income_edu[EDU];
    double sum_incomesq_edu[EDU];
    double sd_income_edu[EDU];

    for (int e = 0; e < EDU; e++){
        sum_income_edu[e] = 0;
    }

    for (int id = 0; id < num_id; id++){
        for (int t = 1; t < 8; t++) {
            int e = edu_sim[id];
            sum_income_edu[e] += income_sim[id][t];
        }
    }

    for (int e = 0; e < EDU; e++){
        avg_income_edu[e] = (double) sum_income_edu[e]/sum_edu[e]/(8-1);
        fout << avg_income_edu[e] << ", " ;
    }



    // Wage/Income by career
    double sum_income_career[CAREER_CHOICES];
    double avg_income_career[CAREER_CHOICES];
    double sum_incomesq_career[CAREER_CHOICES];
    double sd_income_career[CAREER_CHOICES];

    for (int car = 0; car < CAREER_CHOICES; car++){
        sum_income_career[car] = 0;
    }

    for (int id = 0; id < num_id; id++){
        for (int t = 1; t < 8; t++) {
            int car = career_sim[id][t];
            sum_income_career[car] += income_sim[id][t];
        }
    }

    for (int car = 0; car < CAREER_CHOICES; car++){
        avg_income_career[car] = (double) sum_income_career[car]/sum_career[car];
        fout << avg_income_career[car] << ", ";
    }


    // Transition matrix
    double sum_career_lag[CAREER_CHOICES];
    double sum_career_tran[CAREER_CHOICES][CAREER_CHOICES];
    double frac_career_tran[CAREER_CHOICES][CAREER_CHOICES];
    double sum_income_tran[CAREER_CHOICES][CAREER_CHOICES];
    double avg_income_tran[CAREER_CHOICES][CAREER_CHOICES];
    vector<vector<vector<double> > > income_vector(CAREER_CHOICES, vector<vector<double> >(CAREER_CHOICES));
    vector<vector<vector<double> > > income_lag_vector(CAREER_CHOICES, vector<vector<double> >(CAREER_CHOICES));

    for (int car_lag = 0; car_lag < CAREER_CHOICES; car_lag++){
        sum_career_lag[car_lag] = 0;
        for (int car = 0; car < CAREER_CHOICES; car++){
            sum_career_tran[car_lag][car] = 0;
            sum_income_tran[car_lag][car] = 0;
        }
    }

    for (int id = 0; id < num_id; id++){
        for (int t = 2; t < 8; t++) {
            int car_lag = career_sim[id][t-1];
            int car = career_sim[id][t];
            sum_career_tran[car_lag][car] += 1;
            sum_career_lag[car_lag] += 1;
            sum_income_tran[car_lag][car] += income_sim[id][t-1];
            income_lag_vector[car_lag][car].push_back(income_sim[id][t-1]);
            income_vector[car_lag][car].push_back(income_sim[id][t]);
        }
    }

    for (int car_lag = 0; car_lag < CAREER_CHOICES; car_lag++){
        for (int car = 0; car < CAREER_CHOICES; car++){
            frac_career_tran[car_lag][car] = sum_career_tran[car_lag][car]/sum_career_lag[car_lag];
            fout << frac_career_tran[car_lag][car] << ", ";
        }
    }

    //Age of starting business
    double num_age_bus[2][4];
    double sum_age_bus[2][4];
    double avg_age_bus[2][4];

    for (int bus = 0; bus < 2; bus++){
        for (int edu = 0; edu < 4; edu++){
            num_age_bus[bus][edu] = 0;
            sum_age_bus[bus][edu] = 0;
        }
    }


    for (int id = 0; id < num_id; id++){
        double flag_bus_ub = 0;
        double flag_bus_ib = 0;
        int edu = edu_sim[id];
        for (int t = 0; t < RETIRE_AGE - 1; t++) {
            if (flag_bus_ub == 0 && career_sim[id][t] == 1) {
                flag_bus_ub = 1;
                num_age_bus[0][edu] += 1;
                sum_age_bus[0][edu] += 20 + t*5;
                num_age_bus[0][3] += 1;
                sum_age_bus[0][3] += 20 + t*5;
            } else if (flag_bus_ib == 0 && career_sim[id][t] == 2) {
                flag_bus_ib = 1;
                num_age_bus[1][edu] += 1;
                sum_age_bus[1][edu] += 20 + t*5;
                num_age_bus[1][3] += 1;
                sum_age_bus[1][3] += 20 + t*5;
            }
        }
    }

    for (int bus = 0; bus < 2; bus++){
        for (int edu = 3; edu < 4; edu++){
            avg_age_bus[bus][edu] = sum_age_bus[bus][edu]/num_age_bus[bus][edu];
            fout << avg_age_bus[bus][edu] << ", ";
        }
    }

    // Duation of business
    double num_dur_bus[2][4];
    double sum_dur_bus[2][4];
    double avg_dur_bus[2][4];

    for (int bus = 0; bus < 2; bus++){
        for (int edu = 0; edu < 4; edu++){
            num_dur_bus[bus][edu] = 0;
            sum_dur_bus[bus][edu] = 0;
        }
    }


    for (int id = 0; id < num_id; id++){
        double flag_ub = 0;
        double flag_ib = 0;
        int edu = edu_sim[id];
        double dur_ub;
        double dur_ib;
        for (int t = 0; t < RETIRE_AGE - 1; t++) {
            if (flag_ub == 0 && career_sim[id][t] == 1) {
                flag_ub = 1;
                dur_ub = 1;
            } else if (flag_ub == 1 && career_sim[id][t] == 1) {
                dur_ub += 1;
                if (t == RETIRE_AGE - 2) {
                    num_dur_bus[0][edu] += 1;
                    sum_dur_bus[0][edu] += dur_ub * 5;
                    num_dur_bus[0][3] += 1;
                    sum_dur_bus[0][3] += dur_ub * 5;
                }
            } else if (flag_ub == 1 && career_sim[id][t] != 1) {
                flag_ub = 0;
                num_dur_bus[0][edu] += 1;
                sum_dur_bus[0][edu] += dur_ub * 5;
                num_dur_bus[0][3] += 1;
                sum_dur_bus[0][3] += dur_ub * 5;
            } else if (flag_ib == 0 && career_sim[id][t] == 2) {
                flag_ib = 1;
                dur_ib = 1;
            } else if (flag_ib == 1 && career_sim[id][t] == 2) {
                dur_ib += 1;
                if (t == RETIRE_AGE - 2) {
                    num_dur_bus[1][edu] += 1;
                    sum_dur_bus[1][edu] += dur_ib * 5;
                    num_dur_bus[1][3] += 1;
                    sum_dur_bus[1][3] += dur_ib * 5;
                }
            } else if (flag_ib == 1 && career_sim[id][t] != 2) {
                flag_ib = 0;
                num_dur_bus[1][edu] += 1;
                sum_dur_bus[1][edu] += dur_ib * 5;
                num_dur_bus[1][3] += 1;
                sum_dur_bus[1][3] += dur_ib * 5;
            }
        }
    }

    for (int bus = 0; bus < 2; bus++){
        for (int edu = 3; edu < 4; edu++){
            avg_dur_bus[bus][edu] = sum_dur_bus[bus][edu]/num_dur_bus[bus][edu];
            fout << avg_dur_bus[bus][edu] << ", ";
        }
    }

    // Intergenerational income persistency
    double son_avginc[num_id];
    double father_avginc[num_id];
    for (int id = 0; id < num_id; id++) {
        double avginc = 0;
        double fa_avginc = 0;
        int flag_n_chi = 0;
        int flag_i_chi = 0;
        int flag_n_par = 0;
        int flag_i_par = 0;
        for (int t = 2; t < 6; t++) {
            avginc += income_sim2[id][t];
            fa_avginc += income_sim[id][t];
            if (flag_n_par == 0 && career_sim[id][t] == 1) {
                flag_n_par = 1;
            }
            if (flag_n_chi == 0 && career_sim2[id][t] == 1) {
                flag_n_chi = 1;
            }
            if (flag_i_par == 0 && career_sim[id][t] == 2) {
                flag_i_par = 1;
            }
            if (flag_i_chi == 0 && career_sim2[id][t] == 2) {
                flag_i_chi = 1;
            }
        }
        int car_par = 0;
        int car_chi = 0;
        if (flag_i_par == 1) {
            car_par = 2;
        } else if (flag_n_par == 1) {
            car_par = 1;
        }
        if (flag_i_chi == 1) {
            car_chi = 2;
        } else if (flag_n_chi == 1) {
            car_chi = 1;
        }
        if (avginc > 0) {
            son_avginc[id] = log(avginc / 4);
        } else {
            son_avginc[id] = 0;
        }
        if (fa_avginc > 0) {
            father_avginc[id] = log(fa_avginc / 4);
        } else {
            father_avginc[id] = 0;
        }
    }

    fout << olsCoefficient(son_avginc, father_avginc, num_id) << ", ";

    // Welfare
    double welfare_sum = 0;
    double welfare_avg;


    for (int id = 0; id < num_id; id++) {
        int iter = 0;
        int a_em = abi_em_sim[id];
        int a_ub = abi_ub_sim[id];
        int a_ib = abi_ib_sim[id];
        int k_idx = k_idx_sim[id][0];
        int k_fam = k_fam_idx_sim[id];
        int consump_c = consump_c_sim[id];
        int consump_e = consump_e_sim[id];
        int e = edu_sim[id];

        welfare_sum += schoolvalue[iter][a_em][a_ub][a_ib][k_idx][k_fam][consump_c][consump_e];
    }

    welfare_avg = welfare_sum / num_id - 10.133;
    fout << welfare_avg << ", ";

    // income Gini coefficient
    int alive_pop = 0;
    for (int id = 0; id < num_id; id++){
        for (int t = 0; t < RETIRE_AGE; t++) {
            alive_pop += 1;
        }
    }
    for (int id = 0; id < num_id; id++){
        for (int t = RETIRE_AGE; t < AGE; t++) {
            if (career_sim[id][t] > -1) {
                alive_pop += 1;
            }
        }
    }

    double income_gini_n = 0;
    double income_gini_d = 0;
    double income_gini;
    double income_gini_vector[alive_pop];

    int pop_iter = 0;
    for (int id = 0; id < num_id; id++){
        for (int t = 0; t < RETIRE_AGE; t++) {
            income_gini_vector[pop_iter] = income_sim[id][t];
            income_gini_d += income_sim[id][t];
            pop_iter += 1;
        }
    }

    for (int id = 0; id < num_id; id++){
        for (int t = RETIRE_AGE; t < AGE; t++) {
            if (career_sim[id][t] > -1) {
                income_gini_vector[pop_iter] = income_sim[id][t];
                income_gini_d += income_sim[id][t];
                pop_iter += 1;
            }
        }
    }

    sort(income_gini_vector, income_gini_vector + alive_pop);
    for (int i = 0; i < alive_pop; i++) {
        income_gini_n += (alive_pop + 1 - i) * income_gini_vector[i];
    }
    income_gini = (alive_pop + 1 - 2 * income_gini_n / income_gini_d) / alive_pop;
    fout << income_gini << ", ";

    // Ability and initial wealth by edu group
    double sum_abi_edu_em[EDU];
    double frac_abi_edu_em[EDU];
    double sum_abi_edu_ub[EDU];
    double frac_abi_edu_ub[EDU];
    double sum_abi_edu_ib[EDU];
    double frac_abi_edu_ib[EDU];
    double sum_wealth_edu[EDU];
    double frac_wealth_edu[EDU];

    for (int e = 0; e < EDU; e++){
        sum_abi_edu_em[e] = 0;
        sum_abi_edu_ub[e] = 0;
        sum_abi_edu_ib[e] = 0;
        sum_wealth_edu[e] = 0;
    }
    for (int id = 0; id < num_id; id++) {
        int e = edu_sim[id];
        sum_abi_edu_em[e] += abi_grid_em[abi_em_sim[id]];
        sum_abi_edu_ub[e] += abi_grid_ub[abi_ub_sim[id]];
        sum_abi_edu_ib[e] += abi_grid_ib[abi_ib_sim[id]];
        sum_wealth_edu[e] += capital(k_fam_idx_sim[id]);
    }
    for (int e = 0; e < EDU; e++){
        frac_abi_edu_em[e] = (double)sum_abi_edu_em[e] / sum_edu[e];
        frac_abi_edu_ub[e] = (double)sum_abi_edu_ub[e] / sum_edu[e];
        frac_abi_edu_ib[e] = (double)sum_abi_edu_ib[e] / sum_edu[e];
        frac_wealth_edu[e] = (double)sum_wealth_edu[e] / sum_edu[e];
    }
    fout << frac_abi_edu_em[2] << ", ";
    fout << frac_abi_edu_ub[2] << ", ";
    fout << frac_abi_edu_ib[2] << ", ";
    fout << frac_wealth_edu[2] << std::endl;

//    double wealth_gini_n = 0;
//    double wealth_gini_d = 0;
//    double wealth_gini;
//    double wealth_vector[alive_pop];
//
//    int pop_iter = 0;
//    for (int id = 0; id < num_id; id++){
//        for (int t = 0; t < RETIRE_AGE; t++) {
//            wealth_vector[pop_iter] = capital(k_idx_sim[id][t]);
//            wealth_gini_d += capital(k_idx_sim[id][t]);
//            pop_iter += 1;
//        }
//    }
//    for (int id = 0; id < num_id; id++){
//        for (int t = RETIRE_AGE; t < AGE; t++) {
//            if (career_sim[id][t] > -1) {
//                wealth_vector[pop_iter] = capital(k_idx_sim[id][t]);
//                wealth_gini_d += capital(k_idx_sim[id][t]);
//                pop_iter += 1;
//            }
//        }
//    }
//
//
//    // wealth Gini
//    sort(wealth_vector, wealth_vector + alive_pop);
//    for (int i = 0; i < alive_pop; i++) {
//        wealth_gini_n += (alive_pop + 1 - i) * wealth_vector[i];
//    }
//    wealth_gini = (alive_pop + 1 - 2 * wealth_gini_n / wealth_gini_d) / alive_pop;
//    fout << wealth_gini << std::endl;

    std::cout << "finish moment calculation" << std::endl;
    return 0;
}




int main(int argc, const char * argv[]) {
    fout.open("./debug_out.txt");
    for (double subsidy = 0; subsidy <= 1; subsidy += 0.1) {
        backward_induction(ABI_EM_STD, ABI_UB_STD, ABI_IB_STD, W_STD, N_STD, I_STD, LAMBDA_E, LAMBDA_C, OMEGA, THETA_EM, THETA_UB, THETA_IB, subsidy);
    }
    fout.close();
    return 0;
}
