//
//  main.cpp
//  Entrepreneurship
//
//  Created by Huang Zhiyi on 23/1/2016.
//  Modified by Guo Naijia
//  Copyright © 2016 Huang Zhiyi. All rights reserved.
//  Temporary shock on non-pecuniary utility for unincorporated and incorporated

#include <cmath>
#include <iostream>
#include <fstream>
#include <vector>
#include <cstdio>
#include <algorithm>
#include <iomanip>
#include <Eigen/Dense>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/normal_distribution.hpp>
#include <bits/stdc++.h>

#include "parameters.h"
#include "normal.h"
#include "norm_gen.h"

using namespace std;

float correlationCoefficient(double X[], double Y[], int n)
{

    double sum_X = 0, sum_Y = 0, sum_XY = 0;
    double squareSum_X = 0, squareSum_Y = 0;

    for (int i = 0; i < n; i++)
    {
        // sum of elements of array X.
        sum_X = sum_X + X[i];

        // sum of elements of array Y.
        sum_Y = sum_Y + Y[i];

        // sum of X[i] * Y[i].
        sum_XY = sum_XY + X[i] * Y[i];

        // sum of square of array elements.
        squareSum_X = squareSum_X + X[i] * X[i];
        squareSum_Y = squareSum_Y + Y[i] * Y[i];
    }

    // use formula for calculating correlation coefficient.
    float corr = (float)(n * sum_XY - sum_X * sum_Y)
    / sqrt((n * squareSum_X - sum_X * sum_X)
           * (n * squareSum_Y - sum_Y * sum_Y));

    return corr;
}

float olsCoefficient(double X[], double Y[], int n)
{

    double sum_X = 0, sum_Y = 0, sum_XY = 0;
    double squareSum_X = 0, squareSum_Y = 0;

    for (int i = 0; i < n; i++)
    {
        // sum of elements of array X.
        sum_X = sum_X + X[i];

        // sum of elements of array Y.
        sum_Y = sum_Y + Y[i];

        // sum of X[i] * Y[i].
        sum_XY = sum_XY + X[i] * Y[i];

        // sum of square of array elements.
        squareSum_X = squareSum_X + X[i] * X[i];
        squareSum_Y = squareSum_Y + Y[i] * Y[i];
    }

    // use formula for calculating correlation coefficient.
    // float ols = (float)(sum_XY / squareSum_X);
    float ols = (float)(n * sum_XY - sum_X * sum_Y) / (n * squareSum_X - sum_X * sum_X);

    return ols;
}

double sum(vector<double> a)
{
    double s = 0;
    for (int i = 0; i < a.size(); i++)
    {
        s += a[i];
    }
    return s;
}

double mean(vector<double> a)
{
    return sum(a) / a.size();
}

double sqsum(vector<double> a)
{
    double s = 0;
    for (int i = 0; i < a.size(); i++)
    {
        s += pow(a[i], 2);
    }
    return s;
}

double stdev(vector<double> nums)
{
    double N = nums.size();
    return pow(sqsum(nums) / N - pow(sum(nums) / N, 2), 0.5);
}


vector<double> operator-(vector<double> a, double b)
{
    vector<double> retvect;
    for (int i = 0; i < a.size(); i++)
    {
        retvect.push_back(a[i] - b);
    }
    return retvect;
}

vector<double> operator*(vector<double> a, vector<double> b)
{
    vector<double> retvect;
    for (int i = 0; i < a.size() ; i++)
    {
        retvect.push_back(a[i] * b[i]);
    }
    return retvect;
}

double pearsoncoeff(vector<double> X, vector<double> Y)
{
    return sum((X - mean(X))*(Y - mean(Y))) / (X.size()*stdev(X)* stdev(Y));
}


namespace Eigen {
    namespace internal {
        template<typename Scalar>
        struct scalar_normal_dist_op
        {
            static boost::mt19937 rng;    // The uniform pseudo-random algorithm
            mutable boost::normal_distribution<Scalar> norm;  // The gaussian combinator

            EIGEN_EMPTY_STRUCT_CTOR(scalar_normal_dist_op)

            template<typename Index>
            inline const Scalar operator() (Index, Index = 0) const { return norm(rng); }
        };

        template<typename Scalar> boost::mt19937 scalar_normal_dist_op<Scalar>::rng;

        template<typename Scalar>
        struct functor_traits<scalar_normal_dist_op<Scalar> >
        { enum { Cost = 50 * NumTraits<Scalar>::MulCost, PacketAccess = false, IsRepeatable = false }; };
    } // end namespace internal
} // end namespace Eigen


int backward_induction(double abi_em_std,
                       double abi_ub_std,
                       double abi_ib_std,
                       double w_h_std,
                       double w_c_std,
                       double w_e_std,
                       double n_std,
                       double i_std,
                       double lambda_e,
                       double lambda_c,
                       double omega,
                       double theta_em,
                       double theta_ub,
                       double theta_ib) {


    std::ofstream fout("./debug_out.txt");
    std::ofstream fout_est("./estimation_out.txt");
    //std::ofstream fout_c1("./c1_out.txt");
    std::ofstream fout_c2("./c2_out.txt");

    // ability grid
    double abi_grid_em[ABI];
    double abi_grid_ub[ABI];
    double abi_grid_ib[ABI];

    for (int i = 0; i < ABI; i++) {
        abi_grid_em[i] = -RANGE_ABI * ABI_EM_STD + 2 * RANGE_ABI * ABI_EM_STD * i / (ABI - 1);
        abi_grid_ub[i] = -RANGE_ABI * ABI_UB_STD + 2 * RANGE_ABI * ABI_UB_STD * i / (ABI - 1);
        abi_grid_ib[i] = -RANGE_ABI * ABI_IB_STD + 2 * RANGE_ABI * ABI_IB_STD * i / (ABI - 1);
        //std::cout << "abi_grid_em[" << i << "] = " << abi_grid_em[i] << std::endl;
        //std::cout << "abi_grid_ub[" << i << "] = " << abi_grid_ub[i] << std::endl;
        //std::cout << "abi_grid_ib[" << i << "] = " << abi_grid_ib[i] << std::endl;
    }

    double mid_abi_grid_em[ABI - 1];
    double mid_abi_grid_ub[ABI - 1];
    double mid_abi_grid_ib[ABI - 1];
    double prob_abi_em[ABI];
    double prob_abi_ub[ABI];
    double prob_abi_ib[ABI];

    for (int i = 0; i < ABI - 1; i++) {
        mid_abi_grid_em[i] = (abi_grid_em[i] + abi_grid_em[i + 1]) / 2;
        mid_abi_grid_ub[i] = (abi_grid_ub[i] + abi_grid_ub[i + 1]) / 2;
        mid_abi_grid_ib[i] = (abi_grid_ib[i] + abi_grid_ib[i + 1]) / 2;
        //std::cout << "mid_abi_grid_em[" << i << "] = " << mid_abi_grid_em[i] << std::endl;
        //std::cout << "mid_abi_grid_ub[" << i << "] = " << mid_abi_grid_ub[i] << std::endl;
        //std::cout << "mid_abi_grid_ib[" << i << "] = " << mid_abi_grid_ib[i] << std::endl;
    }

    prob_abi_em[0] = norm_cdf(mid_abi_grid_em[0], 0, ABI_EM_STD);
    for (int i = 1; i < ABI - 1; i++) {
        prob_abi_em[i] = norm_cdf(mid_abi_grid_em[i], 0, ABI_EM_STD) - norm_cdf(mid_abi_grid_em[i-1], 0, ABI_EM_STD);
    }
    prob_abi_em[ABI-1] = 1 - norm_cdf(mid_abi_grid_em[ABI-2], 0, ABI_EM_STD);

    prob_abi_ub[0] = norm_cdf(mid_abi_grid_ub[0], 0, ABI_UB_STD);
    for (int i = 1; i < ABI - 1; i++) {
        prob_abi_ub[i] = norm_cdf(mid_abi_grid_ub[i], 0, ABI_UB_STD) - norm_cdf(mid_abi_grid_ub[i-1], 0, ABI_UB_STD);
    }
    prob_abi_ub[ABI-1] = 1 - norm_cdf(mid_abi_grid_ub[ABI-2], 0, ABI_UB_STD);

    prob_abi_ib[0] = norm_cdf(mid_abi_grid_ib[0], 0, ABI_IB_STD);
    for (int i = 1; i < ABI - 1; i++) {
        prob_abi_ib[i] = norm_cdf(mid_abi_grid_ib[i], 0, ABI_IB_STD) - norm_cdf(mid_abi_grid_ib[i-1], 0, ABI_IB_STD);
    }
    prob_abi_ib[ABI-1] = 1 - norm_cdf(mid_abi_grid_ib[ABI-2], 0, ABI_IB_STD);

    //    for (int i = 0; i < ABI; i++){
    //        std::cout << "prob_abi_em[" << i << "] = " << prob_abi_em[i] << std::endl;
    //        std::cout << "prob_abi_ub[" << i << "] = " << prob_abi_ub[i] << std::endl;
    //        std::cout << "prob_abi_ib[" << i << "] = " << prob_abi_ib[i] << std::endl;
    //    }

    // shock grid
    // wage and investment
    // consumption value of non-incorporated and incorporated
    // consumption value of elite and non-elite college
    double shock_grid_w[EDU][SHOCK];
    double shock_grid_n[SHOCK];
    double shock_grid_i[SHOCK];
    double consump_grid_n[SHOCK];
    double consump_grid_i[SHOCK];
    double consump_grid_c[SHOCK];
    double consump_grid_e[SHOCK];

    for (int i = 0; i < SHOCK; i++) {
        shock_grid_w[0][i] = -RANGE_SHOCK * W_H_STD + 2 * RANGE_SHOCK * W_H_STD * i / (SHOCK - 1);
        shock_grid_w[1][i] = -RANGE_SHOCK * W_C_STD + 2 * RANGE_SHOCK * W_C_STD * i / (SHOCK - 1);
        shock_grid_w[2][i] = -RANGE_SHOCK * W_E_STD + 2 * RANGE_SHOCK * W_E_STD * i / (SHOCK - 1);
        shock_grid_n[i] = -RANGE_SHOCK * N_STD + 2 * RANGE_SHOCK * N_STD * i / (SHOCK - 1);
        shock_grid_i[i] = -RANGE_SHOCK * I_STD + 2 * RANGE_SHOCK * I_STD * i / (SHOCK - 1);
        consump_grid_n[i] = -RANGE_SHOCK * CONSUMP_N_STD + 2 * RANGE_SHOCK * CONSUMP_N_STD * i / (SHOCK - 1);
        consump_grid_i[i] = -RANGE_SHOCK * CONSUMP_I_STD + 2 * RANGE_SHOCK * CONSUMP_I_STD * i / (SHOCK - 1);
        consump_grid_c[i] = -RANGE_SHOCK * CONSUMP_C_STD + 2 * RANGE_SHOCK * CONSUMP_C_STD * i / (SHOCK - 1);
        consump_grid_e[i] = -RANGE_SHOCK * CONSUMP_E_STD + 2 * RANGE_SHOCK * CONSUMP_E_STD * i / (SHOCK - 1);
        //std::cout << "consump_grid_n[" << i << "] = " << consump_grid_n[i] << std::endl;
        //std::cout << "shock_grid_i[" << i << "] = " << shock_grid_i[i] << std::endl;
    }

    double mid_shock_grid_w[EDU][SHOCK - 1];
    double mid_shock_grid_n[SHOCK - 1];
    double mid_shock_grid_i[SHOCK - 1];
    double mid_consump_grid_n[SHOCK - 1];
    double mid_consump_grid_i[SHOCK - 1];
    double mid_consump_grid_c[SHOCK - 1];
    double mid_consump_grid_e[SHOCK - 1];
    double prob_shock_w[EDU][SHOCK];
    double prob_shock_n[SHOCK];
    double prob_shock_i[SHOCK];
    double prob_consump_n[SHOCK];
    double prob_consump_i[SHOCK];
    double prob_consump_c[SHOCK];
    double prob_consump_e[SHOCK];

    for (int i = 0; i < SHOCK - 1; i++) {
        mid_shock_grid_w[0][i] = (shock_grid_w[0][i] + shock_grid_w[0][i + 1]) / 2;
        mid_shock_grid_w[1][i] = (shock_grid_w[1][i] + shock_grid_w[1][i + 1]) / 2;
        mid_shock_grid_w[2][i] = (shock_grid_w[2][i] + shock_grid_w[2][i + 1]) / 2;
        mid_shock_grid_n[i] = (shock_grid_n[i] + shock_grid_n[i + 1]) / 2;
        mid_shock_grid_i[i] = (shock_grid_i[i] + shock_grid_i[i + 1]) / 2;
        mid_consump_grid_n[i] = (consump_grid_n[i] + consump_grid_n[i + 1]) / 2;
        mid_consump_grid_i[i] = (consump_grid_i[i] + consump_grid_i[i + 1]) / 2;
        mid_consump_grid_c[i] = (consump_grid_n[i] + consump_grid_n[i + 1]) / 2;
        mid_consump_grid_e[i] = (consump_grid_i[i] + consump_grid_i[i + 1]) / 2;
        //std::cout << "mid_shock_grid_w[" << i << "] = " << mid_shock_grid_w[i] << std::endl;
        //std::cout << "mid_shock_grid_i[" << i << "] = " << mid_shock_grid_i[i] << std::endl;
    }

    prob_shock_w[0][0] = norm_cdf(mid_shock_grid_w[0][0], 0, W_H_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_shock_w[0][i] = norm_cdf(mid_shock_grid_w[0][i], 0, W_H_STD) - norm_cdf(mid_shock_grid_w[0][i-1], 0, W_H_STD);
    }
    prob_shock_w[0][SHOCK-1] = 1 - norm_cdf(mid_shock_grid_w[0][SHOCK-2], 0, W_H_STD);

    prob_shock_w[1][0] = norm_cdf(mid_shock_grid_w[1][0], 0, W_C_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_shock_w[1][i] = norm_cdf(mid_shock_grid_w[1][i], 0, W_C_STD) - norm_cdf(mid_shock_grid_w[1][i-1], 0, W_C_STD);
    }
    prob_shock_w[1][SHOCK-1] = 1 - norm_cdf(mid_shock_grid_w[1][SHOCK-2], 0, W_C_STD);

    prob_shock_w[2][0] = norm_cdf(mid_shock_grid_w[2][0], 0, W_E_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_shock_w[2][i] = norm_cdf(mid_shock_grid_w[2][i], 0, W_E_STD) - norm_cdf(mid_shock_grid_w[2][i-1], 0, W_E_STD);
    }
    prob_shock_w[2][SHOCK-1] = 1 - norm_cdf(mid_shock_grid_w[2][SHOCK-2], 0, W_E_STD);

    prob_shock_n[0] = norm_cdf(mid_shock_grid_n[0], 0, N_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_shock_n[i] = norm_cdf(mid_shock_grid_n[i], 0, N_STD) - norm_cdf(mid_shock_grid_n[i-1], 0, N_STD);
    }
    prob_shock_n[SHOCK-1] = 1 - norm_cdf(mid_shock_grid_n[SHOCK-2], 0, N_STD);

    prob_shock_i[0] = norm_cdf(mid_shock_grid_i[0], 0, I_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_shock_i[i] = norm_cdf(mid_shock_grid_i[i], 0, I_STD) - norm_cdf(mid_shock_grid_i[i-1], 0, I_STD);
    }
    prob_shock_i[SHOCK-1] = 1 - norm_cdf(mid_shock_grid_i[SHOCK-2], 0, I_STD);

    prob_consump_n[0] = norm_cdf(mid_consump_grid_n[0], 0, CONSUMP_N_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_consump_n[i] = norm_cdf(mid_consump_grid_n[i], 0, CONSUMP_N_STD) - norm_cdf(mid_consump_grid_n[i-1], 0, CONSUMP_N_STD);
    }
    prob_consump_n[SHOCK-1] = 1 - norm_cdf(mid_consump_grid_n[SHOCK-2], 0, CONSUMP_N_STD);

    prob_consump_i[0] = norm_cdf(mid_consump_grid_i[0], 0, CONSUMP_I_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_consump_i[i] = norm_cdf(mid_consump_grid_i[i], 0, CONSUMP_I_STD) - norm_cdf(mid_consump_grid_i[i-1], 0, CONSUMP_I_STD);
    }
    prob_consump_i[SHOCK-1] = 1 - norm_cdf(mid_consump_grid_i[SHOCK-2], 0, CONSUMP_I_STD);

    prob_consump_c[0] = norm_cdf(mid_consump_grid_c[0], 0, CONSUMP_C_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_consump_c[i] = norm_cdf(mid_consump_grid_c[i], 0, CONSUMP_C_STD) - norm_cdf(mid_consump_grid_c[i-1], 0, CONSUMP_C_STD);
    }
    prob_consump_c[SHOCK-1] = 1 - norm_cdf(mid_consump_grid_c[SHOCK-2], 0, CONSUMP_C_STD);

    prob_consump_e[0] = norm_cdf(mid_consump_grid_e[0], 0, CONSUMP_E_STD);
    for (int i = 1; i < SHOCK - 1; i++) {
        prob_consump_e[i] = norm_cdf(mid_consump_grid_e[i], 0, CONSUMP_E_STD) - norm_cdf(mid_consump_grid_e[i-1], 0, CONSUMP_E_STD);
    }
    prob_consump_e[SHOCK-1] = 1 - norm_cdf(mid_consump_grid_e[SHOCK-2], 0, CONSUMP_E_STD);

    //for (int i = 0; i < SHOCK; i++){
    //std::cout << "prob_shock_n[" << i << "] = " << prob_shock_w[i] << std::endl;
    //std::cout << "prob_shock_i[" << i << "] = " << prob_shock_i[i] << std::endl;
    //}

    // ability intergenerational transition probability
    double tranprob_abi_em[ABI][ABI];
    double tranprob_abi_ub[ABI][ABI];
    double tranprob_abi_ib[ABI][ABI];
    for (int i = 0; i < ABI; i++){
        tranprob_abi_em[i][0] = norm_cdf(mid_abi_grid_em[0] - THETA_EM * mid_abi_grid_em[i], 0, ABI_EM_STD);
        tranprob_abi_em[i][ABI-1] = 1 - norm_cdf(mid_abi_grid_em[ABI-2] - THETA_EM * mid_abi_grid_em[i], 0, ABI_EM_STD);
        tranprob_abi_ub[i][0] = norm_cdf(mid_abi_grid_ub[0] - THETA_UB * mid_abi_grid_ub[i], 0, ABI_UB_STD);
        tranprob_abi_ub[i][ABI-1] = 1 - norm_cdf(mid_abi_grid_ub[ABI-2] - THETA_UB * mid_abi_grid_ub[i], 0, ABI_UB_STD);
        tranprob_abi_ib[i][0] = norm_cdf(mid_abi_grid_ib[0] - THETA_IB * mid_abi_grid_ib[i], 0, ABI_IB_STD);
        tranprob_abi_ib[i][ABI-1] = 1 - norm_cdf(mid_abi_grid_ib[ABI-2] - THETA_IB * mid_abi_grid_ib[i], 0, ABI_IB_STD);
        for (int j = 1; j < ABI - 1; j++) {
            tranprob_abi_em[i][j] = norm_cdf(mid_abi_grid_em[j] - THETA_EM * mid_abi_grid_em[i], 0, ABI_EM_STD) - norm_cdf(mid_abi_grid_em[j - 1] - THETA_EM * mid_abi_grid_em[i], 0, ABI_EM_STD);
            tranprob_abi_ub[i][j] = norm_cdf(mid_abi_grid_ub[j] - THETA_UB * mid_abi_grid_ub[i], 0, ABI_UB_STD) - norm_cdf(mid_abi_grid_ub[j - 1] - THETA_UB * mid_abi_grid_ub[i], 0, ABI_UB_STD);
            tranprob_abi_ib[i][j] = norm_cdf(mid_abi_grid_ib[j] - THETA_IB * mid_abi_grid_ib[i], 0, ABI_IB_STD) - norm_cdf(mid_abi_grid_ib[j - 1] - THETA_IB * mid_abi_grid_ib[i], 0, ABI_IB_STD);
        }
    }


    //    for (int i = 0; i < ABI; i++){
    //        for (int j = 0; j < ABI; j++){
    //            std::cout << "tranprob_abi_w[" << i << "][" << j << "] = " << tranprob_abi_w[i][j] << std::endl;
    //            std::cout << "tranprob_abi_e[" << i << "][" << j << "] = " << tranprob_abi_e[i][j] << std::endl;
    //        }
    //    }

    // value function
    vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<double> > > > > > > > > > value_retire(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > > > career_retire(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > > > > capitalchoice_retire(ITER);

    vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<double> > > > > > > > > > value(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > > > career(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > > > > capitalchoice(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > > > transfer(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<double> > > > > > > > value_worker_noshock(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<double> > > > > > > > value_non_incorp_noshock(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<double> > > > > > > > value_incorp_noshock(ITER);

    vector<vector<vector<vector<vector<vector<vector<vector<double> > > > > > > > schoolvalue(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > schoolchoice(ITER);
    vector<vector<vector<vector<vector<vector<vector<vector<short> > > > > > > > schoolcapital(ITER);
    vector<vector<vector<vector<vector<vector<double> > > > > > value_non_elite_noshock(ITER);
    vector<vector<vector<vector<vector<vector<double> > > > > > value_elite_noshock(ITER);

    for(int iter = 0; iter < ITER; iter++) {
        std::cout << "Iteration : " << iter << std::endl;

        value_retire[iter].resize(AGE-RETIRE_AGE);
        career_retire[iter].resize(AGE-RETIRE_AGE);
        capitalchoice_retire[iter].resize(AGE-RETIRE_AGE);

        std::cout << "Processing Retirement age" << std::endl;
        for(int t = AGE-1; t >= RETIRE_AGE; t--) {
            value_retire[iter][t-RETIRE_AGE].resize(EDU);
            career_retire[iter][t-RETIRE_AGE].resize(EDU);
            capitalchoice_retire[iter][t-RETIRE_AGE].resize(EDU);
            for(int e = 0; e < EDU; e++) {
                value_retire[iter][t-RETIRE_AGE][e].resize(ABI);
                career_retire[iter][t-RETIRE_AGE][e].resize(ABI);
                capitalchoice_retire[iter][t-RETIRE_AGE][e].resize(ABI);
                for(int a_em = 0; a_em < ABI; a_em++) {
                    value_retire[iter][t-RETIRE_AGE][e][a_em].resize(ABI);
                    career_retire[iter][t-RETIRE_AGE][e][a_em].resize(ABI);
                    capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em].resize(ABI);
                    for(int a_ub = 0; a_ub < ABI; a_ub++) {
                        value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub].resize(ABI);
                        career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub].resize(ABI);
                        capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub].resize(ABI);
                        for(int a_ib = 0; a_ib < ABI; a_ib++) {
                            value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE_RETIRE);
                            career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE_RETIRE);
                            capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE_RETIRE);
                            for(int exp_bus = 0; exp_bus < EXP_BUS_TYPE_RETIRE; exp_bus++) {
                                value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                for(int k = 0; k < CAP; k++) {
                                    value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k].resize(SHOCK);
                                    career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k].resize(SHOCK);
                                    capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k].resize(SHOCK);
                                    for(int consump_n = 0; consump_n < SHOCK; consump_n++) {
                                        value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n].resize(SHOCK);
                                        career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n].resize(SHOCK);
                                        capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n].resize(SHOCK);
                                        for(int consump_i = 0; consump_i < SHOCK; consump_i++) {
                                            capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i].resize(SHOCK);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        for(int t = AGE-1; t >= RETIRE_AGE; t--) {
            std::cout << "Processing age = " << age(t) << std::endl;
#pragma omp parallel for collapse(4)
            for(int e = 0; e < EDU; e++) {
                for(int a_em = 0; a_em < ABI; a_em++) {
                    for(int a_ub = 0; a_ub < ABI; a_ub++) {
                        for(int a_ib = 0; a_ib < ABI; a_ib++) {
                            for(int exp_bus = 0; exp_bus < EXP_BUS_TYPE_RETIRE; exp_bus++) {
                                vector<double> futurevalue_retire0(CAP, 0);
                                vector<double> futurevalue_retire1(CAP, 0);
                                vector<double> futurevalue_retire2(CAP, 0);
                                for (int k_next_idx = 0; k_next_idx < CAP; k_next_idx++){
                                    if (t < AGE - 1){
                                        for(int consump_n_next = 0; consump_n_next < SHOCK; consump_n_next++) {
                                            for(int consump_i_next = 0; consump_i_next < SHOCK; consump_i_next++) {
                                                futurevalue_retire0[k_next_idx] += BETA * ZETA[t-RETIRE_AGE] * value_retire[iter][t-RETIRE_AGE+1][e][a_em][a_ub][a_ib][0][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                                futurevalue_retire1[k_next_idx] += BETA * ZETA[t-RETIRE_AGE] * value_retire[iter][t-RETIRE_AGE+1][e][a_em][a_ub][a_ib][1][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                                futurevalue_retire2[k_next_idx] += BETA * ZETA[t-RETIRE_AGE] * value_retire[iter][t-RETIRE_AGE+1][e][a_em][a_ub][a_ib][2][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                            }
                                        }
                                    }
                                }
                                for(int k = 0; k < CAP; k++) {
                                    // worker's decision
                                    double value_worker = -9999;
                                    double capitalchoice_worker = -1;
                                    double k_worker = capital(k) * (1 + R) + P * W * h_em(a_em, e) * exp(4 * ALPHA_1[e] + 22.7 * ALPHA_2[e]);
                                    if (t == AGE - 1){
                                        value_worker = util(k_worker);
                                        capitalchoice_worker = 4; // capital = 0
                                    } else {
                                        for(int k_next = 0; k_next <= k_worker && k_next < capital(CAP); k_next += CAP_STEP) {
                                            double value_tmp = util(k_worker - k_next) + futurevalue_retire0[(k_next - CAP_MIN)/CAP_STEP];
                                            if (value_worker < value_tmp) {
                                                value_worker = value_tmp;
                                                capitalchoice_worker = (k_next - CAP_MIN)/CAP_STEP;
                                            }
                                        }
                                    }


                                    // non-incorporated business owner decision
                                    double value_non_incorp = 0;
                                    double value_non_incorp_tmp;
                                    vector<double> capitalchoice_non_incorp(SHOCK);
                                    // only able to run a business when there is no debt
                                    if (capital(k) >= 0){
                                        for(int shock_n = 0; shock_n < SHOCK; shock_n++) {
                                            value_non_incorp_tmp = -9999;
                                            capitalchoice_non_incorp[shock_n] = -1;
                                            double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                                            double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                                            double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                                            // liquidity constraint
                                            if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                                                invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                                            double k_non_incorp = (1 - DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) + (capital(k) - invest_non_incorp) * (1 + R);
                                            if (t == AGE - 1){
                                                value_non_incorp_tmp = util(k_non_incorp);
                                                capitalchoice_non_incorp[shock_n] = 4;
                                            } else {
                                                for(int k_next = 0; k_next <= k_non_incorp && k_next < capital(CAP); k_next += CAP_STEP) {
                                                    double value_tmp = util(k_non_incorp - k_next) + futurevalue_retire1[(k_next - CAP_MIN)/CAP_STEP];
                                                    if (value_non_incorp_tmp < value_tmp) {
                                                        value_non_incorp_tmp = value_tmp;
                                                        capitalchoice_non_incorp[shock_n] = (k_next - CAP_MIN)/CAP_STEP;
                                                    }
                                                }
                                            }
                                            value_non_incorp += value_non_incorp_tmp * prob_shock_n[shock_n];
                                        }
                                    } else {
                                        value_non_incorp = -999;
                                    }

                                    // incorporated business owner decision
                                    double value_incorp = 0;
                                    double value_incorp_tmp;
                                    vector<double> capitalchoice_incorp(SHOCK);
                                    // only able to run a business when there is no debt
                                    if (capital(k) >= 0){
                                        for(int shock_i = 0; shock_i < SHOCK; shock_i++) {
                                            value_incorp_tmp = -9999;
                                            capitalchoice_incorp[shock_i] = -1;
                                            double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                                            double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                                            double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                                            // liquidity constraint
                                            if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                                                invest_incorp = (1 + LAMBDA_E) * capital(k);
                                            double k_incorp = (1 - DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - I_COST + (capital(k) - invest_incorp) * (1 + R);
                                            if (exp_bus == 2) {
                                                k_incorp = (1 - DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) + (capital(k) - invest_incorp) * (1 + R);
                                            }
                                            if (t == AGE - 1){
                                                value_incorp_tmp = util(k_incorp);
                                                capitalchoice_incorp[shock_i] = 4;
                                            } else {
                                                for(int k_next = 0; k_next <= k_incorp && k_next < capital(CAP); k_next += CAP_STEP) {
                                                    double value_tmp = util(k_incorp - k_next) + futurevalue_retire2[(k_next - CAP_MIN)/CAP_STEP];
                                                    if (value_incorp_tmp < value_tmp) {
                                                        value_incorp_tmp = value_tmp;
                                                        capitalchoice_incorp[shock_i] = (k_next - CAP_MIN)/CAP_STEP;
                                                    }
                                                }
                                            }
                                            value_incorp += value_incorp_tmp * prob_shock_i[shock_i];
                                        }
                                    } else {
                                        value_incorp = -999;
                                    }

                                    for(int consump_n = 0; consump_n < SHOCK; consump_n++){
                                        for (int consump_i = 0; consump_i < SHOCK; consump_i++) {
                                            if (exp_bus > 0){
                                                if (value_worker > value_non_incorp + consump_grid_n[consump_n] && value_worker > value_incorp + consump_grid_i[consump_i]) {
                                                    value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_worker;
                                                    career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = WORKER;
                                                    for (int shock_w = 0; shock_w < SHOCK; shock_w++) {
                                                        capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w] = capitalchoice_worker;
                                                    }
                                                }
                                                else if (value_non_incorp + consump_grid_n[consump_n] >= value_incorp + consump_grid_i[consump_i]) {
                                                    value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_non_incorp + consump_grid_n[consump_n];
                                                    career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = NON_INCORP;
                                                    for (int shock_n = 0; shock_n < SHOCK; shock_n++) {
                                                        capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n] = capitalchoice_non_incorp[shock_n];
                                                    }
                                                }
                                                else {
                                                    value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_incorp + consump_grid_i[consump_i];
                                                    career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = INCORP;
                                                    for (int shock_i = 0; shock_i < SHOCK; shock_i++) {
                                                        capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i] = capitalchoice_incorp[shock_i];
                                                    }
                                                }
                                            }
                                            else{
                                                value_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_worker;
                                                career_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = WORKER;
                                                for (int shock_w = 0; shock_w < SHOCK; shock_w++) {
                                                    capitalchoice_retire[iter][t-RETIRE_AGE][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w] = capitalchoice_worker;
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        /*fout << "high school value function at age 100" << std::endl;
         for(int k = 0; k < 20; k++) {
         fout << "valuefunction_retire[" << 0 << "][" << 7 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << value_retire[0][7][0][3][3][0][k][4][4] << std::endl;
         fout << "capitalchoice_retire[" << 0 << "][" << 7 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << capitalchoice_retire[0][7][0][3][3][0][k][4][4][4] << std::endl;
         fout << "careerchoice_retire[" << 0 << "][" << 7 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << career_retire[0][7][0][3][3][0][k][4][4] << std::endl;
         }

         fout << "college value function at age 100" << std::endl;
         for(int k = 0; k < 20; k++) {
         fout << "valuefunction_retire[" << 0 << "][" << 7 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << value_retire[0][7][1][3][3][0][k][4][4] << std::endl;
         fout << "capitalchoice_retire[" << 0 << "][" << 7 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << capitalchoice_retire[0][7][1][3][3][0][k][4][4][4] << std::endl;
         fout << "careerchoice_retire[" << 0 << "][" << 7 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << career_retire[0][7][1][3][3][0][k][4][4] << std::endl;
         }

         fout << "elite college value function at age 100" << std::endl;
         for(int k = 0; k < 20; k++) {
         fout << "valuefunction[" << 0 << "][" << 7 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << value_retire[0][7][2][3][3][0][k][4][4] << std::endl;
         fout << "capitalchoice_retire[" << 0 << "][" << 7 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << capitalchoice_retire[0][7][2][3][3][0][k][4][4][4] << std::endl;
         fout << "careerchoice_retire[" << 0 << "][" << 7 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << career_retire[0][7][2][3][3][0][k][4][4] << std::endl;
         }

         fout << "high school value function and policy function at age 65" << std::endl;
         for(int k = 0; k < 30; k++) {
         fout << "valuefunction_retire[" << 0 << "][" << 0 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << value_retire[0][0][0][3][3][0][k][4][4] << std::endl;
         fout << "capitalchoice_retire[" << 0 << "][" << 0 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << capitalchoice_retire[0][0][0][3][3][0][k][4][4][4] << std::endl;
         fout << "careerchoice_retire[" << 0 << "][" << 0 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << career_retire[0][0][0][3][3][0][k][4][4] << std::endl;
         }

         fout << "college value function and policy function at age 65" << std::endl;
         for(int k = 0; k < 30; k++) {
         fout << "valuefunction_retire[" << 0 << "][" << 0 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << value_retire[0][0][1][3][3][0][k][4][4] << std::endl;
         fout << "capitalchoice_retire[" << 0 << "][" << 0 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << capitalchoice_retire[0][0][1][3][3][0][k][4][4][4] << std::endl;
         fout << "careerchoice_retire[" << 0 << "][" << 0 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << career_retire[0][0][1][3][3][0][k][4][4] << std::endl;
         }

         fout << "elite college value function and policy function at age 65" << std::endl;
         for(int k = 0; k < 30; k++) {
         fout << "valuefunction_retire[" << 0 << "][" << 0 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << value_retire[0][0][2][3][3][0][k][4][4] << std::endl;
         fout << "capitalchoice_retire[" << 0 << "][" << 0 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << capitalchoice_retire[0][0][2][3][3][0][k][4][4][4] << std::endl;
         fout << "careerchoice_retire[" << 0 << "][" << 0 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "][" << 4 << "] = " << career_retire[0][0][2][3][3][0][k][4][4] << std::endl;
         }*/


        std::cout << "Processing Working age" << std::endl;
        value[iter].resize(RETIRE_AGE);
        career[iter].resize(RETIRE_AGE);
        capitalchoice[iter].resize(RETIRE_AGE);
        value_worker_noshock[iter].resize(RETIRE_AGE);
        value_non_incorp_noshock[iter].resize(RETIRE_AGE);
        value_incorp_noshock[iter].resize(RETIRE_AGE);
        for(int t = RETIRE_AGE-1; t >= 0; t--) {
            value[iter][t].resize(EDU);
            career[iter][t].resize(EDU);
            capitalchoice[iter][t].resize(EDU);
            value_worker_noshock[iter][t].resize(EDU);
            value_non_incorp_noshock[iter][t].resize(EDU);
            value_incorp_noshock[iter][t].resize(EDU);
            for(int e = 0; e < EDU; e++) {
                value[iter][t][e].resize(ABI);
                career[iter][t][e].resize(ABI);
                capitalchoice[iter][t][e].resize(ABI);
                value_worker_noshock[iter][t][e].resize(ABI);
                value_non_incorp_noshock[iter][t][e].resize(ABI);
                value_incorp_noshock[iter][t][e].resize(ABI);
                for(int a_em = 0; a_em < ABI; a_em++) {
                    value[iter][t][e][a_em].resize(ABI);
                    career[iter][t][e][a_em].resize(ABI);
                    capitalchoice[iter][t][e][a_em].resize(ABI);
                    value_worker_noshock[iter][t][e][a_em].resize(ABI);
                    value_non_incorp_noshock[iter][t][e][a_em].resize(ABI);
                    value_incorp_noshock[iter][t][e][a_em].resize(ABI);
                    for(int a_ub = 0; a_ub < ABI; a_ub++) {
                        value[iter][t][e][a_em][a_ub].resize(ABI);
                        career[iter][t][e][a_em][a_ub].resize(ABI);
                        capitalchoice[iter][t][e][a_em][a_ub].resize(ABI);
                        value_worker_noshock[iter][t][e][a_em][a_ub].resize(ABI);
                        value_non_incorp_noshock[iter][t][e][a_em][a_ub].resize(ABI);
                        value_incorp_noshock[iter][t][e][a_em][a_ub].resize(ABI);
                        for(int a_ib = 0; a_ib < ABI; a_ib++) {
                            value[iter][t][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE);
                            career[iter][t][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE);
                            capitalchoice[iter][t][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE);
                            value_worker_noshock[iter][t][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE);
                            value_non_incorp_noshock[iter][t][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE);
                            value_incorp_noshock[iter][t][e][a_em][a_ub][a_ib].resize(EXP_BUS_TYPE);
                            for(int exp_bus = 0; exp_bus < EXP_BUS_TYPE; exp_bus++) {
                                value[iter][t][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                career[iter][t][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                value_worker_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                value_non_incorp_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                value_incorp_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                                for(int k = 0; k < CAP; k++) {
                                    value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k].resize(SHOCK);
                                    career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k].resize(SHOCK);
                                    capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k].resize(SHOCK);
                                    for(int consump_n = 0; consump_n < SHOCK; consump_n++) {
                                        value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n].resize(SHOCK);
                                        career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n].resize(SHOCK);
                                        capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n].resize(SHOCK);
                                        for(int consump_i = 0; consump_i < SHOCK; consump_i++) {
                                            capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i].resize(SHOCK);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        for(int t = RETIRE_AGE-1; t >= 0; t--) {
            std::cout << "Processing age = " << age(t) << std::endl;
#pragma omp parallel for collapse(4)
            for(int e = 0; e < EDU; e++) {
                for(int a_em = 0; a_em < ABI; a_em++) {
                    for(int a_ub = 0; a_ub < ABI; a_ub++) {
                        for(int a_ib = 0; a_ib < ABI; a_ib++) {
                            for(int exp_bus = 0; exp_bus < EXP_BUS_TYPE; exp_bus++) {
                                vector<double> futurevalue0(CAP, 0);
                                vector<double> futurevalue1(CAP, 0);
                                vector<double> futurevalue_retire0(CAP, 0);
                                vector<double> futurevalue_retire1(CAP, 0);
                                vector<double> futurevalue_retire2(CAP, 0);
                                for (int k_next_idx = 0; k_next_idx < CAP; k_next_idx++){
                                    if (t == RETIRE_AGE - 1){
                                        for(int consump_n_next = 0; consump_n_next < SHOCK; consump_n_next++) {
                                            for(int consump_i_next = 0; consump_i_next < SHOCK; consump_i_next++) {
                                                futurevalue_retire0[k_next_idx] += BETA * value_retire[iter][0][e][a_em][a_ub][a_ib][0][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                                futurevalue_retire1[k_next_idx] += BETA * value_retire[iter][0][e][a_em][a_ub][a_ib][1][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                                futurevalue_retire2[k_next_idx] += BETA * value_retire[iter][0][e][a_em][a_ub][a_ib][2][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                            }
                                        }
                                    } else {
                                        for(int consump_n_next = 0; consump_n_next < SHOCK; consump_n_next++) {
                                            for(int consump_i_next = 0; consump_i_next < SHOCK; consump_i_next++) {
                                                futurevalue0[k_next_idx] += BETA * value[iter][t+1][e][a_em][a_ub][a_ib][0][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                                futurevalue1[k_next_idx] += BETA * value[iter][t+1][e][a_em][a_ub][a_ib][1][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                            }
                                        }
                                    }
                                }

                                for(int k = 0; k < CAP; k++) {
                                    // worker's decision
                                    double value_worker = 0;
                                    double value_worker_tmp;
                                    vector<double> capitalchoice_worker(SHOCK);
                                    for(int shock_w = 0; shock_w < SHOCK; shock_w++) {
                                        value_worker_tmp = -9999;
                                        capitalchoice_worker[shock_w] = -1;
                                        double k_worker = capital(k) * (1 + R) + W * h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e) + shock_grid_w[e][shock_w]);
                                        for(int k_next = 0; k_next <= k_worker && k_next < capital(CAP); k_next += CAP_STEP) {
                                            double value_tmp = util(k_worker - k_next);
                                            if (t == RETIRE_AGE - 1) {
                                                value_tmp += futurevalue_retire0[(k_next - CAP_MIN)/CAP_STEP];
                                            }
                                            else {
                                                value_tmp += futurevalue0[(k_next - CAP_MIN)/CAP_STEP];
                                            }
                                            if (value_worker_tmp < value_tmp) {
                                                value_worker_tmp = value_tmp;
                                                capitalchoice_worker[shock_w] = (k_next - CAP_MIN)/CAP_STEP;
                                            }
                                        }
                                        value_worker += value_worker_tmp * prob_shock_w[e][shock_w];
                                        //if (t == 4 && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 14 ){
                                        //    std::cout << t << ", " << k << ", " << k_worker << ", " << h_w(a_em, e) * exp(ALPHA_0 + ALPHA_1[e] * exp_w(t, e) + ALPHA_2[e] * exp_w(t, e) * exp_w(t, e) + shock_grid_w[e][shock_w]) << ", " << capital(capitalchoice_worker[shock_w]) << ", " << util(k_worker - capital(capitalchoice_worker[shock_w])) << ", " << futurevalue0[capitalchoice_worker[shock_w]] << ", " << value_worker_tmp << ", " << value_worker << std::endl;
                                        //}

                                    }

                                    // non-incorporated business owner decision
                                    double value_non_incorp = 0;
                                    double value_non_incorp_tmp;
                                    vector<double> capitalchoice_non_incorp(SHOCK);
                                    // only able to run a business when there is no debt
                                    if (capital(k) >= 0){
                                        for(int shock_n = 0; shock_n < SHOCK; shock_n++) {
                                            value_non_incorp_tmp = -9999;
                                            capitalchoice_non_incorp[shock_n] = -1;
                                            double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                                            double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                                            double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                                            // liquidity constraint
                                            if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                                                invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                                            double k_non_incorp = (1 - DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) + (capital(k) - invest_non_incorp) * (1 + R);
                                            for(int k_next = 0; k_next <= k_non_incorp && k_next < capital(CAP); k_next += CAP_STEP) {
                                                double value_tmp = util(k_non_incorp - k_next);
                                                if (t == RETIRE_AGE - 1) {
                                                    value_tmp += futurevalue_retire1[(k_next - CAP_MIN)/CAP_STEP];
                                                }
                                                else {
                                                    value_tmp += futurevalue0[(k_next - CAP_MIN)/CAP_STEP];
                                                }
                                                if (value_non_incorp_tmp < value_tmp) {
                                                    value_non_incorp_tmp = value_tmp;
                                                    capitalchoice_non_incorp[shock_n] = (k_next - CAP_MIN)/CAP_STEP;
                                                }
                                            }
                                            value_non_incorp += value_non_incorp_tmp * prob_shock_n[shock_n];
                                            //if (t == 4 && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 14 ){
                                            //    std::cout << t << ", " << k << ", " << i_scaler_non_incorp << ", " << invest_non_incorp << ", " << k_non_incorp << ", " << i_scaler_non_incorp * pow(invest_non_incorp, NU) - N_COST - invest_non_incorp * (1 + R) << ", " << capital(capitalchoice_non_incorp[shock_n]) << ", " << util(k_non_incorp - capital(capitalchoice_non_incorp[shock_n])) << ", " << futurevalue1[capitalchoice_non_incorp[shock_n]] << ", " << value_non_incorp_tmp << ", " << value_non_incorp << std::endl;
                                            //}
                                        }
                                    } else {
                                        value_non_incorp = -999;
                                    }

                                    // incorporated business owner decision
                                    double value_incorp = 0;
                                    double value_incorp_tmp;
                                    vector<double> capitalchoice_incorp(SHOCK);
                                    // only able to run a business when there is no debt
                                    if (capital(k) >= 0){
                                        for(int shock_i = 0; shock_i < SHOCK; shock_i++) {
                                            value_incorp_tmp = -9999;
                                            capitalchoice_incorp[shock_i] = -1;
                                            double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                                            double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                                            double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                                            // liquidity constraint
                                            if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                                                invest_incorp = (1 + LAMBDA_E) * capital(k);
                                            double k_incorp = (1 - DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - I_COST + (capital(k) - invest_incorp) * (1 + R);
                                            if (exp_bus == 1) {
                                                k_incorp = (1 - DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) + (capital(k) - invest_incorp) * (1 + R);
                                            }
                                            for(int k_next = 0; k_next <= k_incorp && k_next < capital(CAP); k_next += CAP_STEP) {
                                                double value_tmp = util(k_incorp - k_next);
                                                if (t == RETIRE_AGE - 1) {
                                                    value_tmp += futurevalue_retire2[(k_next - CAP_MIN)/CAP_STEP];
                                                }
                                                else {
                                                    value_tmp += futurevalue1[(k_next - CAP_MIN)/CAP_STEP];
                                                }
                                                if (value_incorp_tmp < value_tmp) {
                                                    value_incorp_tmp = value_tmp;
                                                    capitalchoice_incorp[shock_i] = (k_next - CAP_MIN)/CAP_STEP;
                                                }
                                            }
                                            value_incorp += value_incorp_tmp * prob_shock_i[shock_i];
                                        }
                                    } else {
                                        value_incorp = -999;
                                    }

                                    /*if (k > 10 && k < 30 && exp_bus == 0) {
                                     fout << "t = " << t << ", e = " << e << ", a_em = " << a_em << ", a_ub = " << a_ub << ", k = " << k << "value_worker = " << value_worker << ", value_non_incorp = " << value_non_incorp << ", value_incorp =" << value_incorp << std::endl;
                                     }*/

                                    value_worker_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus][k] = value_worker;
                                    value_non_incorp_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus][k] = value_non_incorp;
                                    value_incorp_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus][k] = value_incorp;

                                    for(int consump_n = 0; consump_n < SHOCK; consump_n++) {
                                        for(int consump_i = 0; consump_i < SHOCK; consump_i++) {
                                            //if (t == 4  && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 14 && consump_n == 5 && consump_i == 2){
                                            //    std::cout << t << ", " << k << ", " << exp_bus << ",  " << value_worker << ", " << value_non_incorp + consump_grid_n[consump_n] << ", " << value_incorp + consump_grid_n[consump_n] + consump_grid_i[consump_i] << std::endl;
                                            //}
                                            if (value_worker > value_non_incorp + consump_grid_n[consump_n] && value_worker > value_incorp + consump_grid_i[consump_i]) {
                                                value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_worker;
                                                career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = WORKER;
                                                for (int shock_w = 0; shock_w < SHOCK; shock_w++) {
                                                    capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w] = capitalchoice_worker[shock_w];
                                                }
                                            }
                                            else if (value_non_incorp + consump_grid_n[consump_n] >= value_incorp + consump_grid_i[consump_i]) {
                                                value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_non_incorp + consump_grid_n[consump_n];
                                                career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = NON_INCORP;
                                                for (int shock_n = 0; shock_n < SHOCK; shock_n++) {
                                                    capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n] = capitalchoice_non_incorp[shock_n];
                                                }
                                            }
                                            else {
                                                value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_incorp + consump_grid_i[consump_i];
                                                career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = INCORP;
                                                for (int shock_i = 0; shock_i < SHOCK; shock_i++) {
                                                    capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i] = capitalchoice_incorp[shock_i];
                                                }
                                            }
                                            //if (t == 4 && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 14 && consump_n == 5 && consump_i == 2){
                                            //    std::cout << t << ", " << k << ", " << career[0][t][e][a_em][a_ub][exp_bus][k][consump_n][consump_i] << std::endl;
                                            //}
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        /*fout << "high school value function and policy function at age 35" << std::endl;
         for(int k = 0; k < 30; k++) {
         fout << "valuefunction[" << 0 << "][" << 4 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 1 << "][" << k << "][" << 4 << "][" << 4 << "] = " << value[0][3][0][3][3][1][k][4][4] << std::endl;
         fout << "careerchoice[" << 0 << "][" << 4 << "][" << 0 << "][" << 3 << "][" << 3 << "][" << 1 << "][" << k << "][" << 4 << "][" << 4 << "] = " << career[0][3][0][3][3][1][k][4][4] << std::endl;
         }


         fout << "college value function and policy function at age 35" << std::endl;
         for(int k = 0; k < 30; k++) {
         fout << "valuefunction[" << 0 << "][" << 3 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 1 << "][" << k << "][" << 4 << "][" << 4 << "] = " << value[0][3][1][3][3][1][k][4][4] << std::endl;
         fout << "careerchoice[" << 0 << "][" << 3 << "][" << 1 << "][" << 3 << "][" << 3 << "][" << 1 << "][" << k << "][" << 4 << "][" << 4 << "] = " << career[0][3][1][3][3][1][k][4][4] << std::endl;
         }

         fout << "elite college value function and policy function at age 35" << std::endl;
         for(int k = 0; k < 30; k++) {
         fout << "valuefunction[" << 0 << "][" << 3 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 1 << "][" << k << "][" << 4 << "][" << 4 << "] = " << value[0][3][2][3][3][1][k][4][4] << std::endl;
         fout << "careerchoice[" << 0 << "][" << 3 << "][" << 2 << "][" << 3 << "][" << 3 << "][" << 1 << "][" << k << "][" << 4 << "][" << 4 << "] = " << career[0][3][2][3][3][1][k][4][4] << std::endl;
         }*/


        std::cout << "start schooling decision" << std::endl;

        // schooling decision
        schoolchoice[iter].resize(ABI);
        schoolvalue[iter].resize(ABI);
        schoolcapital[iter].resize(ABI);
        value_non_elite_noshock[iter].resize(ABI);
        value_elite_noshock[iter].resize(ABI);

        for(int a_em = 0; a_em < ABI; a_em++) {
            schoolchoice[iter][a_em].resize(ABI);
            schoolvalue[iter][a_em].resize(ABI);
            schoolcapital[iter][a_em].resize(ABI);
            value_non_elite_noshock[iter][a_em].resize(ABI);
            value_elite_noshock[iter][a_em].resize(ABI);
            for(int a_ub = 0; a_ub < ABI; a_ub++) {
                schoolchoice[iter][a_em][a_ub].resize(ABI);
                schoolvalue[iter][a_em][a_ub].resize(ABI);
                schoolcapital[iter][a_em][a_ub].resize(ABI);
                value_non_elite_noshock[iter][a_em][a_ub].resize(ABI);
                value_elite_noshock[iter][a_em][a_ub].resize(ABI);
                for(int a_ib = 0; a_ib < ABI; a_ib++) {
                    schoolchoice[iter][a_em][a_ub][a_ib].resize(CAP);
                    schoolvalue[iter][a_em][a_ub][a_ib].resize(CAP);
                    schoolcapital[iter][a_em][a_ub][a_ib].resize(CAP);
                    value_non_elite_noshock[iter][a_em][a_ub][a_ib].resize(CAP);
                    value_elite_noshock[iter][a_em][a_ub][a_ib].resize(CAP);
                    for(int k = 0; k < CAP; k++) {
                        schoolchoice[iter][a_em][a_ub][a_ib][k].resize(CAP);
                        schoolvalue[iter][a_em][a_ub][a_ib][k].resize(CAP);
                        schoolcapital[iter][a_em][a_ub][a_ib][k].resize(CAP);
                        value_non_elite_noshock[iter][a_em][a_ub][a_ib][k].resize(CAP);
                        value_elite_noshock[iter][a_em][a_ub][a_ib][k].resize(CAP);
                        for(int k_fam = 0; k_fam < CAP; k_fam++) {
                            schoolchoice[iter][a_em][a_ub][a_ib][k][k_fam].resize(SHOCK);
                            schoolvalue[iter][a_em][a_ub][a_ib][k][k_fam].resize(SHOCK);
                            schoolcapital[iter][a_em][a_ub][a_ib][k][k_fam].resize(SHOCK);
                            for(int consump_c = 0; consump_c < SHOCK; consump_c++) {
                                schoolchoice[iter][a_em][a_ub][a_ib][k][k_fam][consump_c].resize(SHOCK);
                                schoolvalue[iter][a_em][a_ub][a_ib][k][k_fam][consump_c].resize(SHOCK);
                                schoolcapital[iter][a_em][a_ub][a_ib][k][k_fam][consump_c].resize(SHOCK);
                            }
                        }
                    }
                }
            }
        }

#pragma omp parallel for collapse(3)
        for(int a_em = 0; a_em < ABI; a_em++) {
            for(int a_ub = 0; a_ub < ABI; a_ub++) {
                for(int a_ib = 0; a_ib < ABI; a_ib++) {
                    vector<double> schoolfuture1(CAP, 0);
                    vector<double> schoolfuture2(CAP, 0);
                    for (int k_next_idx = 0; k_next_idx < CAP; k_next_idx++){
                        for(int consump_n_next = 0; consump_n_next < SHOCK; consump_n_next++) {
                            for(int consump_i_next = 0; consump_i_next < SHOCK; consump_i_next++) {
                                schoolfuture1[k_next_idx] += BETA * value[iter][1][1][a_em][a_ub][a_ib][0][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                schoolfuture2[k_next_idx] += BETA * value[iter][1][2][a_em][a_ub][a_ib][0][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                            }
                        }
                    }

                    for(int k = 0; k < CAP; k++) {
                        for(int k_fam = 0; k_fam < CAP; k_fam++) {
                            // high school edu = 0
                            double value_highschool = 0;
                            for(int consump_n = 0; consump_n < SHOCK; consump_n++) {
                                for(int consump_i = 0; consump_i < SHOCK; consump_i++) {
                                    value_highschool += value[iter][0][0][a_em][a_ub][a_ib][0][k][consump_n][consump_i] * prob_consump_n[consump_n] * prob_consump_i[consump_i];
                                }
                            }

                            // non-elite college edu = 1
                            double value_college = -9999;
                            double capitalchoice_college = -1;
                            double k_college = (1 + R) * (capital(k) - COLLEGE_TUITION + financial_aid_nonelite(k_fam, a_em));
                            for(int k_next = CAP_MIN; k_next <= k_college && k_next < capital(CAP); k_next += CAP_STEP) {
                                double value_tmp = util(k_college - k_next) + CONSUMPTION_COLLEGE + schoolfuture1[(k_next - CAP_MIN)/CAP_STEP];
                                if (value_college < value_tmp) {
                                    value_college = value_tmp;
                                    capitalchoice_college = (k_next - CAP_MIN)/CAP_STEP;
                                }
                            }

                            // elite college = 2
                            double value_elitecollege = -9999;
                            double capitalchoice_elitecollege = -1;
                            double k_elitecollege = (1 + R) * (capital(k) - ELITE_TUITION + financial_aid_elite(k_fam, a_em));
                            for(int k_next = CAP_MIN; k_next <= k_elitecollege && k_next < capital(CAP); k_next += CAP_STEP) {
                                double value_tmp = util(k_elitecollege - k_next) + CONSUMPTION_ELITE + schoolfuture2[(k_next - CAP_MIN)/CAP_STEP];
                                if (value_elitecollege < value_tmp) {
                                    value_elitecollege = value_tmp;
                                    capitalchoice_elitecollege = (k_next - CAP_MIN)/CAP_STEP;
                                }
                            }

                            value_non_elite_noshock[iter][a_em][a_ub][a_ib][k][k_fam] = value_college;
                            value_elite_noshock[iter][a_em][a_ub][a_ib][k][k_fam] = value_elitecollege;

                            for(int consump_c = 0; consump_c < SHOCK; consump_c++) {
                                for(int consump_e = 0; consump_e < SHOCK; consump_e++) {
                                    if (value_highschool > value_college + consump_grid_c[consump_c] && value_highschool > value_elitecollege + consump_grid_e[consump_e]){
                                        schoolchoice[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = NO_COLLEGE;
                                        schoolvalue[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = value_highschool;
                                        schoolcapital[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = -1;
                                    }
                                    else if (value_college + consump_grid_c[consump_c] > value_elitecollege + consump_grid_e[consump_e]) {
                                        schoolchoice[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = COLLEGE;
                                        schoolvalue[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = value_college + consump_grid_c[consump_c];
                                        schoolcapital[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = capitalchoice_college;
                                    }
                                    else {
                                        schoolchoice[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = ELITE;
                                        // check if the student is admitted by elite college
							            int sat_g = 0;
							            if (a_em >= 5) {
								            sat_g = 2;
							            }  else if (a_em >= 2) {
							 	            sat_g = 1;
							            }
                                        schoolvalue[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = ELITE_ADMIT[sat_g] * value_elitecollege + (1-ELITE_ADMIT[sat_g]) * value_college + consump_grid_e[consump_e];
                                        schoolcapital[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e] = capitalchoice_elitecollege;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }


        /*fout << "elite college value function and policy function at age 35" << std::endl;
        for(int k = 0; k < 10; k++) {
        fout << "value_non_elite_noshock[" << 0 << "][" << 6 << "][" << 3 << "][" << 3 << "][" << k << "][" << 4 << "] = " << value_non_elite_noshock[0][6][3][3][k][4] << std::endl;
        fout << "value_elite_noshock[" << 0 << "][" << 6 << "][" << 3 << "][" << 3 << "][" << k << "][" << 4 << "] = " << value_elite_noshock[0][6][3][3][k][4] << std::endl;
        fout << "value_worker_noshock[" << 0 << "][" << 0 << "][" << 0 << "][" << 6 << "][" << 3 << "][" << 3 << "][" << 0 << "][" << k << "] = " << value_worker_noshock[0][0][0][6][3][3][0][k] << std::endl;
      }*/

        //for (int k = 0; k < CAP; k++) {
        //    fout << "schoolvalue[ " << k << "] = " << schoolvalue[iter][3][3][k][16][3][3] << std::endl;
        //}
        /*fout << "school choice" << std::endl;
         for (int a_em = 0; a_em < ABI; a_em++)
         for(int a_ub = 0; a_ub < ABI; a_ub++)
         for(int k = 0; k < 30; k++)
         for (int k_fam = 0; k_fam < 30; k_fam++)
         for (int consump_c = 0; consump_c < SHOCK; consump_c++)
         for (int consump_e = 0; consump_e < SHOCK; consump_e++){
         fout << "schoolchoice[" << 0 << "][" << a_em << "][" << a_ub << "][" << k << "]["  << k_fam << "][" << consump_c << "][" << consump_e << "] = " << schoolchoice[0][a_em][a_ub][k][k_fam][consump_c][consump_e] << std::endl;
         }*/


        /*std::cout << "start parental transfer" << std::endl;
        // parent transfer at age 50 (t = 6)
        transfer[iter].resize(EDU);
        for(int e = 0; e < EDU; e++) {
            transfer[iter][e].resize(ABI);
            for(int a_em = 0; a_em < ABI; a_em++) {
                transfer[iter][e][a_em].resize(ABI);
                for(int a_ub = 0; a_ub < ABI; a_ub++) {
                    transfer[iter][e][a_em][a_ub].resize(ABI);
                    for(int a_ib = 0; a_ib < ABI; a_ib++) {
                        transfer[iter][e][a_em][a_ub][a_ib].resize(exp_em(6,e)+1);
                        for(int exp_bus = 0; exp_bus < EXP_BUS_TYPE; exp_bus++) {
                            transfer[iter][e][a_em][a_ub][a_ib][exp_bus].resize(CAP);
                            for(int k = 0; k < CAP; k++) {
                                transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k].resize(SHOCK);
                                for(int consump_n = 0; consump_n < SHOCK; consump_n++) {
                                    transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n].resize(SHOCK);
                                    for(int consump_i = 0; consump_i < SHOCK; consump_i++) {
                                        transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i].resize(SHOCK);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }


        for(int t = 6; t <= 6; t++) {
            std::cout << "Processing age = " << age(t) << std::endl;
#pragma omp parallel for collapse(4)
            for(int e = 0; e < EDU; e++) {
                for(int a_em = 0; a_em < ABI; a_em++) {
                    for(int a_ub = 0; a_ub < ABI; a_ub++) {
                        for(int a_ib = 0; a_ib < ABI; a_ib++) {
                            vector< vector <double> > childvalue(CAP);
                            for (int c_transfer_idx = 0; c_transfer_idx < CAP; c_transfer_idx++){
                                childvalue[c_transfer_idx].resize(CAP);
                                for (int k = 0; k < CAP; k++) {
                                    childvalue[c_transfer_idx][k] = 0;
                                    for(int consump_c_next = 0; consump_c_next < SHOCK; consump_c_next++) {
                                        for(int consump_e_next = 0; consump_e_next < SHOCK; consump_e_next++) {
                                            for (int aem_next = 0; aem_next < ABI; aem_next++) {
                                                for (int aub_next = 0; aub_next < ABI; aub_next++) {
                                                    for (int aib_next = 0; aib_next < ABI; aib_next++) {
                                                        childvalue[c_transfer_idx][k] += OMEGA * schoolvalue[iter][aem_next][aub_next][aib_next][c_transfer_idx][k][consump_c_next][consump_e_next] * prob_consump_c[consump_c_next] * prob_consump_e[consump_e_next] * tranprob_abi_em[a_em][aem_next] * tranprob_abi_ub[a_ub][aub_next] * tranprob_abi_ib[a_ib][aib_next];
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            for(int exp_bus = 0; exp_bus < EXP_BUS_TYPE; exp_bus++) {
                                vector<double> futurevalue0(CAP, 0);
                                vector<double> futurevalue1(CAP, 0);
                                for (int k_next_idx = 0; k_next_idx < CAP; k_next_idx++){
                                    for(int consump_n_next = 0; consump_n_next < SHOCK; consump_n_next++) {
                                        for(int consump_i_next = 0; consump_i_next < SHOCK; consump_i_next++) {
                                            futurevalue0[k_next_idx] += BETA * value[iter][t+1][e][a_em][a_ub][a_ib][0][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                            futurevalue1[k_next_idx] += BETA * value[iter][t+1][e][a_em][a_ub][a_ib][1][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                                        }
                                    }
                                }

                                for(int k = 0; k < CAP; k++) {
                                    // worker's decision
                                    double value_worker = 0;
                                    double value_worker_tmp;
                                    vector<double> capitalchoice_worker(SHOCK);
                                    vector<double> transferchoice_worker(SHOCK);
                                    for(int shock_w = 0; shock_w < SHOCK; shock_w++) {
                                        value_worker_tmp = -9999;
                                        capitalchoice_worker[shock_w] = -1;
                                        transferchoice_worker[shock_w] = -1;
                                        double k_worker = capital(k) * (1 + R) + W * h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e) + shock_grid_w[e][shock_w]);
                                        for(int k_next = 0; k_next <= k_worker && k_next < capital(CAP); k_next += CAP_STEP) {
                                            for (int c_transfer = 0; c_transfer <= k_worker - k_next && c_transfer < capital(CAP); c_transfer += CAP_STEP) {
                                                double value_tmp = util(k_worker - k_next - c_transfer) + childvalue[(c_transfer - CAP_MIN)/CAP_STEP][k] + futurevalue0[(k_next - CAP_MIN)/CAP_STEP];
                                                if (value_worker_tmp < value_tmp) {
                                                    value_worker_tmp = value_tmp;
                                                    capitalchoice_worker[shock_w] = (k_next - CAP_MIN)/CAP_STEP;
                                                    transferchoice_worker[shock_w] = (c_transfer - CAP_MIN)/CAP_STEP;
                                                }
                                            }
                                        }
                                        value_worker += value_worker_tmp * prob_shock_w[e][shock_w];
                                        //if (t == 6 && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 18 ){
                                        //    std::cout << t << ", " << k << ", " << k_worker << ", " << h_w(a_em, e) * exp(ALPHA_0 + ALPHA_1[e] * exp_w(t, e) + ALPHA_2[e] * exp_w(t, e) * exp_w(t, e) + shock_grid_w[e][shock_w]) << ", " << capital(capitalchoice_worker[shock_w]) << ", " << util(k_worker - capital(capitalchoice_worker[shock_w])) << ", " << futurevalue0[capitalchoice_worker[shock_w]] << ", " << value_worker_tmp << ", " << value_worker << std::endl;
                                        //}

                                    }

                                    // non-incorporated business owner decision
                                    double value_non_incorp = 0;
                                    double value_non_incorp_tmp;
                                    vector<double> capitalchoice_non_incorp(SHOCK);
                                    vector<double> transferchoice_non_incorp(SHOCK);
                                    // only able to run a business when there is no debt
                                    if (capital(k) >= 0){
                                        for(int shock_n = 0; shock_n < SHOCK; shock_n++) {
                                            value_non_incorp_tmp = -9999;
                                            capitalchoice_non_incorp[shock_n] = -1;
                                            transferchoice_non_incorp[shock_n] = -1;
                                            double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                                            double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                                            double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                                            // liquidity constraint
                                            if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                                                invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                                            double k_non_incorp = (1 - DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) + (capital(k) - invest_non_incorp) * (1 + R);
                                            for(int k_next = 0; k_next <= k_non_incorp && k_next < capital(CAP); k_next += CAP_STEP) {
                                                for (int c_transfer = 0; c_transfer <= k_non_incorp - k_next && c_transfer < capital(CAP); c_transfer += CAP_STEP) {
                                                    double value_tmp = util(k_non_incorp - k_next - c_transfer) + childvalue[(c_transfer - CAP_MIN)/CAP_STEP][k] + futurevalue0[(k_next - CAP_MIN)/CAP_STEP];
                                                    if (value_non_incorp_tmp < value_tmp) {
                                                        value_non_incorp_tmp = value_tmp;
                                                        capitalchoice_non_incorp[shock_n] = (k_next - CAP_MIN)/CAP_STEP;
                                                        transferchoice_non_incorp[shock_n] = (c_transfer - CAP_MIN)/CAP_STEP;
                                                    }
                                                }
                                            }
                                            value_non_incorp += value_non_incorp_tmp * prob_shock_n[shock_n];
                                            //if (t == 6 && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 18 ){
                                            //    std::cout << t << ", " << k << ", " << i_scaler_non_incorp << ", " << invest_non_incorp << ", " << k_non_incorp << ", " << i_scaler_non_incorp * pow(invest_non_incorp, NU) - N_COST - invest_non_incorp * (1 + R) << ", " << capital(capitalchoice_non_incorp[shock_n]) << ", " << util(k_non_incorp - capital(capitalchoice_non_incorp[shock_n])) << ", " << futurevalue1[capitalchoice_non_incorp[shock_n]] << ", " << value_non_incorp_tmp << ", " << value_non_incorp << std::endl;
                                            //}

                                        }
                                    } else {
                                        value_non_incorp = -999;
                                    }

                                    // incorporated business owner decision
                                    double value_incorp = 0;
                                    double value_incorp_tmp;
                                    vector<double> capitalchoice_incorp(SHOCK);
                                    vector<double> transferchoice_incorp(SHOCK);
                                    // only able to run a business when there is no debt
                                    if (capital(k) >= 0){
                                        for(int shock_i = 0; shock_i < SHOCK; shock_i++) {
                                            value_incorp_tmp = -9999;
                                            capitalchoice_incorp[shock_i] = -1;
                                            transferchoice_incorp[shock_i] = -1;
                                            double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                                            double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                                            double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                                            // liquidity constraint
                                            if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                                                invest_incorp = (1 + LAMBDA_E) * capital(k);
                                            double k_incorp = (1 - DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - I_COST + (capital(k) - invest_incorp) * (1 + R);
                                            if (exp_bus == 1) {
                                                k_incorp = (1 - DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) + (capital(k) - invest_incorp) * (1 + R);
                                            }
                                            for(int k_next = 0; k_next <= k_incorp && k_next < capital(CAP); k_next += CAP_STEP) {
                                                for (int c_transfer = 0; c_transfer < k_incorp - k_next && c_transfer < capital(CAP); c_transfer += CAP_STEP) {
                                                    double value_tmp = util(k_incorp - k_next - c_transfer) + childvalue[(c_transfer - CAP_MIN)/CAP_STEP][k] + futurevalue1[(k_next - CAP_MIN)/CAP_STEP];
                                                    if (value_incorp_tmp < value_tmp) {
                                                        value_incorp_tmp = value_tmp;
                                                        capitalchoice_incorp[shock_i] = (k_next - CAP_MIN)/CAP_STEP;
                                                        transferchoice_incorp[shock_i] = (c_transfer - CAP_MIN)/CAP_STEP;
                                                    }
                                                }
                                            }
                                            value_incorp += value_incorp_tmp * prob_shock_i[shock_i];
                                        }
                                    } else {
                                        value_incorp = -999;
                                    }

                                    value_worker_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus][k] = value_worker;
                                    value_non_incorp_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus][k] = value_non_incorp;
                                    value_incorp_noshock[iter][t][e][a_em][a_ub][a_ib][exp_bus][k] = value_incorp;

                                    for(int consump_n = 0; consump_n < SHOCK; consump_n++) {
                                        for(int consump_i = 0; consump_i < SHOCK; consump_i++) {
                                            //if (t == 6  && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 18 && consump_n == 5 && consump_i == 2){
                                            //    std::cout << t << ", " << k << ", " << exp_bus << ",  " << value_worker << ", " << value_non_incorp + consump_grid_n[consump_n] << ", " << value_incorp + consump_grid_n[consump_n] + consump_grid_i[consump_i] << std::endl;
                                            //}
                                            if (value_worker > value_non_incorp + consump_grid_n[consump_n] && value_worker > value_incorp + consump_grid_i[consump_i]) {
                                                value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_worker;
                                                career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = WORKER;
                                                for (int shock_w = 0; shock_w < SHOCK; shock_w++) {
                                                    capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w] = capitalchoice_worker[shock_w];
                                                    transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w] = transferchoice_worker[shock_w];
                                                }
                                            }
                                            else if (value_non_incorp + consump_grid_n[consump_n] >= value_incorp + consump_grid_i[consump_i]) {
                                                value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_non_incorp + consump_grid_n[consump_n];
                                                career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = NON_INCORP;
                                                for (int shock_n = 0; shock_n < SHOCK; shock_n++) {
                                                    capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n] = capitalchoice_non_incorp[shock_n];
                                                    transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n] = transferchoice_non_incorp[shock_n];
                                                }
                                            }
                                            else {
                                                value[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = value_incorp + consump_grid_i[consump_i];
                                                career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i] = INCORP;
                                                for (int shock_i = 0; shock_i < SHOCK; shock_i++) {
                                                    capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i] = capitalchoice_incorp[shock_i];
                                                    transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i] = transferchoice_incorp[shock_i];
                                                }
                                            }
                                            //if (t == 6 && e == 0 && a_em == 5 && a_ub == 2 && exp_bus == 1 && k == 18 && consump_n == 5 && consump_i == 2){
                                            //    std::cout << t << ", " << k << ", " << career[0][t][e][a_em][a_ub][exp_bus][k][consump_n][consump_i] << std::endl;
                                            //}
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }*/
    }



    /*fout << "transfer" << std::endl;
     for (int e =0; e < EDU; e++)
     for(int k = 0; k < 30; k++){
     fout << "transfer[" << 0 << "][" << e << "][" << (ABI-1)/2 << "][" << (ABI-1)/2 << "][" << 0 << "][" << k << "][" << (SHOCK-1)/2 << "][" << (SHOCK-1)/2 << "] = " << transfer[0][e][(ABI-1)/2][(ABI-1)/2][0][k][(SHOCK-1)/2][(SHOCK-1)/2] << std::endl;
     }*/

    /*for (int e =0; e < EDU; e++)
     for (int a_em = 0; a_em < ABI; a_em++)
     for(int a_ub = 0; a_ub < ABI; a_ub++)
     for(int k = 0; k < 30; k++)
     for (int u = 0; u < SHOCK; u++)
     for (int eps = 0; eps < SHOCK; eps++){
     fout << "transfer[" << 0 << "][" << e << "][" << a_em << "][" << a_ub << "][" << exp_w(6,e)-2 << "][" << k << "][" << u << "][" << eps << "] = " << transfer[0][e][a_em][a_ub][exp_w(6,e)-2][k][u][eps] << std::endl;
     }*/
    // "capitalchoice[" << 0 << "][" << 6 << "][" << e << "][" << a_em << "][" << a_ub << "][" << exp_w(6,e) << "][" << k << "][" << u << "][" << eps << "] = " << capitalchoice[0][6][e][a_em][a_ub][exp_w(6,e)][k][u][eps] <<

    //***************************************** Simulation **********************************************//
    //*********************** Simulate shocks and initial value ***************************//


    std::cout << "start simulation" << std::endl;
    int size = 3; // Dimensionality (rows)

    Eigen::internal::scalar_normal_dist_op<double> randN; // Gaussian functor
    Eigen::internal::scalar_normal_dist_op<double>::rng.seed(1); // Seed the rng

    // Define mean and covariance of the distribution
    Eigen::VectorXd mean(size);
    Eigen::MatrixXd covar(size,size);

    mean  <<  0,  0,  0;
    covar << ABI_EM_STD*ABI_EM_STD, 0, 0,
    0,  ABI_UB_STD*ABI_UB_STD, 0,
    0, 0, ABI_IB_STD*ABI_IB_STD;

    Eigen::MatrixXd normTransform(size,size);

    Eigen::LLT<Eigen::MatrixXd> cholSolver(covar);

    // We can only use the cholesky decomposition if
    // the covariance matrix is symmetric, pos-definite.
    // But a covariance matrix might be pos-semi-definite.
    // In that case, we'll go to an EigenSolver
    if (cholSolver.info()==Eigen::Success) {
        // Use cholesky solver
        normTransform = cholSolver.matrixL();
    } else {
        // Use eigen solver
        Eigen::SelfAdjointEigenSolver<Eigen::MatrixXd> eigenSolver(covar);
        normTransform = eigenSolver.eigenvectors()
        * eigenSolver.eigenvalues().cwiseSqrt().asDiagonal();
    }


    Eigen::MatrixXd samples = (normTransform * Eigen::MatrixXd::NullaryExpr(size,num_id,randN)).colwise() + mean;

    // simulation array
    int abi_em_sim[num_id];
    int abi_ub_sim[num_id];
    int abi_ib_sim[num_id];
    double abi_em_org_sim[num_id];
    double abi_ub_org_sim[num_id];
    double abi_ib_org_sim[num_id];
    int edu_sim[num_id];
    int k_idx_sim[num_id][AGE + 1];
    int k_fam_idx_sim[num_id];
    int career_sim[num_id][AGE];
    double income_sim[num_id][AGE];
    int exp_bus_sim[num_id][AGE];
    int shock_w_sim[num_id][AGE];
    int shock_n_sim[num_id][AGE];
    int shock_i_sim[num_id][AGE];
    int shock_s_sim[num_id][AGE];
    int consump_n_sim[num_id][AGE];
    int consump_i_sim[num_id][AGE];
    int consump_c_sim[num_id];
    int consump_e_sim[num_id];
    // int transfer_sim[num_id];

    // initial value
    int seed = 1987;
    srand(1);
    for (int id = 0; id < num_id; id++){
        // initial ability
        double abi_em_rand = samples(0, id);
        abi_em_org_sim[id] = abi_em_rand;
        double abi_em_min = abi_grid_em[0];
        double abi_em_step = 2 * RANGE_ABI * ABI_EM_STD / (ABI - 1);
        abi_em_sim[id] = round((abi_em_rand - abi_em_min)/abi_em_step);
        if (abi_em_sim[id] > ABI - 1){
            abi_em_sim[id] = ABI - 1;
        } else if (abi_em_sim[id] < 0){
            abi_em_sim[id] = 0;
        }
        double abi_ub_rand = samples(1, id);
        abi_ub_org_sim[id] = abi_ub_rand;
        double abi_ub_min = abi_grid_ub[0];
        double abi_ub_step = 2 * RANGE_ABI * ABI_UB_STD / (ABI - 1);
        abi_ub_sim[id] = round((abi_ub_rand - abi_ub_min)/abi_ub_step);
        if (abi_ub_sim[id] > ABI - 1){
            abi_ub_sim[id] = ABI - 1;
        } else if (abi_ub_sim[id] < 0){
            abi_ub_sim[id] = 0;
        }
        double abi_ib_rand = samples(2, id);
        abi_ib_org_sim[id] = abi_ib_rand;
        double abi_ib_min = abi_grid_ib[0];
        double abi_ib_step = 2 * RANGE_ABI * ABI_IB_STD / (ABI - 1);
        abi_ib_sim[id] = round((abi_ib_rand - abi_ib_min)/abi_ib_step);
        if (abi_ib_sim[id] > ABI - 1){
            abi_ib_sim[id] = ABI - 1;
        } else if (abi_ib_sim[id] < 0){
            abi_ib_sim[id] = 0;
        }
        // initial wealth
        double k_random_number = ((double) rand() / (RAND_MAX)) * 100; //draw a random number between 0 and 100
        double cdf = K_CON_DIST[abi_em_sim[id]][0];
        int group_id = 0;
        while (cdf < k_random_number) {
            group_id ++;
            cdf += K_CON_DIST[abi_em_sim[id]][group_id];
        }
        k_idx_sim[id][0] = group_mapping(group_id);
        // initial family wealth
        double k_fam_rand = r8_normal(0, K_FAM_STD, seed);
        double k_fam;
        if (capital(k_idx_sim[id][0]) > 0) {
            k_fam = exp(log(capital(k_idx_sim[id][0])) * COEF_K + COEF_ABI[abi_em_sim[id]] + k_fam_rand);
        } else {
            k_fam = exp(COEF_ABI[abi_em_sim[id]] + k_fam_rand);
        }
        k_fam_idx_sim[id] = round((k_fam - CAP_MIN)/CAP_STEP);
        //if (id < 500)
        //    std::cout << k_idx_sim[id][0] << ", " << k_fam_rand << ", " << k_fam << ", " << k_fam_idx_sim[id] << std::endl;
        if (k_fam_idx_sim[id] < 0){
            k_fam_idx_sim[id] = 0;
        } else if (k_fam_idx_sim[id] > CAP-1){
            k_fam_idx_sim[id] = CAP - 1;
        }
        // if (id < 10)
        //     fout << abi_em_rand << ", " << abi_ub_rand << ", " << k_random_number << ", " << k_fam_rand << std::endl;
    }

    // Initial ability distribution
    int sum_abi_em[ABI];
    double frac_abi_em[ABI];
    int sum_abi_ub[ABI];
    double frac_abi_ub[ABI];
    int sum_abi_ib[ABI];
    double frac_abi_ib[ABI];

    for (int a = 0; a < ABI; a++){
        sum_abi_em[a] = 0;
        sum_abi_ub[a] = 0;
        sum_abi_ib[a] = 0;
    }
    for (int id = 0; id < num_id; id++) {
        int a_em = abi_em_sim[id];
        int a_ub = abi_ub_sim[id];
        int a_ib = abi_ib_sim[id];
        sum_abi_em[a_em] += 1;
        sum_abi_ub[a_ub] += 1;
        sum_abi_ib[a_ib] += 1;
    }
    for (int a = 0; a < ABI; a++){
        frac_abi_em[a] = (double)sum_abi_em[a] / num_id;
        // std::cout << "frac_abi_em[" << a << "] = " << frac_abi_em[a] << std::endl;
        frac_abi_ub[a] = (double)sum_abi_ub[a] / num_id;
        // std::cout << "frac_abi_ub[" << a << "] = " << frac_abi_ub[a] << std::endl;
        frac_abi_ib[a] = (double)sum_abi_ib[a] / num_id;
        // std::cout << "frac_abi_ib[" << a << "] = " << frac_abi_ib[a] << std::endl;
    }

    // Initial wealth distribution
    int sum_wealth[CAP];
    double frac_wealth[CAP];

    for (int k = 0; k < CAP; k++){
        sum_wealth[k] = 0;
    }
    for (int id = 0; id < num_id; id++) {
        int k = k_idx_sim[id][0];
        sum_wealth[k] += 1;
    }
    for (int k = 0; k < CAP; k++){
        frac_wealth[k] = (double)sum_wealth[k] / num_id;
        // std::cout << "frac_wealth[" << k << "] = " << frac_wealth[k] << std::endl;
    }

    // Initial family wealth distribution
    int sum_wealth_fam[CAP];
    double frac_wealth_fam[CAP];

    for (int k = 0; k < CAP; k++){
        sum_wealth_fam[k] = 0;
    }
    for (int id = 0; id < num_id; id++) {
        int k = k_fam_idx_sim[id];
        sum_wealth_fam[k] += 1;
    }
    for (int k = 0; k < CAP; k++){
        frac_wealth_fam[k] = (double)sum_wealth_fam[k] / num_id;
        // std::cout << "frac_wealth_fam[" << k << "] = " << frac_wealth_fam[k] << std::endl;
    }

    // simulate shocks
    for (int id = 0; id < num_id; id++) {
        for (int t = 0; t < RETIRE_AGE; t++) {
            double shock_w_rand = r8_normal(0, W_H_STD, seed);
            double shock_w_min = shock_grid_w[0][0];
            double shock_w_step = 2 * RANGE_SHOCK * W_H_STD / (SHOCK - 1);
            shock_w_sim[id][t] = round((shock_w_rand - shock_w_min)/shock_w_step);
            if (shock_w_sim[id][t] > SHOCK - 1){
                shock_w_sim[id][t] = SHOCK - 1;
            } else if (shock_w_sim[id][t] < 0){
                shock_w_sim[id][t] = 0;
            }
            double shock_n_rand = r8_normal(0, N_STD, seed);
            double shock_n_min = shock_grid_n[0];
            double shock_n_step = 2 * RANGE_SHOCK * N_STD / (SHOCK - 1);
            shock_n_sim[id][t] = round((shock_n_rand - shock_n_min)/shock_n_step);
            if (shock_n_sim[id][t] > SHOCK - 1){
                shock_n_sim[id][t] = SHOCK - 1;
            } else if (shock_n_sim[id][t] < 0){
                shock_n_sim[id][t] = 0;
            }
            double shock_i_rand = r8_normal(0, I_STD, seed);
            double shock_i_min = shock_grid_i[0];
            double shock_i_step = 2 * RANGE_SHOCK * I_STD / (SHOCK - 1);
            shock_i_sim[id][t] = round((shock_i_rand - shock_i_min)/shock_i_step);
            if (shock_i_sim[id][t] > SHOCK - 1){
                shock_i_sim[id][t] = SHOCK - 1;
            } else if (shock_i_sim[id][t] < 0){
                shock_i_sim[id][t] = 0;
            }
            double consump_n_rand = r8_normal(0, CONSUMP_N_STD, seed);
            double consump_n_min = consump_grid_n[0];
            double consump_n_step = 2 * RANGE_SHOCK * CONSUMP_N_STD / (SHOCK - 1);
            consump_n_sim[id][t] = round((consump_n_rand - consump_n_min)/consump_n_step);
            if (consump_n_sim[id][t] > SHOCK - 1){
                consump_n_sim[id][t] = SHOCK - 1;
            } else if (consump_n_sim[id][t] < 0){
                consump_n_sim[id][t] = 0;
            }
            double consump_i_rand = r8_normal(0, CONSUMP_I_STD, seed);
            double consump_i_min = consump_grid_i[0];
            double consump_i_step = 2 * RANGE_SHOCK * CONSUMP_I_STD / (SHOCK - 1);
            consump_i_sim[id][t] = round((consump_i_rand - consump_i_min)/consump_i_step);
            if (consump_i_sim[id][t] > SHOCK - 1){
                consump_i_sim[id][t] = SHOCK - 1;
            } else if (consump_i_sim[id][t] < 0){
                consump_i_sim[id][t] = 0;
            }
        }
        double consump_c_rand = r8_normal(0, CONSUMP_C_STD, seed);
        double consump_c_min = consump_grid_c[0];
        double consump_c_step = 2 * RANGE_SHOCK * CONSUMP_C_STD / (SHOCK - 1);
        consump_c_sim[id] = round((consump_c_rand - consump_c_min)/consump_c_step);
        if (consump_c_sim[id] > SHOCK - 1){
            consump_c_sim[id] = SHOCK - 1;
        } else if (consump_c_sim[id] < 0){
            consump_c_sim[id] = 0;
        }
        double consump_e_rand = r8_normal(0, CONSUMP_E_STD, seed);
        double consump_e_min = consump_grid_e[0];
        double consump_e_step = 2 * RANGE_SHOCK * CONSUMP_E_STD / (SHOCK - 1);
        consump_e_sim[id] = round((consump_e_rand - consump_e_min)/consump_e_step);
        if (consump_e_sim[id] > SHOCK - 1){
            consump_e_sim[id] = SHOCK - 1;
        } else if (consump_e_sim[id] < 0){
            consump_e_sim[id] = 0;
        }

    }

    for (int id = 0; id < num_id; id++) {
        for (int t = RETIRE_AGE; t < AGE; t++) {
            double shock_n_rand = r8_normal(0, N_STD, seed);
            double shock_n_min = shock_grid_n[0];
            double shock_n_step = 2 * RANGE_SHOCK * N_STD / (SHOCK - 1);
            shock_n_sim[id][t] = round((shock_n_rand - shock_n_min)/shock_n_step);
            if (shock_n_sim[id][t] > SHOCK - 1){
                shock_n_sim[id][t] = SHOCK - 1;
            } else if (shock_n_sim[id][t] < 0){
                shock_n_sim[id][t] = 0;
            }
            double shock_i_rand = r8_normal(0, I_STD, seed);
            double shock_i_min = shock_grid_i[0];
            double shock_i_step = 2 * RANGE_SHOCK * I_STD / (SHOCK - 1);
            shock_i_sim[id][t] = round((shock_i_rand - shock_i_min)/shock_i_step);
            if (shock_i_sim[id][t] > SHOCK - 1){
                shock_i_sim[id][t] = SHOCK - 1;
            } else if (shock_i_sim[id][t] < 0){
                shock_i_sim[id][t] = 0;
            }
            double consump_n_rand = r8_normal(0, CONSUMP_N_STD, seed);
            double consump_n_min = consump_grid_n[0];
            double consump_n_step = 2 * RANGE_SHOCK * CONSUMP_N_STD / (SHOCK - 1);
            consump_n_sim[id][t] = round((consump_n_rand - consump_n_min)/consump_n_step);
            if (consump_n_sim[id][t] > SHOCK - 1){
                consump_n_sim[id][t] = SHOCK - 1;
            } else if (consump_n_sim[id][t] < 0){
                consump_n_sim[id][t] = 0;
            }
            double consump_i_rand = r8_normal(0, CONSUMP_I_STD, seed);
            double consump_i_min = consump_grid_i[0];
            double consump_i_step = 2 * RANGE_SHOCK * CONSUMP_I_STD / (SHOCK - 1);
            consump_i_sim[id][t] = round((consump_i_rand - consump_i_min)/consump_i_step);
            if (consump_i_sim[id][t] > SHOCK - 1){
                consump_i_sim[id][t] = SHOCK - 1;
            } else if (consump_i_sim[id][t] < 0){
                consump_i_sim[id][t] = 0;
            }
            shock_s_sim[id][t] = ((double) rand() / RAND_MAX);
        }
    }

    double laborw_sum = 0;
    double capital_sum = 0;
    double labor_sum = 0;
    double capitalr_sum = 0;
    // simulate first period: education decision
    for (int id = 0; id < num_id; id++) {
        int iter = 0;
        int t = 0;
        int a_em = abi_em_sim[id];
        int a_ub = abi_ub_sim[id];
        int a_ib = abi_ib_sim[id];
        int k = k_idx_sim[id][0];
        int k_fam = k_fam_idx_sim[id];
        int consump_c = consump_c_sim[id];
        int consump_e = consump_e_sim[id];
        double edu_random_number = ((double) rand() / (RAND_MAX)); 

        edu_sim[id] = schoolchoice[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e];
        // check if the student is admitted by elite college
	    int flag = 0;
	    if (edu_sim[id] == 2){
		    int sat_group = 0;
		    if (a_em >= 5) {
			    sat_group = 2;
		    } else if (a_em >= 2) {
			    sat_group = 1;
		    }
		    if (edu_random_number > ELITE_ADMIT[sat_group]) {
			    edu_sim[id] = 1;
			    flag = 1;
		    }   
	    }
        //if (edu_sim[id] == 2){
        //    std::cout << id << ", " << abi_em_sim[id] << ", " << abi_ub_sim[id] << ", " << abi_ib_sim[id] << ", " << k_idx_sim[id][0] << ", " << k_fam_idx_sim[id] << std::endl;
        //}
        // high school - work
        if (edu_sim[id] == 0){
            int e = edu_sim[id];
            int exp_bus = 0;
            int shock_w = shock_w_sim[id][t];
            int shock_n = shock_n_sim[id][t];
            int shock_i = shock_i_sim[id][t];
            int consump_n = consump_n_sim[id][t];
            int consump_i = consump_i_sim[id][t];

            career_sim[id][t] = career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i];
            if (career_sim[id][t] == 0){
                // worker
                income_sim[id][t] = W * h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e) + shock_grid_w[e][shock_w]);
                exp_bus_sim[id][t] = 0;
                k_idx_sim[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w];
                labor_sum += h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e) + shock_grid_w[e][shock_w]);
                laborw_sum += income_sim[id][t];
                capital_sum += k_idx_sim[id][t];
                capitalr_sum += R * k_idx_sim[id][t];
            } else if (career_sim[id][0] == 1) {
                double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                // liquidity constraint
                if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                    invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                income_sim[id][0] = (1 - DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) - invest_non_incorp * (1 + R);
                exp_bus_sim[id][0] = 0;
                k_idx_sim[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n];
                capital_sum += (invest_non_incorp - k_idx_sim[id][t]);
                capitalr_sum += R * (invest_non_incorp - k_idx_sim[id][t]);
            } else {
                double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                // liquidity constraint
                if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                    invest_incorp = (1 + LAMBDA_E) * capital(k);
                income_sim[id][0] = (1- DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - I_COST - invest_incorp * (1 + R);
                exp_bus_sim[id][1] = 1;
                k_idx_sim[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i];
                capital_sum += (invest_incorp - k_idx_sim[id][t]);
                capitalr_sum += R * (invest_incorp - k_idx_sim[id][t]);
            }
        } else {
            // college
            career_sim[id][0] = -1;
            income_sim[id][0] = 0;
            exp_bus_sim[id][0] = 0;
            k_idx_sim[id][t+1] = schoolcapital[iter][a_em][a_ub][a_ib][k][k_fam][consump_c][consump_e];
            if (flag == 1) {
			    k_idx_sim[id][t+1] = k_idx_sim[id][t+1] + 8;
		    }
            capital_sum += k_idx_sim[id][t];
            capitalr_sum += R * k_idx_sim[id][t];
        }
        //if (id < 50)
        //    std::cout << "ability = " << abi_w_sim[id] << ", " << abi_e_sim[id] << ", shock = " << consump_c_sim[id] << ", " << consump_e_sim[id] << ", edu = " << edu_sim[id] << ", k = " << k_idx_sim[id][0] << ", " << k_idx_sim[id][1] << ", career/income/exp_bus = " << career_sim[id][0] << ", " << income_sim[id][0] << ", " << exp_bus_sim[id][0] << std::endl;
    }

    // simulate period 2 - RETIRE_AGE
    for (int id = 0; id < num_id; id++) {
        for (int t = 1; t < RETIRE_AGE; t++) {
            int iter = 0;
            int a_em = abi_em_sim[id];
            int a_ub = abi_ub_sim[id];
            int a_ib = abi_ib_sim[id];
            int e = edu_sim[id];
            int exp_bus = exp_bus_sim[id][t-1];
            int k = k_idx_sim[id][t];
            int consump_n = consump_n_sim[id][t];
            int consump_i = consump_i_sim[id][t];
            int shock_w = shock_w_sim[id][t];
            int shock_n = shock_n_sim[id][t];
            int shock_i = shock_i_sim[id][t];


            career_sim[id][t] = career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i];
            if (career_sim[id][t] == 0){
                income_sim[id][t] = W * h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e) + shock_grid_w[e][shock_w]);
                exp_bus_sim[id][t] = 0;
                k_idx_sim[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w];
                //if (t == 6){
                //    transfer_sim[id] = transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w];
                //}
                labor_sum += h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e) + shock_grid_w[e][shock_w]);
                laborw_sum += income_sim[id][t];
                capital_sum += k_idx_sim[id][t];
                capitalr_sum += R * k_idx_sim[id][t];
            } else if (career_sim[id][t] == 1) {
                double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                // liquidity constraint
                if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                    invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                income_sim[id][t] = (1-DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) - invest_non_incorp * (1 + R);
                exp_bus_sim[id][t] = 0;
                if (t == RETIRE_AGE - 1) {
                    exp_bus_sim[id][t] = 1;
                }
                k_idx_sim[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n];
                //if (t == 6){
                //    transfer_sim[id] = transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n];
                //}
                capital_sum += (invest_non_incorp - k_idx_sim[id][t]);
                capitalr_sum += R * (invest_non_incorp - k_idx_sim[id][t]);
            } else {
                double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                // liquidity constraint
                if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                    invest_incorp = (1 + LAMBDA_E) * capital(k);
                income_sim[id][t] = (1-DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - I_COST - invest_incorp * (1 + R);
                if (exp_bus == 1) {
                    income_sim[id][t] = (1-DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - invest_incorp * (1 + R);
                }
                exp_bus_sim[id][t] = 1;
                if (t == RETIRE_AGE - 1) {
                    exp_bus_sim[id][t] = 2;
                }
                k_idx_sim[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i];
                //if (t == 6){
                //    transfer_sim[id] = transfer[iter][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i];
                //}
                capital_sum += (invest_incorp - k_idx_sim[id][t]);
                capitalr_sum += R * (invest_incorp - k_idx_sim[id][t]);
            }

            //if (id < 50){
            //    std::cout << id << ", " << t << ", ability =" << abi_w_sim[id] << ", " << abi_e_sim[id] << ", shock = " << consump_n_sim[id] << ", " << shock_w_sim[id][t] << ", " << shock_n_sim[id][t] << ", " << shock_i_sim[id][t] << ", edu = " << edu_sim[id] << ", k = " << k_idx_sim[id][t] << ", " << k_idx_sim[id][t+1] << ", career/income/exp_bus = " << career_sim[id][t] << ", " << income_sim[id][t] << ", " << exp_bus_sim[id][t] << std::endl;
            //}
        }
    }



    //
    // College premium
    fout << "College premium" << std::endl;
    double ec_count = 0;
    double total_income = 0;
    double total_income_nonelite = 0;
    double total_ib = 0;
    double total_ib_nonelite = 0;
    double total_ub = 0;
    double total_ub_nonelite = 0;


    double total_income_ind[num_id];
    double total_income_ind_nonelite[num_id];
    double total_ib_ind[num_id];
    double total_ib_ind_nonelite[num_id];
    double total_ub_ind[num_id];
    double total_ub_ind_nonelite[num_id];
    int k_idx_sim_nonelite[num_id][AGE + 1];
    int career_sim_nonelite[num_id][AGE];
    double income_sim_nonelite[num_id][AGE];
    int exp_bus_sim_nonelite[num_id][AGE];
    double total_income_diff = 0;

      for (int id = 0; id < num_id; id++) {
          if (edu_sim[id] == 2) {
              ec_count += 1;
              int iter = 0;
              int a_em = abi_em_sim[id];
              int a_ub = abi_ub_sim[id];
              int a_ib = abi_ib_sim[id];
              int k = k_idx_sim[id][0];
              int k_fam = k_fam_idx_sim[id];

              total_income_ind[id] = - ELITE_TUITION + financial_aid_elite(k_fam, a_em);
              total_income_ind_nonelite[id] = - COLLEGE_TUITION + financial_aid_nonelite(k_fam, a_em);
              total_ib_ind[id] = 0;
              total_ub_ind[id] = 0;
              total_ib_ind_nonelite[id] = 0;
              total_ub_ind_nonelite[id] = 0;

              vector<double> schoolfuture1(CAP, 0);
              for (int k_next_idx = 0; k_next_idx < CAP; k_next_idx++){
                  for(int consump_n_next = 0; consump_n_next < SHOCK; consump_n_next++) {
                      for(int consump_i_next = 0; consump_i_next < SHOCK; consump_i_next++) {
                          schoolfuture1[k_next_idx] += BETA * value[iter][1][1][a_em][a_ub][a_ib][0][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                      }
                  }
              }

              double value_college = -9999;
              double capitalchoice_college = -1;
              double k_college = (1 + R) * (capital(k) - COLLEGE_TUITION + financial_aid_nonelite(k_fam, a_em));
              for(int k_next = CAP_MIN; k_next <= k_college && k_next < capital(CAP); k_next += CAP_STEP) {
                  double value_tmp = util(k_college - k_next) + CONSUMPTION_COLLEGE + schoolfuture1[(k_next - CAP_MIN)/CAP_STEP];
                  if (value_college < value_tmp) {
                      value_college = value_tmp;
                      capitalchoice_college = (k_next - CAP_MIN)/CAP_STEP;
                  }
              }

              income_sim_nonelite[id][0] = 0;
              exp_bus_sim_nonelite[id][0] = 0;
              k_idx_sim_nonelite[id][1] = capitalchoice_college;

              //fout_c2 << id << ", " << 0 << ", " << total_income_elite[id] << ", " << total_income_nonelite[id] << std::endl;

              for (int t = 1; t < RETIRE_AGE - 1; t++) {

                  int e = 1;
                  int exp_bus = exp_bus_sim_nonelite[id][t-1];
                  int k = k_idx_sim_nonelite[id][t];
                  int consump_n = consump_n_sim[id][t];
                  int consump_i = consump_i_sim[id][t];
                  int shock_w = shock_w_sim[id][t];
                  int shock_n = shock_n_sim[id][t];
                  int shock_i = shock_i_sim[id][t];

                  career_sim_nonelite[id][t] = career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i];

                  if (career_sim_nonelite[id][t] == 0){
                      income_sim_nonelite[id][t] = W * h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e) + shock_grid_w[e][shock_w]);
                      exp_bus_sim_nonelite[id][t] = 0;
                      k_idx_sim_nonelite[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w];
                  } else if (career_sim_nonelite[id][t] == 1) {
                      double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                      double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                      double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                      // liquidity constraint
                      if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                          invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                      income_sim_nonelite[id][t] = (1-DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) - invest_non_incorp * (1 + R);
                      exp_bus_sim_nonelite[id][t] = 0;
                      if (t == RETIRE_AGE - 1) {
                          exp_bus_sim_nonelite[id][t] = 1;
                      }
                      k_idx_sim_nonelite[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n];
                  } else {
                      double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                      double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                      double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                      // liquidity constraint
                      if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                          invest_incorp = (1 + LAMBDA_E) * capital(k);
                      income_sim_nonelite[id][t] = (1-DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - I_COST - invest_incorp * (1 + R);
                      if (exp_bus == 1) {
                          income_sim_nonelite[id][t] = (1-DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - invest_incorp * (1 + R);
                      }
                      exp_bus_sim_nonelite[id][t] = 1;
                      if (t == RETIRE_AGE - 1) {
                          exp_bus_sim_nonelite[id][t] = 2;
                      }
                      k_idx_sim_nonelite[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i];
                  }

                  total_income_ind[id] += pow(BETA, t) * income_sim[id][t];
                  total_income_ind_nonelite[id] += pow(BETA, t) * income_sim_nonelite[id][t];
                  if (career_sim[id][t] == 1) {
                    total_ub_ind[id] += 1;
                  } else if (career_sim[id][t] == 2) {
                    total_ib_ind[id] += 1;
                  }
                  if (career_sim_nonelite[id][t] == 1) {
                    total_ub_ind_nonelite[id] += 1;
                  } else if (career_sim_nonelite[id][t] == 2) {
                    total_ib_ind_nonelite[id] += 1;
                  }

                  //fout_c2 << id << ", " << t << ", " << income_sim[id][t] << ", " << income_sim_nonelite[id][t] << std::endl;

            }
            total_income += total_income_ind[id];
            total_income_nonelite += total_income_ind_nonelite[id];
            total_ib += total_ib_ind[id]/(RETIRE_AGE - 2);
            total_ub += total_ub_ind[id]/(RETIRE_AGE - 2);
            total_ib_nonelite += total_ib_ind_nonelite[id]/(RETIRE_AGE - 2);
            total_ub_nonelite += total_ub_ind_nonelite[id]/(RETIRE_AGE - 2);

        }
    }
    fout_c2 << "lifetime income (elite college graduates) =" << total_income /ec_count << std::endl;
    fout_c2 << "lifetime income (if non-elite college graduates) =" << total_income_nonelite /ec_count << std::endl;
    fout_c2 << "IB (elite college graduates) =" << total_ib/ec_count << std::endl;
    fout_c2 << "IB (if non-elite college graduates) =" << total_ib_nonelite/ec_count << std::endl;
    fout_c2 << "UB (elite college graduates) =" << total_ub/ec_count << std::endl;
    fout_c2 << "UB (if non-elite college graduates) =" << total_ub_nonelite/ec_count << std::endl;

    fout_c2 << "changes in lifetime income (elite college graduates) =" << (total_income_nonelite - total_income)/total_income << std::endl;
    fout_c2 << "changes in IB (elite college graduates) =" << (total_ib_nonelite - total_ib)/ec_count << std::endl;
    fout_c2 << "changes in UB (elite college graduates) =" << (total_ub_nonelite - total_ub)/ec_count << std::endl;


    // College premium (fixed ability and wealth)
    fout_c2 << "College premium for fixed ability and wealth" << std::endl;

    int num_sim = 500;

    double total_income_fix_elite;
    double total_income_fix_nonelite;
    double total_ib_fix_elite;
    double total_ib_fix_nonelite;
    double total_ub_fix_elite;
    double total_ub_fix_nonelite;

    double income_fix_elite[num_id];
    double income_fix_nonelite[num_id];
    double ib_fix_elite[num_id];
    double ib_fix_nonelite[num_id];
    double ub_fix_elite[num_id];
    double ub_fix_nonelite[num_id];

    int k_idx_sim_fix_elite[num_sim][AGE + 1];
    int career_sim_fix_elite[num_sim][AGE];
    double income_sim_fix_elite[num_sim][AGE];
    int exp_bus_sim_fix_elite[num_sim][AGE];
    int k_idx_sim_fix_nonelite[num_sim][AGE + 1];
    int career_sim_fix_nonelite[num_sim][AGE];
    double income_sim_fix_nonelite[num_sim][AGE];
    int exp_bus_sim_fix_nonelite[num_sim][AGE];

    int initial_value[16][4] = {
      {1, 1, 1, 7}, {1, 1, 1, 9}, {5, 1, 1, 7}, {5, 1, 1, 9},
      {1, 5, 1, 7}, {1, 5, 1, 9}, {1, 1, 5, 7}, {1, 1, 5, 9},
      {5, 5, 1, 7}, {5, 5, 1, 9}, {1, 5, 5, 7}, {1, 5, 5, 9},
      {5, 1, 5, 7}, {5, 1, 5, 9}, {5, 5, 5, 7}, {5, 5, 5, 9}
    };

    for (int iter_initial = 0; iter_initial < 16; iter_initial++) {

      total_income_fix_elite = 0;
      total_income_fix_nonelite = 0;
      total_ib_fix_elite = 0;
      total_ib_fix_nonelite = 0;
      total_ub_fix_elite = 0;
      total_ub_fix_nonelite = 0;

      for (int id = 0; id < num_sim; id++) {
          int iter = 0;
          int a_em = initial_value[iter_initial][0];
          int a_ub = initial_value[iter_initial][1];
          int a_ib = initial_value[iter_initial][2];
          int k = initial_value[iter_initial][3];
          int k_fam = 15;



          income_fix_elite[id] = - ELITE_TUITION + financial_aid_elite(k_fam, a_em);
          income_fix_nonelite[id] = - COLLEGE_TUITION + financial_aid_nonelite(k_fam, a_em);
          ib_fix_elite[id] = 0;
          ub_fix_elite[id] = 0;
          ib_fix_nonelite[id] = 0;
          ub_fix_nonelite[id] = 0;

          vector<double> schoolfuture1(CAP, 0);
              for (int k_next_idx = 0; k_next_idx < CAP; k_next_idx++){
                  for(int consump_n_next = 0; consump_n_next < SHOCK; consump_n_next++) {
                      for(int consump_i_next = 0; consump_i_next < SHOCK; consump_i_next++) {
                          schoolfuture1[k_next_idx] += BETA * value[iter][1][1][a_em][a_ub][a_ib][0][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                      }
                  }
              }

          double value_college = -9999;
              double capitalchoice_college = -1;
              double k_college = (1 + R) * (capital(k) - COLLEGE_TUITION + financial_aid_nonelite(k_fam, a_em));
              for(int k_next = CAP_MIN; k_next <= k_college && k_next < capital(CAP); k_next += CAP_STEP) {
                  double value_tmp = util(k_college - k_next) + CONSUMPTION_COLLEGE + schoolfuture1[(k_next - CAP_MIN)/CAP_STEP];
                  if (value_college < value_tmp) {
                      value_college = value_tmp;
                      capitalchoice_college = (k_next - CAP_MIN)/CAP_STEP;
                  }
              }

              if (capitalchoice_college == -1) {
                  capitalchoice_college = 0;
              }

          income_sim_fix_nonelite[id][0] = 0;
          exp_bus_sim_fix_nonelite[id][0] = 0;
          k_idx_sim_fix_nonelite[id][1] = capitalchoice_college;

          vector<double> schoolfuture2(CAP, 0);
              for (int k_next_idx = 0; k_next_idx < CAP; k_next_idx++){
                  for(int consump_n_next = 0; consump_n_next < SHOCK; consump_n_next++) {
                      for(int consump_i_next = 0; consump_i_next < SHOCK; consump_i_next++) {
                          schoolfuture2[k_next_idx] += BETA * value[iter][1][2][a_em][a_ub][a_ib][0][k_next_idx][consump_n_next][consump_i_next] * prob_consump_n[consump_n_next] * prob_consump_i[consump_i_next];
                      }
                  }
              }

          double value_elitecollege = -9999;
              double capitalchoice_elitecollege = -1;
              double k_elitecollege = (1 + R) * (capital(k) - ELITE_TUITION + financial_aid_elite(k_fam, a_em));
              for(int k_next = CAP_MIN; k_next <= k_elitecollege && k_next < capital(CAP); k_next += CAP_STEP) {
                  double value_tmp = util(k_elitecollege - k_next) + CONSUMPTION_ELITE + schoolfuture2[(k_next - CAP_MIN)/CAP_STEP];
                  if (value_elitecollege < value_tmp) {
                      value_elitecollege = value_tmp;
                      capitalchoice_elitecollege = (k_next - CAP_MIN)/CAP_STEP;
                  }
              }
              if (capitalchoice_elitecollege == -1) {
                  capitalchoice_elitecollege = 0;
              }

          income_sim_fix_elite[id][0] = 0;
          exp_bus_sim_fix_elite[id][0] = 0;
          k_idx_sim_fix_elite[id][1] = capitalchoice_elitecollege;


          //fout_c2 << id << ", " << 0 << ", " << total_income_elite[id] << ", " << total_income_nonelite[id] << std::endl;

          for (int t = 1; t < RETIRE_AGE - 1; t++) {

              int e = 1;
              int exp_bus = exp_bus_sim_fix_nonelite[id][t-1];
              int k = k_idx_sim_fix_nonelite[id][t];
              int consump_n = consump_n_sim[id][t];
              int consump_i = consump_i_sim[id][t];
              int shock_w = shock_w_sim[id][t];
              int shock_n = shock_n_sim[id][t];
              int shock_i = shock_i_sim[id][t];

              if (k > 6) {
                  career_sim_fix_nonelite[id][t] = career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i];
              } else {
                  career_sim_fix_nonelite[id][t] = 0;
              }
                  if (career_sim_fix_nonelite[id][t] == 0){
                      income_sim_fix_nonelite[id][t] = W * h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e) + shock_grid_w[e][shock_w]);
                      exp_bus_sim_fix_nonelite[id][t] = 0;
                      k_idx_sim_fix_nonelite[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w];
                  } else if (career_sim_fix_nonelite[id][t] == 1) {
                      double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                      double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                      double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                      // liquidity constraint
                      if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                          invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                      income_sim_fix_nonelite[id][t] = (1-DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) - invest_non_incorp * (1 + R);
                      exp_bus_sim_fix_nonelite[id][t] = 0;
                      if (t == RETIRE_AGE - 1) {
                          exp_bus_sim_fix_nonelite[id][t] = 1;
                      }
                      k_idx_sim_fix_nonelite[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n];
                  } else {
                      double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                      double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                      double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                      // liquidity constraint
                      if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                          invest_incorp = (1 + LAMBDA_E) * capital(k);
                      income_sim_fix_nonelite[id][t] = (1-DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - I_COST - invest_incorp * (1 + R);
                      if (exp_bus == 1) {
                          income_sim_fix_nonelite[id][t] = (1-DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - invest_incorp * (1 + R);
                      }
                      exp_bus_sim_fix_nonelite[id][t] = 1;
                      if (t == RETIRE_AGE - 1) {
                          exp_bus_sim_fix_nonelite[id][t] = 2;
                      }
                      k_idx_sim_fix_nonelite[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i];
                  }

                  if (k_idx_sim_fix_nonelite[id][t+1] < 0){
                    k_idx_sim_fix_nonelite[id][t+1] = 0;
                  }

                  income_fix_nonelite[id] += pow(BETA, t) * income_sim_fix_nonelite[id][t];
                  if (career_sim_fix_nonelite[id][t] == 1) {
                    ub_fix_nonelite[id] += 1;
                  } else if (career_sim_fix_nonelite[id][t] == 2) {
                    ib_fix_nonelite[id] += 1;
                  }
          }

          for (int t = 1; t < RETIRE_AGE - 1; t++) {

              int e = 2;
              int exp_bus = exp_bus_sim_fix_elite[id][t-1];
              int k = k_idx_sim_fix_elite[id][t];
              int consump_n = consump_n_sim[id][t];
              int consump_i = consump_i_sim[id][t];
              int shock_w = shock_w_sim[id][t];
              int shock_n = shock_n_sim[id][t];
              int shock_i = shock_i_sim[id][t];

              if (k > 6) {
                  career_sim_fix_elite[id][t] = career[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i];
              } else {
                  career_sim_fix_elite[id][t] = 0;
              }
              if (career_sim_fix_elite[id][t] == 0){
                  income_sim_fix_elite[id][t] = W * h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e) + shock_grid_w[e][shock_w]);
                  exp_bus_sim_fix_elite[id][t] = 0;
                  k_idx_sim_fix_elite[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_w];
              } else if (career_sim_fix_elite[id][t] == 1) {
                  double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                  double i_scaler_non_incorp = PROD_N * h_ub(a_ub, e) * exp(shock_grid_n[shock_n]) * pow(h_em, RHO_UB);
                  double invest_non_incorp = pow((DELTA+R)/(i_scaler_non_incorp * NU_UB), 1/(NU_UB - 1));
                  // liquidity constraint
                  if (invest_non_incorp > (1 + LAMBDA_E) * capital(k))
                      invest_non_incorp = (1 + LAMBDA_E) * capital(k);
                  income_sim_fix_elite[id][t] = (1-DELTA) * invest_non_incorp + i_scaler_non_incorp * pow(invest_non_incorp, NU_UB) - invest_non_incorp * (1 + R);
                  exp_bus_sim_fix_elite[id][t] = 0;
                  if (t == RETIRE_AGE - 1) {
                      exp_bus_sim_fix_elite[id][t] = 1;
                  }
                  k_idx_sim_fix_elite[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_n];
              } else {
                  double h_em = h_em(a_em, e) * exp(ALPHA_1[e] * exp_em(t, e) + ALPHA_2[e] * exp_em(t, e) * exp_em(t, e));
                  double i_scaler_incorp = PROD_I * h_ib(a_ib, e) * exp(shock_grid_i[shock_i]) * pow(h_em, RHO_IB);
                  double invest_incorp = pow((DELTA+R)/(i_scaler_incorp * NU_IB), 1/(NU_IB - 1));
                  // liquidity constraint
                  if (invest_incorp > (1 + LAMBDA_E) * capital(k))
                      invest_incorp = (1 + LAMBDA_E) * capital(k);
                  income_sim_fix_elite[id][t] = (1-DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - I_COST - invest_incorp * (1 + R);
                  if (exp_bus == 1) {
                      income_sim_fix_elite[id][t] = (1-DELTA) * invest_incorp + i_scaler_incorp * pow(invest_incorp, NU_IB) - invest_incorp * (1 + R);
                  }
                  exp_bus_sim_fix_elite[id][t] = 1;
                  if (t == RETIRE_AGE - 1) {
                      exp_bus_sim_fix_elite[id][t] = 2;
                  }
                  k_idx_sim_fix_elite[id][t+1] = capitalchoice[iter][t][e][a_em][a_ub][a_ib][exp_bus][k][consump_n][consump_i][shock_i];
              }
              if (k_idx_sim_fix_elite[id][t+1] < 0){
                k_idx_sim_fix_elite[id][t+1] = 0;
              }

              income_fix_elite[id] += pow(BETA, t) * income_sim_fix_elite[id][t];
              if (career_sim_fix_elite[id][t] == 1) {
                ub_fix_elite[id] += 1;
              } else if (career_sim_fix_elite[id][t] == 2) {
                ib_fix_elite[id] += 1;
              }

            }

            total_income_fix_elite += income_fix_elite[id];
            total_income_fix_nonelite += income_fix_nonelite[id];
            total_ib_fix_elite += ib_fix_elite[id]/(RETIRE_AGE - 2);
            total_ib_fix_nonelite += ib_fix_nonelite[id]/(RETIRE_AGE - 2);
            total_ub_fix_elite += ub_fix_elite[id]/(RETIRE_AGE - 2);
            total_ub_fix_nonelite += ub_fix_nonelite[id]/(RETIRE_AGE - 2);

          }
          fout_c2 << initial_value[iter_initial][0] << ", " << initial_value[iter_initial][1] << ", " << initial_value[iter_initial][2] << ", " << initial_value[iter_initial][3] << std::endl;

          fout_c2 << "lifetime income (elite college graduates), " << total_income_fix_elite /num_sim << std::endl;
          fout_c2 << "lifetime income (if non-elite college graduates), " << total_income_fix_nonelite /num_sim << std::endl;
          fout_c2 << "IB (elite college graduates), " << total_ib_fix_elite/num_sim << std::endl;
          fout_c2 << "IB (if non-elite college graduates), " << total_ib_fix_nonelite/num_sim << std::endl;
          fout_c2 << "UB (elite college graduates), " << total_ub_fix_elite/num_sim << std::endl;
          fout_c2 << "UB (if non-elite college graduates), " << total_ub_fix_nonelite/num_sim << std::endl;
    }




    fout.close();
    fout_est.close();
    //fout_c1.close();
    fout_c2.close();

    std::cout << "finish moment calculation" << std::endl;
    return 0;
}

int main(int argc, const char * argv[]) {
    backward_induction(ABI_EM_STD, ABI_UB_STD, ABI_IB_STD, W_H_STD, W_C_STD, W_E_STD, N_STD, I_STD, LAMBDA_E, LAMBDA_C, OMEGA, THETA_EM, THETA_UB, THETA_IB);

    return 0;
}
